Package: samr
Title: SAM: Significance Analysis of Microarrays
Version: 3.0.1
Description: Significance Analysis of Microarrays for differential expression analysis, RNAseq data and related problems.
Imports: impute, matrixStats, shiny, shinyFiles, openxlsx, GSA
License: LGPL
Packaged: 2026-04-10 06:51:13 UTC; hornik
Repository: CRAN
Date/Publication: 2026-04-10 07:11:47 UTC
NeedsCompilation: yes
Authors@R: c(person(given = "Rob",
 	            family = "Tibshirani",
                    role = c("aut", "cre"),
		    email = "tibs@stanford.edu"),
             person(given = c("Michael", "J."),
                    family = "Seo",
                    role = "aut"),
             person(given = "G.",
                    family = "Chu",
                    role = "aut"),
             person(given = "Balasubramanian",
                    family = "Narasimhan",
                    role = "aut"),
             person(given = "Jun",
                    family = "Li",
                    role = "aut"))
Author: Rob Tibshirani [aut, cre],
  Michael J. Seo [aut],
  G. Chu [aut],
  Balasubramanian Narasimhan [aut],
  Jun Li [aut]
Maintainer: Rob Tibshirani <tibs@stanford.edu>
Built: R 4.5.2; aarch64-apple-darwin20; 2026-04-15 05:10:00 UTC; unix
Archs: samr.so.dSYM

Package: samr
Title: SAM: Significance Analysis of Microarrays
Version: 3.0.1
Description: Significance Analysis of Microarrays for differential expression analysis, RNAseq data and related problems.
Imports: impute, matrixStats, shiny, shinyFiles, openxlsx, GSA
License: LGPL
Packaged: 2026-04-10 06:51:13 UTC; hornik
Repository: CRAN
Date/Publication: 2026-04-10 07:11:47 UTC
NeedsCompilation: yes
Authors@R: c(person(given = "Rob",
 	            family = "Tibshirani",
                    role = c("aut", "cre"),
		    email = "tibs@stanford.edu"),
             person(given = c("Michael", "J."),
                    family = "Seo",
                    role = "aut"),
             person(given = "G.",
                    family = "Chu",
                    role = "aut"),
             person(given = "Balasubramanian",
                    family = "Narasimhan",
                    role = "aut"),
             person(given = "Jun",
                    family = "Li",
                    role = "aut"))
Author: Rob Tibshirani [aut, cre],
  Michael J. Seo [aut],
  G. Chu [aut],
  Balasubramanian Narasimhan [aut],
  Jun Li [aut]
Maintainer: Rob Tibshirani <tibs@stanford.edu>
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-28 21:05:04 UTC; unix
Archs: samr.so.dSYM

import(shiny)
importFrom(shinyFiles, shinyFilesButton, shinyFileChoose, getVolumes)
importFrom(openxlsx, read.xlsx, writeData, saveWorkbook, insertImage, addStyle, addWorksheet,
           createWorkbook, setColWidths)
import(matrixStats)
importFrom(GSA, GSA, GSA.correlate, GSA.func, GSA.genescores, GSA.listsets, GSA.make.features,
           GSA.plot, GSA.read.gmt)
importFrom(impute, impute.knn)
importFrom("grDevices", "dev.off")
importFrom("graphics", "abline", "lines", "mtext", "par", "plot",
           "points", "title")
importFrom("stats", "mad", "median", "predict", "quantile", "rpois",
           "runif", "smooth.spline", "var")

S3method(print, SAMoutput)
S3method(plot, SAMoutput)

export(SAM,
       SAMseq,
       plot.SAMoutput,
       print.SAMoutput,
       samr,
       check.format,
       quantitative.func,
       ttest.func,
       samr.assess.samplesize,
       samr.assess.samplesize.plot,
       samr.compute.delta.table,
       samr.compute.siggenes.table,
       samr.plot,
       samr.missrate,
       samr.norm.data,
       samr.estimate.depth,
       samr.pvalues.from.perms,
       samr.tail.strength,
       samr.const.quantitative.response,
       samr.const.twoclass.unpaired.response,
       samr.const.survival.response,
       samr.const.multiclass.response,
       samr.const.oneclass.response,
       samr.const.twoclass.paired.response,
       samr.const.twoclass.unpaired.timecourse.response,
       samr.const.twoclass.paired.timecourse.response,
       samr.const.oneclass.timecourse.response,
       samr.const.patterndiscovery.response,
       runSAM)
useDynLib(samr)

# Web-enabled and Cross-platform SAM via Shiny

This version of SAM provided adds a web interface using Shiny.

Install the current version of the package, after removing any
older versions you may have already installed. _Later, when this
package makes it to CRAN, you will install it the usual way_.

The web interface is invoked via:

```
library(samr)
runSAM()
```

That will bring up a browser window with a user interface. More
details are provided in the manual `sam.pdf` in this directory

Note that users should use Firefox or Chrome as the _default_browser
on Windows: IE will not work.

Michael Seo

# Web-enabled and Cross-platform SAM via Shiny

First make sure you have a very recent version of R or RStudio.

Next install required packages. Cut and paste what's below in an R session.

```{r}
install.packages(c("samr", "matrixStats", "GSA", "shiny", "openxlsx"))
source("http://bioconductor.org/biocLite.R")
biocLite("impute")
```
You only need to do this once.

Then, you may run SAM any time in an R session as follows.

```{r}
library(shiny)
runGitHub("SAM", "MikeJSeo")
```

That will bring up a browser window with a user interface. More details are provided in the manual sam.pdf in this directory

Please post to the group regarding any issues. This will help us ensure we have all the kinks ironed out before merging the code 
into the next version of the `samr` package.

Thank you,    
Michael Seo

library(openxlsx)
library(samr)
library(GSA)
library(shiny)
library(shinyFiles)

shinyUI(fluidPage(

  # Application title
  titlePanel("SAM - Significance Analysis of Microarrays"),

  # Control panel;
  fluidRow(
    column(3,
    wellPanel(

    textInput("FileName", "File:", value = ""),

    tags$div(title="Click here to select data you would like to analyze",
      shinyFilesButton('file', 'Select a File', 'Please select a file', FALSE)
    ),
    br(),
    conditionalPanel(condition = "input.analysisType == 'Standard'",
      tags$div(title= "Specify the type of analysis from below, then press Run",
        actionButton("goButton", "Run")
      )
    ),
    conditionalPanel(condition = "input.analysisType == 'Gene sets'",
      tags$div(title= "Specify the type of analysis from below, then press Run",
        actionButton("goButton2", "Run")
      )
    ),

    conditionalPanel(condition = "input.goButton != 0 && input.analysisType == 'Standard'",

    tags$hr(),
    radioButtons("deltaChoice", "Delta", c("Delta Slider" = "Delta Slider", "Manually Enter Delta" = "Manually Enter Delta")),
    conditionalPanel(condition = "input.deltaChoice == 'Delta Slider'",
      uiOutput("slider")
    ),

    conditionalPanel(condition = "input.deltaChoice == 'Manually Enter Delta'",
      numericInput("deltaInput", label = "Delta value", min = 0, max = 10, value = 0, step = 0.01)
    ),

    conditionalPanel(condition = "(input.assayType == 'array' && (input.responseType_array == 'Two class unpaired' || input.responseType_array == 'Two class paired' )) || (input.assayType == 'seq' && (input.responseType_seq == 'Two class unpaired' || input.responseType_seq == 'Two class paired'))",
      numericInput("min.foldchange", label = "Minimum fold change", min = 1, value = 0, step = 0.01)
    ),

    conditionalPanel(condition = "(input.assayType == 'array' && (input.responseType_array == 'Two class unpaired' || input.responseType_array == 'Two class paired' || input.responseType_array == 'One class' || input.responseType_array == 'Survival')) || (input.assayType == 'seq' && (input.responseType_seq == 'Two class unpaired' || input.responseType_seq == 'Two class paired' || input.responseType_seq == 'Survival'))",
      numericInput("dif", label = "Hypothesized mean difference in expression", value = 0),
      textInput("sampleSizeFactors", label = "Same size factors - four comma separated values", value = "1,2,3,5")
    ),

    radioButtons("localFDR", "Output local FDRs", c("No" = FALSE, "Yes" = TRUE)),

    textInput("dir", "Paste the filepath to save the output", value = getwd()),
    textInput("fname", "Type the file name you would like to save as", value = "result"),
    actionButton("saveButton", "Save")

    ),


    conditionalPanel(condition = "input.goButton2 != 0 && input.analysisType == 'Gene sets'",

      tags$hr(),
      radioButtons("fdrChoice", "FDR", c("FDR Slider" = "FDR Slider", "Manually Enter FDR" = "Manually Enter FDR")),
      conditionalPanel(condition = "input.fdrChoice == 'FDR Slider'",
        sliderInput("fdrSlider", label = "FDR value", min = 0, max =1, value = 0.5, step = 0.01)
      ),

      conditionalPanel(condition = "input.fdrChoice == 'Manually Enter FDR'",
        numericInput("fdrInput", label = "FDR value", min = 0, max = 1, value = 0.5, step = 0.01)
      ),
      textInput("dir2", "Paste the filepath to save the output", value = getwd()),
      textInput("fname2", "Type the file name you would like to save as", value = "result"),
      actionButton("saveButton2", "Save")

    )
    ),

    wellPanel(
    radioButtons("assayType", "Data type", c("Array" = "array", "Sequencing" = "seq")),

    conditionalPanel(condition = "input.assayType == 'seq'",
      selectInput("responseType_seq","Response Type", c('Quantitative','Two class unpaired', 'Survival', 'Multiclass', 'Two class paired'))
    ),

    conditionalPanel(condition = "input.assayType == 'array'",
      selectInput("responseType_array","Response Type", c('Quantitative','Two class unpaired', 'Survival', 'Multiclass', 'One class', 'Two class paired', 'Two class unpaired timecourse', 'One class timecourse', 'Two class paired timecourse','Pattern discovery')),

      conditionalPanel(condition = paste("(input.responseType_array == 'Quantitative' ||", "input.responseType_array == 'Two class unpaired' ||", "input.responseType_array == 'Survival' ||", "input.responseType_array == 'Multiclass' ||", "input.responseType_array == 'Two class paired' ||", "input.responseType_array == 'Two class unpaired')"),
        radioButtons("analysisType", "Analysis Type", c("Standard (genes)" = "Standard", "Gene sets" = "Gene sets"))
      ),

      conditionalPanel(condition = paste("input.analysisType == 'Gene sets' && ", "(input.responseType_array == 'Quantitative' ||", "input.responseType_array == 'Two class unpaired' ||", "input.responseType_array == 'Survival' ||", "input.responseType_array == 'Multiclass' ||", "input.responseType_array == 'Two class paired' ||", "input.responseType_array == 'Two class unpaired')"),
        fileInput(inputId = "gmtFile", label = "Browse for Gene Sets .gmt file"),
        numericInput("minGeneSet", label = "Minimum gene set size", min = 1, value = 15, step = 1),
        numericInput("maxGeneSet", label = "Maximum gene set size", min = 1, value = 500, step = 1)
      ),

      conditionalPanel(condition = paste("(input.responseType_array == 'Two class unpaired' ||", "input.responseType_array == 'Two class unpaired timecourse')"),
        radioButtons("testStatistic", "Test Statistic", c("T-statistic"="standard", "Wilcoxon"="wilcoxon"))
      ),

      conditionalPanel(condition = paste("(input.responseType_array == 'Two class unpaired timecourse' ||", "input.responseType_array == 'One class timecourse' ||", "input.responseType_array == 'Two class paired timecourse')"),
        radioButtons("timeSummaryType", "Time Summary Type", c("Slope"='slope', "Signed Area"='signed.area'))
      ),

      radioButtons("centerArrays", "Median center the arrays", c("No" = FALSE, "Yes" = TRUE)),
      conditionalPanel(condition = paste("(input.responseType_array == 'Two class unpaired' ||", "input.responseType_array == 'Two class paired')"),
        radioButtons("dataLogged", "Are data in log scale (base 2)", c("No" = FALSE, "Yes" = TRUE))
      ),

      conditionalPanel(condition = "input.responseType_array == 'Quantitative'",
        radioButtons("regressionMethod", "Regression method", c("Standard"='standard', "Ranks"='ranks'))
      ),

      radioButtons("s0", "Estimate of s0 factor for denominator", c("Automatic"="Automatic", "Use fixed percentile (eg 50)"="useFixed")),
      conditionalPanel(condition = "input.s0 == 'useFixed'",
        numericInput("s0.perc", "(-1 implies s0 is zero)", step=1, value = 50)
      ),
      numericInput("numberOfNeighbors", "K-Nearest Neighbors Imputer: Number of Neighbors", value= 10, step=1)
    ),

    numericInput("nperms", "Number of Permutations", value=100, min=25, max=5000, step=1),
    uiOutput("rseed"),
    actionButton("randomButton", "Generate Random Seed")
    )

    ),


  column(9,

    conditionalPanel(condition = "input.analysisType == 'Gene sets'",
      tabsetPanel(id = 'gene set',
        tabPanel("GSA Plot", h3(textOutput("GSAPlotText")), plotOutput("geneSetPlot")),
        tabPanel("Significant Gene Sets", h3(textOutput("geneSetPositive")), tableOutput("positiveGeneSet"), h3(textOutput("geneSetNegative")), tableOutput("negativeGeneSet"), h3(textOutput("GeneScoreText")), numericInput("geneset.number", "geneset number", value= 1, step=1), tableOutput("testGeneSet")),
        tabPanel("All Gene Sets",  h3(textOutput("geneSetFullPositive")), tableOutput("positiveFullGeneSet"), h3(textOutput("geneSetFullNegative")), tableOutput("negativeFullGeneSet")),
        tabPanel("Gene Set Collection Info", h3(textOutput("geneSetInfoText")), tableOutput("geneSetTable"), h3(textOutput("geneSetInfoText2")), tableOutput("geneSetQuantile"), h3(textOutput("geneSetInfoText3")), tableOutput("geneSetTableGenes")),
        tabPanel("Current Settings", h3(textOutput("inputParametersText2")), tableOutput("inputParameters2"), h3(textOutput("computedValuesText2")), tableOutput("computedValues2"))
        )
    ),

    conditionalPanel(condition = "input.analysisType == 'Standard'",
      tabsetPanel(id='SAM',
        tabPanel("SAM Plot", h3(textOutput("samPlotText")), plotOutput("samrPlot")),
        tabPanel("Delta Table", h3(textOutput("deltaTableText")), tableOutput("deltaTable"), h3(textOutput("missRateText")), tableOutput("missRate")),
        tabPanel("Significant Genes", h3(textOutput("positiveGenesText")), dataTableOutput("siggenes.table.up"), h3(textOutput("negativeGenesText")), dataTableOutput("siggenes.table.lo")),
        tabPanel("All Genes", h3(textOutput("allPositiveGenesText")), dataTableOutput("Allgenes.table.up"), h3(textOutput("allNegativeGenesText")), dataTableOutput("Allgenes.table.lo")),
        tabPanel("Sample Size", h3(textOutput("sampleSizePlotText")), plotOutput("samplePlot"),  h3(textOutput("sampleTableText1")), tableOutput("sampleTable1"), h3(textOutput("sampleTableText2")), tableOutput("sampleTable2"), h3(textOutput("sampleTableText3")), tableOutput("sampleTable3"), h3(textOutput("sampleTableText4")), tableOutput("sampleTable4")),
        tabPanel("Current Settings", h3(textOutput("inputParametersText")), tableOutput("inputParameters"),  h3(textOutput("computedValuesText")), tableOutput("computedValues"), h3(textOutput("eigengeneText")), tableOutput("eigengene") )
      )
    )

  )

)
))

GSA.correlate.revised =function(GSA.genesets.obj, genenames){
  nsets=length(GSA.genesets.obj$genesets)
  ngenes=unlist(lapply(GSA.genesets.obj$genesets,length))
  allgenes=unlist(GSA.genesets.obj$genesets)
  sets.in.exp=match(unique(allgenes),genenames)
  exp.in.sets=match(genenames,allgenes)
  
  result = matrix(nrow = 6, ncol = 1)
  colnames(result) = "value"
  rownames(result) = c("Number of gene-sets", "Total number of genes in gene-set collection", "Total number of unique genes in gene-set collection", "Total number of genes in  genenames list", "Total number of unique genes in genenames list", "Number of unique genes in both collections")
  result[1,1] = nsets
  result[2,1] = sum(ngenes)
  result[3,1] = length(unique(allgenes))
  result[4,1] = length(genenames)
  result[5,1] = length(unique(genenames))
  result[6,1] = sum(!is.na(sets.in.exp))
  
  nn=rep(NA,nsets)
  for(i in 1:nsets){
    nn[i]=sum(!is.na(match(GSA.genesets.obj$genesets[[i]],genenames)))
  }
  
  QuantileCoverage =  quantile(nn/ngenes, seq(0,1,by=.1))
  res = matrix(as.numeric(NA), 1, 11)
  res[1,] = QuantileCoverage
  dimnames(res) = list("value", names(QuantileCoverage))
  QuantileCoverage = res
  
  tableGenes = table(ngenes, dnn = "Table of number of genes in genesets")
  tableGenesN = length(names(tableGenes))
  res2 = matrix(as.numeric(NA), tableGenesN, 1)
  dimnames(res2) = list(names(tableGenes), "count")
  for(i in 1:tableGenesN){
    res2[i,1] = tableGenes[[i]]
  }
  tableGenes = res2
  #  cat(c("Quantiles of fraction coverage of gene-sets"),fill=T)
  #  print(quantile(nn/ngenes, seq(0,1,by=.1)),digits=4, fill = T)
  #  cat("", fill = T)
  #  table(ngenes, dnn = "Table of number of genes in genesets")
  list(result = result, QuantileCoverage = QuantileCoverage, tableGenes = tableGenes)
}

GSA.listsets.revised =
function (GSA.obj, geneset.names = NULL, maxchar = 20, FDRcut = 0.2) 
{
if(is.null(geneset.names)){
  geneset.names=rep("xxxxxx", length(GSA.obj$GSA.scores))
}
negflag= !(GSA.obj$resp.type=="Multiclass" | GSA.obj$resp.type=="taCorr")
    r = GSA.obj$GSA.scores
    rstar = GSA.obj$GSA.scores.perm
    r[is.na(r)] = 0
    rstar[is.na(rstar)] = 0
    nperms = ncol(GSA.obj$GSA.scores.perm)
    np = length(r)
geneset.names = substring(geneset.names, 1, maxchar)
    pvalues.lo = GSA.obj$pvalues.lo
    pvalues.hi = GSA.obj$pvalues.hi
    m = sum(!is.na(pvalues.hi))

    make.monotone.increasing = function(x) {
        n = length(x)
        if (n==0) return(NULL)  ## added to prevent error when x is NULL
        if (n==1) x[1] <- x[1]  ## added to prevent error when length of x =1
        else {
   	      for (i in n:2) {
         	  if (x[i - 1] > x[i]) {
           	    x[i - 1] = x[i]
           	}
		      }
        }
        return(x)
    }



    oo = (1:length(r))[!is.na(pvalues.hi)]
    fdr.lo = fdr.hi = rep(NA, length(r))
    for (i in oo) {
    if(negflag){ fdr.lo[i] = round(m * pvalues.lo[i]/sum(pvalues.lo[!is.na(pvalues.lo)] <= pvalues.lo[i]), 4) }
        fdr.hi[i] = round(m * pvalues.hi[i]/sum(pvalues.hi[!is.na(pvalues.hi)] <= pvalues.hi[i]), 4)
    }

 fdr.lo=pmin(fdr.lo,1)
 fdr.hi=pmin(fdr.hi,1)

    res1=NULL
    if(negflag){
        oo1 = (1:length(r))[r < 0]
        res1 = NULL
        for (i in oo1) {
                res1 = rbind(res1, c(i, geneset.names[i], round(GSA.obj$GSA.scores[i], 
                    4), pvalues.lo[i], fdr.lo[i]))
        }
        if (!is.null(res1)) o1 = order(res1[, 4],decreasing = FALSE)  
        else o1 = NULL

        res1 = res1[o1, , drop = F]
        res1[, 5] = make.monotone.increasing(as.numeric(res1[, 5]))
   }

    oo2 = (1:length(r))[r > 0]
    res2 = NULL
    for (i in oo2) {
            res2 = rbind(res2, c(i, geneset.names[i], round(GSA.obj$GSA.scores[i], 
                4), pvalues.hi[i], fdr.hi[i]))
    }
    if (!is.null(res2)) o2 = order(res2[, 4], decreasing = FALSE)  else o2 = NULL

    res2 = res2[o2, , drop = F]
    res2[, 5] = make.monotone.increasing(as.numeric(res2[, 5]))
    if (length(res1) == 0) {
        res1 = NULL
    }
    if (length(res2) == 0) {
        res2 = NULL
    }
    if ( (length(res1) > 0) & negflag) {
            dimnames(res1) = list(NULL, c("Gene_set", "Gene_set_name", 
                "Score", "p-value", "FDR"))
    }
    if (length(res2) > 0) {
            dimnames(res2) = list(NULL, c("Gene_set", "Gene_set_name", 
                "Score", "p-value", "FDR"))
    }
nsets.neg=NULL
   if(negflag){ res1 = res1[as.numeric(res1[, 5]) <= FDRcut,,drop=FALSE ]
        nsets.neg = nrow(res1)
      if (is.null(res1)) {
        nsets.neg = 0
    }}
    res2 = res2[as.numeric(res2[, 5]) <= FDRcut,,drop=FALSE ]
    nsets.pos = nrow(res2)
    if (is.null(res2)) {
        nsets.pos = 0
    }
    return(list(FDRcut = FDRcut, negative = res1, positive = res2, 
        nsets.neg = nsets.neg, nsets.pos = nsets.pos))
}

options(shiny.maxRequestSize=10000*1024^2)

library(openxlsx)
library(samr)
library(GSA)
library(shiny)
library(shinyFiles)

source("GSA.listsets.revised.R")
source("GSA.correlate.revised.R")
source("GSA.plot.revised.R")

shinyServer(function(input, output, session) {

  roots = c(examples =  system.file("excel", package="samr"), root = getVolumes()())
  shinyFileChoose(input, 'file', roots= roots, session=session)

  observe({
    filename = basename(paste(parseFilePaths(roots, input$file)$datapath[1]))
    if(filename != "NA")
      updateTextInput(session, "FileName",  value = filename)
  })


  ##########Read uploaded data!

  getData = reactive({

    objFile = parseFilePaths(roots, input$file)$datapath[1]
    if(!is.na(objFile)){

      dat = read.xlsx(as.character(objFile), 1, colNames = FALSE)

      geneid = dat[-1,1]
      genenames = dat[-1,2]

      imputedX = NULL
      x = as.matrix(dat[-1, c(-1,-2)])
      class(x) = "numeric"

      originalX = NULL
      originalX = cbind(geneid, genenames, x)
      colnamesOriginalX = as.vector(dat[1,])
      colnamesOriginalX[1] = " "
      colnamesOriginalX[2] = " "
      colnames(originalX) = colnamesOriginalX

      if(sum(is.na(x)) > 0){
        imputedummy = impute.knn(x, k = input$numberOfNeighbors)
        x = imputedummy$data
        imputedX = cbind(geneid, genenames, x)
        colnamesImputedX = as.vector(dat[1,])
        colnamesImputedX[1] = " "
        colnamesImputedX[2] = " "
        colnames(imputedX) = colnamesImputedX
      }

      firstrow = as.vector(dat[1,c(-1,-2)])
      censoring.status = NULL
      eigengene.number = NULL

      if( (input$responseType_seq == "Survival" && input$assayType == "seq") || (input$responseType_array == "Survival" && input$assayType == "array")){
        survival = survival(firstrow)
        y = survival$y
        censoring.status = survival$censoring.status
      }
      else if( (input$responseType_array == "Pattern discovery" && input$assayType == "array") ){
        y = NULL
        eigengene.number = firstrow[1]
        last = nchar(eigengene.number)
        eigengene.number = as.numeric(substr(eigengene.number, last, last))
      }
      else{
        y = firstrow
      }

      data =list(x=x,y=y, genenames=genenames, geneid=geneid, logged2= as.logical(input$dataLogged), censoring.status = censoring.status, eigengene.number = eigengene.number, imputedX = imputedX, originalX = originalX)
      data
    }
  })

  #########Modify changes for survival datasets
  survival = function(firstrow){

    y = vector(length = length(firstrow), mode ="integer")
    censoring.status = vector(length = length(firstrow), mode ="integer")
    for(i in seq.int(length(y))) {
      split = strsplit(as.character(firstrow[,i]), ",")
      y[i] = as.character(data.frame(split)[1,])
      y[i] = gsub(" ","", y[i])
      y[i] = substring(y[i], 2)

      censoring.status[i] = as.character(data.frame(split)[2,])
      censoring.status[i] = gsub(" ","", censoring.status[i])
      censoring.status[i] = substring(censoring.status[i], 1, 1)
    }

    y = as.numeric(y)
    censoring.status = as.numeric(censoring.status)

    list(y= y, censoring.status = censoring.status)
  }

  ##########GSA ##############################

  ######get results from GSA
  getGSA = reactive({

    if(input$goButton2 != 0){
      data = getData()
      gmtFile = input$gmtFile

      if(!is.null(data) && !is.null(gmtFile)){

        x = data$x
        y = data$y

        class(y) = "numeric"

        genenames = data$genenames
        censoring.status = data$censoring.status

        geneset.obj = GSA.read.gmt(gmtFile$datapath)
        genesets = geneset.obj$genesets
        geneset.names = geneset.obj$geneset.names

        s0.perc = if(is.na(input$s0.perc) || input$s0 == "Automatic"){NULL}else{input$s0.perc}

        GSA.obj = GSA(x, y, genenames=genenames, genesets=genesets, method = "maxmean", resp.type= input$responseType_array, minsize = input$minGeneSet, maxsize = input$maxGeneSet, random.seed = input$random.seed, knn.neighbors = input$numberOfNeighbors, s0.perc = s0.perc, nperms = input$nperms, censoring.status = censoring.status)
        list(GSA.obj  = GSA.obj, geneset.names = geneset.names, genenames = genenames, genesets = genesets, geneset.obj = geneset.obj)
      }
    }
  })

  getGSACorrelateList = reactive({
    GSA = getGSA()
    if(!is.null(GSA)){
      geneset.obj = GSA$geneset.obj
      genenames = GSA$genenames
      GSA.correlate.list = GSA.correlate.revised(geneset.obj, genenames)
      GSA.correlate.list
    }
  })

  getGSAList = reactive({
    GSA = getGSA()
    if(!is.null(GSA)){
      GSA.obj = GSA$GSA.obj
      geneset.names = GSA$geneset.names
      GSA.list = GSA.listsets.revised(GSA.obj, geneset.names = geneset.names, FDRcut = findFDR())
      GSA.list
    }
  })

  getGSAFullList = reactive({
    GSA = getGSA()
    if(!is.null(GSA)){
      GSA.obj = GSA$GSA.obj
      geneset.names = GSA$geneset.names
      GSA.list = GSA.listsets.revised(GSA.obj, geneset.names = geneset.names, FDRcut = 1)
      GSA.list
    }
  })

  getComputedValues2 = reactive({
    GSA = getGSA()
    if(!is.null(GSA)){
      GSA.obj = GSA$GSA.obj
      computedValues = matrix(NA, nrow = 2, ncol = 1)
      colnames(computedValues) = "Values"
      rownames(computedValues) = c("Exchangibility factor s0", "s0 percentile")

      computedValues[1,1] = GSA.obj$s0
      computedValues[2,1] = GSA.obj$s0.perc
      computedValues
    }
  })


  getInputParameters2 = reactive({

    input$goButton2
    findFDR()

    isolate({

      if(input$goButton2!= 0){
        current = matrix(NA, nrow = 10, ncol = 1)

        objFile = parseFilePaths(roots, input$file)$datapath[1]
        gmtFile = input$gmtFile
        s0.perc = if(is.na(input$s0.perc) || input$s0 == "Automatic"){"Automatic"}else{paste(input$s0.perc, " percentile")}

        current[1,1] = as.character(objFile)
        current[2,1] = gmtFile$name
        current[3,1] = input$responseType_array
        current[4,1] = findFDR()
        current[5,1] = input$nperms
        current[6,1] = s0.perc
        current[7,1] = input$numberOfNeighbors
        current[8,1] = input$minGeneSet
        current[9,1] = input$maxGeneSet
        current[10,1] = input$random.seed

        rownames_current = c("File Path", "Gene set (gmt) file", "Data Type", "False discovery rate (in each tail)", "Number of permutations", "Input percentile for exchangeability factor s0", "Number of neighbors for KNN", "Minimum gene set size", "Maximum gene set size", "Seed for Random number generator")

        rownames(current) = rownames_current
        colnames(current) = "value"
        current
      }

    })
  })

  findFDR = reactive({
    if(input$fdrChoice == "FDR Slider")
      input$fdrSlider
    else if (input$fdrChoice == "Manually Enter FDR")
      input$fdrInput
  })

  #########Output GSA tables/plots
  output$geneSetTable = renderTable({
    GSA.correlate.list = getGSACorrelateList()
    if(!is.null(GSA.correlate.list)){
      GSA.correlate.list$result
    }
  })

  output$geneSetQuantile = renderTable({
    GSA.correlate.list = getGSACorrelateList()
    if(!is.null(GSA.correlate.list)){
      GSA.correlate.list$QuantileCoverage
    }
  })

  output$geneSetTableGenes = renderTable({
    GSA.correlate.list = getGSACorrelateList()
    if(!is.null(GSA.correlate.list)){
      GSA.correlate.list$tableGenes
    }
  })

  output$geneSetPlot = renderPlot({

    GSA = getGSA()
    if(!is.null(GSA)){
      GSA.obj = GSA$GSA.obj
      GSA.plot.revised(GSA.obj, FDRcut = 1, fac = 0)
    }
  })

  output$positiveGeneSet = renderTable({

    GSA.list = getGSAList()
    if(!is.null(GSA.list$positive)){
      if(!is.na(GSA.list$positive[1])){
        GSA.list$positive
      }
    }
  })

  output$negativeGeneSet = renderTable({

    GSA.list = getGSAList()
    if(!is.null(GSA.list$negative)){
      if(!is.na(GSA.list$negative[1])){
        GSA.list$negative
      }
    }
  })

  output$positiveFullGeneSet = renderTable({

    GSAFullList = getGSAFullList()
    if(!is.null(GSAFullList)){
      GSAFullList$positive
    }
  })

  output$negativeFullGeneSet = renderTable({

    GSAFullList = getGSAFullList()
    if(!is.null(GSAFullList)){
      GSAFullList$negative
    }
  })

  output$testGeneSet = renderTable({

    GSA = getGSA()
    if(!is.null(GSA)){
      genesets = GSA$genesets
      GSA.obj = GSA$GSA.obj
      genenames = GSA$genenames
      GSA.genescores(input$geneset.number, genesets, GSA.obj, genenames)
    }
  })

  output$computedValues2 = renderTable({
    if(!is.null(getComputedValues2())){
      getComputedValues2()
    }
  })

  output$inputParameters2 = renderTable({
    inputParameters2 = getInputParameters2()
    if(!is.null(inputParameters2)){
      inputParameters2
    }
  })

  ######output texts in the GSA interface
  output$geneSetInfoText <- renderText({
    if(!is.null(getGSAList())){
      "Information for gene set collection"
    }
  })

  output$geneSetInfoText2 <- renderText({
    if(!is.null(getGSAList())){
      "Quantiles of fraction coverage of gene-sets"
    }
  })

  output$geneSetInfoText3 <- renderText({
    if(!is.null(getGSAList())){
      "Table of number of genes in genesets"
    }
  })

  output$GSAPlotText <- renderText({
    if(!is.null(getGSAList())){
      "GSA Plot"
    }
  })

  output$GeneScoreText <- renderText({
    if(!is.null(getGSAList())){
      "Gene Score"
    }
  })

  output$geneSetPositive = renderText({
    GSA.list = getGSAList()
    if(!is.null(GSA.list$nsets.pos)){
      paste("Positive Gene Sets (", GSA.list$nsets.pos, ")", sep = "")
    }
  })

  output$geneSetNegative = renderText({
    GSA.list = getGSAList()
    if(!is.null(GSA.list$nsets.neg)){
      paste("Negative Gene Sets (", GSA.list$nsets.neg, ")", sep = "")
    }
  })

  output$geneSetFullPositive = renderText({
    GSAFullList = getGSAFullList()
    if(!is.null(GSAFullList)){
      paste("Positive Gene Sets (", GSAFullList$nsets.pos, ")", sep = "")
    }
  })

  output$geneSetFullNegative = renderText({
    GSAFullList = getGSAFullList()
    if(!is.null(GSAFullList)){
      paste("Negative Gene Sets (", GSAFullList$nsets.neg, ")", sep = "")
    }
  })

  output$inputParametersText2 <- renderText({
    if(!is.null(getGSA())){
      "Input Parameters"
    }
  })
  output$computedValuesText2 <- renderText({
    if(!is.null(getGSA())){
      "Computed Values"
    }
  })

  #################################SAM

  ############get results from SAM
  getSamrObj = reactive({

#   input$goButton

    if(input$goButton != 0){
      data = getData()

      if(!is.null(data)){
        resp.type = if(input$assayType == "seq"){input$responseType_seq}else{input$responseType_array}
        s0.perc = if(is.na(input$s0.perc) || input$s0 == "Automatic"){NULL}else{input$s0.perc}
        center.arrays = as.logical(input$centerArrays)

        samr.obj = samr(data, resp.type = resp.type, assay.type = input$assayType, s0.perc = s0.perc, nperms = input$nperms, center.arrays = center.arrays, testStatistic = input$testStatistic, time.summary.type = input$timeSummaryType,  regression.method = input$regressionMethod, random.seed = input$random.seed, knn.neighbors = input$numberOfNeighbors)
        samr.obj
      }
    }
  })

  getDeltaTable = reactive({
    samr.obj = getSamrObj()
    if(!is.null(samr.obj)){
      delta.table = samr.compute.delta.table(samr.obj, min.foldchange = input$min.foldchange)
      delta.table
    }

  })

  getEigengene = reactive({
    samr.obj = getSamrObj()
    if(!is.null(samr.obj) && input$responseType_array == "Pattern discovery"){
      eigengene = samr.obj$eigengene
      eigengene
    }
  })

  getSampleSize = reactive({

    ar = isolate(input$responseType_array)
    seq = isolate(input$responseType_seq)
    samr.obj = getSamrObj()

    if(!is.null(samr.obj) && ((input$assayType == "array" && (ar == "Two class unpaired" || ar == "Two class paired" || ar == "One class" || ar == "Survival")) || (input$assayType == "seq" && (seq == "Two class unpaired" || seq == "Two class paired" || seq == "Survival")))){

      data = getData()
      y = data$y
      dif = input$dif
      #quick fix not allowing blocks for sample size
      if(!substring(y[1],2,6) %in% c("Block", "block")){
        samplesize.factors = input$sampleSizeFactors
        samplesize.factors = as.numeric(unlist(strsplit(samplesize.factors, ",")))
        samr.assess.samplesize.obj =  samr.assess.samplesize(samr.obj, data, dif, samplesize.factors)
        samr.assess.samplesize.obj
      }
    }
  })

  getMissRateTable = reactive({

    samr.obj = getSamrObj()
    if(!is.null(samr.obj)){

      delta = findDelta()
      delta.table = getDeltaTable()

      missrate.table = samr.missrate(samr.obj, delta, delta.table)
      missrate.table
    }

  })

  getSiggenesTable = reactive({

    samr.obj = getSamrObj()
    if(!is.null(samr.obj)){
      delta = findDelta()
      data = getData()
      delta.table = getDeltaTable()
      min.foldchange = input$min.foldchange

      siggenes.table = samr.compute.siggenes.table(samr.obj,delta, data, delta.table, min.foldchange = min.foldchange, compute.localfdr= input$localFDR)
      siggenes.table
    }
  })


  getAllgenesTable = reactive({

    samr.obj = getSamrObj()
    if(!is.null(samr.obj)){
      delta = findDelta()
      data = getData()
      delta.table = getDeltaTable()
      min.foldchange = input$min.foldchange

      Allgenes = samr.compute.siggenes.table(samr.obj,delta, data, delta.table, min.foldchange = min.foldchange, compute.localfdr= input$localFDR, all.genes= TRUE)
      Allgenes
    }
  })

  getImputedX = reactive({

    data = getData()
    imputedX = data$imputedX
    if(!is.null(imputedX)){
      imputedX
    }
  })

  getOriginalX = reactive({

    data = getData()
    originalX = data$originalX
    if(!is.null(originalX)){
      originalX
    }
  })

  getComputedValues= reactive({

    samr.obj = getSamrObj()
    if(!is.null(samr.obj)){

      computedValues = matrix(NA, nrow = 5, ncol = 1)

      colnames(computedValues) = "Values"
      rownames(computedValues) = c("Estimated proportion of non-null features (genes)", "s0 percentile", "False Discovery Rate", "Estimated tail strength", "Estimated standard error of tail strength")
      tail.strength = getTailStrength()

      computedValues[1,1] = samr.obj$pi0
      computedValues[2,1] = samr.obj$s0.perc
      delta.table = samr.compute.delta.table(samr.obj, min.foldchange= input$min.foldchange, dels= findDelta())[5]
      computedValues[3,1] = if(is.na(delta.table)){0.00}else{delta.table}
      computedValues[4,1] = tail.strength$ts
      computedValues[5,1] = tail.strength$se.ts

      if(input$assayType == "array" && input$analysisType == "Standard" && input$responseType_array == "Multiclass"){
        multiclassAdd = matrix(NA, nrow = 2, ncol = 1)
        rownames(multiclassAdd) = c("2.5% null value for multiclass contrasts", "97.5% null value for multiclass contrasts")
        multiclassAdd[1,1] = samr.obj$stand.contrasts.95[1]
        multiclassAdd[2,1] = samr.obj$stand.contrasts.95[2]
        computedValues = rbind(computedValues, multiclassAdd)
      }

      if(input$assayType == "seq"){
        depth = samr.obj$depth
        seqAdd = matrix(NA, nrow = 1, ncol = 1)
        rownames(seqAdd) = "Quantiles of estimated sequencing depths (0%, 25%, 50%, 75%, 100%)"
        seqAdd[1,1] = paste(quantile(depth, 0)[[1]], quantile(depth, 0.25)[[1]], quantile(depth, 0.5)[[1]], quantile(depth, 0.75)[[1]], quantile(depth, 1)[[1]],  sep = ", ")
        computedValues = rbind(computedValues, seqAdd)
      }
      computedValues
    }
  })

  getTailStrength = reactive({
    samr.obj = getSamrObj()
    if(!is.null(samr.obj)){
      samr.tail.strength(samr.obj)
    }
  })

  getInputParameters = reactive({

    input$goButton
    findDelta()
    input$min.foldchange

    isolate({

      if(input$goButton!= 0){


        if(input$assayType == "array"){
          current = matrix(NA, nrow = 14, ncol = 1)

          objFile = parseFilePaths(roots, input$file)$datapath[1]
          s0.perc = if(is.na(input$s0.perc) || input$s0 == "Automatic"){"Automatic"}else{paste(input$s0.perc, " percentile")}

          current[1,1] = as.character(objFile)
          current[2,1] = input$responseType_array
          current[3,1] = capitalize(input$assayType)
          current[4,1] = input$centerArrays
          current[5,1] = findDelta()
          current[6,1] = input$min.foldchange
          current[7,1] = if(input$testStatistic == "standard"){"T-statistic"} else{"Wilcoxon"}
          current[8,1] = capitalize(input$regressionMethod)
          current[9,1] = input$dataLogged
          current[10,1] = input$nperms
          current[11,1] = s0.perc
          current[12,1] = input$numberOfNeighbors
          current[13,1] = input$random.seed
          current[14,1] = capitalize(input$timeSummaryType)

          rownames_current = c("File Path", "Data Type", "Array or Seq data?", "Arrays centered?", "Delta", "Minimum fold change", "Test statistic", "Regression method", "Are data are log scale?", "Number of permutations", "Input percentile for exchangeability factor s0", "Number of neighbors for KNN", "Seed for Random number generator", "Time summary type")

          if(input$responseType_array == "Quantitative"){
            current = matrix(current[c(-6, -7, -9, -14),], ncol = 1)
            rownames_current = rownames_current[c(-6,-7, -9, -14)]
          }
          else if(input$responseType_array == "Two class unpaired"){
            current = matrix(current[c(-8, -14),], ncol = 1)
            rownames_current = rownames_current[c(-8, -14)]
          }
          else if(input$responseType_array == "Survival"){
            current = matrix(current[c(-6, -7,-8,-9,-14),], ncol = 1)
            rownames_current = rownames_current[c(-6, -7,-8,-9, -14)]
          }
          else if(input$responseType_array == "Multiclass" || input$responseType_array == "One class" || input$responseType_array == "Pattern discovery"){
            current = matrix(current[c(-6,-7,-8,-9,-14),], ncol = 1)
            rownames_current = rownames_current[c(-6,-7,-8,-9, -14)]
          }
          else if(input$responseType_array == "Two class paired"){
            current = matrix(current[c(-7,-8,-14),], ncol = 1)
            rownames_current = rownames_current[c(-7,-8,-14)]
          }
          else if(input$responseType_array == "Two class unpaired timecourse"){
            current = matrix(current[c(-6, -8, -9),], ncol = 1)
            rownames_current = rownames_current[c(-6, -8, -9)]
          }
          else if(input$responseType_array == "Two class paired timecourse"){
            current = matrix(current[c(-6, -7, -8, -9),], ncol = 1)
            rownames_current = rownames_current[c(-6, -7, -8, -9)]
          }
          else if(input$responseType_array == "One class timecourse"){
            current = matrix(current[c(-6, -7, -8, -9),], ncol = 1)
            rownames_current = rownames_current[c(-6, -7, -8, -9)]
          }

          rownames(current) = rownames_current
          colnames(current) = "value"
        }

        if(input$assayType == "seq"){

          current = matrix(NA, nrow = 9, ncol = 1)

          objFile = parseFilePaths(roots, input$file)$datapath[1]

          current[1,1] = as.character(objFile)
          current[2,1] = input$responseType_seq
          current[3,1] = capitalize(input$assayType)
          current[4,1] = input$centerArrays
          current[5,1] = findDelta()
          current[6,1] = input$min.foldchange
          current[7,1] = input$nperms
          current[8,1] = input$numberOfNeighbors
          current[9,1] = input$random.seed

          rownames_current = c("File Path", "Data Type", "Array or Seq data?", "Arrays centered?", "Delta", "Minimum fold change", "Number of permutations", "Number of neighbors for KNN", "Seed for Random number generator")

          if(input$responseType_seq == "Quantitative" || input$responseType_seq == "Survival" || input$responseType_seq == "Multiclass"){
            current = matrix(current[c(-6),], ncol = 1)
            rownames_current = rownames_current[c(-6)]
          }

          rownames(current) = rownames_current
          colnames(current) = "value"
        }

        current
      }
    })
  })

  findDelta = reactive({

    if(input$deltaChoice == "Delta Slider")
      input$deltaSlider
    else if (input$deltaChoice == "Manually Enter Delta")
      input$deltaInput

  })

  output$rseed <- renderUI({

    if(input$randomButton == 0){
      value = 1234567
    }else{
      value = round(runif(1, 0, 10^6))
    }
    numericInput("random.seed", "Random Seed", value= value)
  })

  output$slider <- renderUI({

    delta.table = getDeltaTable()
    if(!is.null(delta.table)){
      sliderInput("deltaSlider", label = "Delta value", min = 0, max = round(as.numeric(delta.table[dim(delta.table)[1],1]), 2), value = 0, step = 0.01)
    }

  })




  capitalize = function(word){
    letters = strsplit(word,'')
    theletters = letters[[1]]
    theletters[1] = toupper(theletters[1])
    paste(theletters,collapse='')
  }

  ############Write texts for SAM interface

  output$missRateText = renderText({
    if(!is.null(getSamrObj())){
      paste("Estimated Miss rates for Delta = ", findDelta())
    }

  })

  output$eigengeneText = renderText({
    if(!is.null(getSamrObj()) && input$responseType_array == "Pattern discovery"){
      "Eigengene"
    }
  })


  output$eigengene = renderTable({
    eigengene = getEigengene()
    if(!is.null(eigengene)){
      eigengene
    }
  })

  output$negativeGenesText <- renderText({

    siggenes.table = getSiggenesTable()
    if(!is.null(siggenes.table)){
      paste("Negative Genes (", siggenes.table$ngenes.lo, ")", sep ="")
    }
  })

  output$positiveGenesText <- renderText({

    siggenes.table = getSiggenesTable()
    if(!is.null(siggenes.table)){
      paste("Positive Genes (", siggenes.table$ngenes.up, ")", sep = "")
    }
  })

  output$allPositiveGenesText <- renderText({

    allgenes = getAllgenesTable()
    if(!is.null(allgenes)){
      paste("Positive Genes (", allgenes$ngenes.up, ")", sep = "")
    }
  })


  output$allNegativeGenesText = renderText({

    allgenes = getAllgenesTable()
    if(!is.null(allgenes)){
      paste("Negative Genes (", allgenes$ngenes.lo, ")", sep = "")
    }
  })

  output$deltaTableText <- renderText({
    if(!is.null(getSamrObj())){
      "Delta Table"
    }
  })

  output$samPlotText <- renderText({
    if(!is.null(getSamrObj())){
      "SAM Plot"
    }
  })

  output$inputParametersText <- renderText({
    if(!is.null(getSamrObj())){
      "Input Parameters"
    }
  })

  output$computedValuesText <- renderText({
    if(!is.null(getSamrObj())){
      "Computed Values"
    }
  })

  output$sampleSizePlotText2 <- renderText({
    if(!is.null(getSampleSize())){
      "Sample Size Plot"
    }
  })

  output$sampleTableText1 = renderText({
    samr.assess.samplesize.obj =  getSampleSize()
    if(!is.null(getSampleSize())){
      paste("Sample Size = ", samr.assess.samplesize.obj$samplesize.factor[1] * samr.assess.samplesize.obj$n, sep = "")
    }
  })

  output$sampleTableText2 = renderText({
    samr.assess.samplesize.obj =  getSampleSize()
    if(!is.null(getSampleSize())){
      paste("Sample Size = ", samr.assess.samplesize.obj$samplesize.factor[2] * samr.assess.samplesize.obj$n, sep = "")
    }
  })

  output$sampleTableText3 = renderText({
    samr.assess.samplesize.obj =  getSampleSize()
    if(!is.null(getSampleSize())){
      paste("Sample Size = ", samr.assess.samplesize.obj$samplesize.factor[3] * samr.assess.samplesize.obj$n, sep = "")
    }
  })

  output$sampleTableText4 = renderText({
    samr.assess.samplesize.obj =  getSampleSize()
    if(!is.null(getSampleSize())){
      paste("Sample Size = ", samr.assess.samplesize.obj$samplesize.factor[4] * samr.assess.samplesize.obj$n, sep = "")
    }
  })



  ########Output tables and plots for SAM
  output$missRate = renderTable({
    if(!is.null(getMissRateTable())){
      getMissRateTable()
    }
  })

  output$sampleTable1 = renderTable({
    samr.assess.samplesize.obj =  getSampleSize()
    if(!is.null(samr.assess.samplesize.obj)){
      samr.assess.samplesize.obj$results[,,1]
    }
  })

  output$sampleTable2 = renderTable({
    samr.assess.samplesize.obj =  getSampleSize()
    if(!is.null(samr.assess.samplesize.obj)){
      samr.assess.samplesize.obj$results[,,2]
    }
  })

  output$sampleTable3 = renderTable({
    samr.assess.samplesize.obj =  getSampleSize()
    if(!is.null(samr.assess.samplesize.obj)){
      samr.assess.samplesize.obj$results[,,3]
    }
  })

  output$sampleTable4 = renderTable({
    samr.assess.samplesize.obj =  getSampleSize()
    if(!is.null(samr.assess.samplesize.obj)){
      samr.assess.samplesize.obj$results[,,4]
    }
  })

  output$samplePlot = renderPlot({
    samr.assess.samplesize.obj =  getSampleSize()
    if(!is.null(samr.assess.samplesize.obj)){
      samr.assess.samplesize.plot(samr.assess.samplesize.obj)
    }
  })

  output$samrPlot = renderPlot({
    samr.obj = getSamrObj()
    delta = findDelta()
    min.foldchange = input$min.foldchange
    if(!is.null(samr.obj)){
      samr.plot(samr.obj, delta, min.foldchange = min.foldchange)
    }
  })

  output$deltaTable = renderTable({
    delta.table = getDeltaTable()
    if(!is.null(delta.table)){
      delta.table
    }
  })

  output$Allgenes.table.up = renderDataTable({
    Allgenes = getAllgenesTable()
    if(!is.null(Allgenes$genes.up)){
      genes.up = Allgenes$genes.up
      genes.up[,4:7] = round(as.numeric(as.character(genes.up[,4:7])),4)
      genes.up
    }
  })

  output$Allgenes.table.lo = renderDataTable({
    Allgenes = getAllgenesTable()
    if(!is.null(Allgenes$genes.lo)){
      genes.lo = Allgenes$genes.lo
      genes.lo[,4:7] = round(as.numeric(as.character(genes.lo[,4:7])),4)
      genes.lo
    }
  })

  output$siggenes.table.up <- renderDataTable({
    siggenes.table = getSiggenesTable()
    if(!is.null(siggenes.table$genes.up)){
      genes.up = siggenes.table$genes.up
      genes.up[,4:7] = round(as.numeric(as.character(genes.up[,4:7])),4)
      genes.up
    }

  })

  output$siggenes.table.lo <- renderDataTable({
    siggenes.table = getSiggenesTable()
    if(!is.null(siggenes.table$genes.lo)){
      genes.lo = siggenes.table$genes.lo
      genes.lo[,4:7] = round(as.numeric(as.character(genes.lo[,4:7])),4)
      genes.lo
    }
  })

  output$computedValues = renderTable({
    if(!is.null(getComputedValues())){
      getComputedValues()
    }
  })

  output$inputParameters = renderTable({
    inputParameters = getInputParameters()
    if(!is.null(inputParameters)){
      inputParameters
    }
  })


  ##########Saving results into xlsx format

  #######SAM results save
  savethis = observe({

    if(input$saveButton != 0){

    isolate({

    dir = input$dir
    file = input$fname

    siggenes.table = getSiggenesTable()
    missrate.table = getMissRateTable()
    delta.table = getDeltaTable()
    samr.assess.samplesize.obj =  getSampleSize()
    Allgenes = getAllgenesTable()

    samr.obj = getSamrObj()
    delta = findDelta()
    min.foldchange = input$min.foldchange

    inputParameters = getInputParameters()
    computedValues = getComputedValues()
    eigengene = getEigengene()
    imputedX = getImputedX()
    originalX = getOriginalX()

    titleStyle = createStyle(fontSize = 14, fontColour = "#FFFFFF", halign = "center", fgFill = "#4F81BD")

    wb = createWorkbook()



    if(!is.null(samr.obj)){
      png(file = "SAMPlot.png")
      samr.plot(samr.obj, delta, min.foldchange = min.foldchange)
      dev.off()
    }

    if(!is.null(samr.assess.samplesize.obj)){
      png(file = "samplesizePlot.png")
      samr.assess.samplesize.plot(samr.assess.samplesize.obj)
      dev.off()
    }

    if(!is.null(originalX)){
      addWorksheet(wb, sheetName = "Original Data")
      writeData(wb, originalX, sheet = "Original Data")
    }

    if(!is.null(imputedX)){
      addWorksheet(wb, sheetName = "Imputed Data")
      writeData(wb, imputedX, sheet = "Imputed Data")
    }

    if(!is.null(samr.obj)){
      addWorksheet(wb, sheetName = "SAM Plot")
      insertImage(wb, sheet = "SAM Plot", file = "SAMPlot.png", width = 5, height = 5)
    }

    if(!is.null(delta.table)){
      addWorksheet(wb, sheetName = "Delta Table")
      writeData(wb, sheet = "Delta Table", x = "Delta Table")
      writeData(wb, sheet = "Delta Table", x = delta.table, startRow = 2)

      addStyle(wb, sheet = "Delta Table", titleStyle, rows = 1, cols = 1)
      setColWidths(wb, "Delta Table", cols= 1:8, widths = 18)
    }

    if(!is.null(missrate.table)){
      missraterow = nrow(delta.table) + 4
      writeData(wb, sheet = "Delta Table", x = "Miss Rates", startRow = missraterow)
      writeData(wb, sheet = "Delta Table", x = missrate.table, startRow = missraterow + 1)
      addStyle(wb, sheet = "Delta Table", titleStyle, rows = missraterow, cols = 1)
    }

    if((siggenes.table$ngenes.up != 0) || (siggenes.table$ngenes.lo != 0)){
      addWorksheet(wb, sheetName = "Significant Genes")
      setColWidths(wb, "Significant Genes", cols= 1:11, widths = 18)
    }

    significantgenerow = 1
    if(!is.null(siggenes.table$genes.up)){
      writeData(wb, sheet = "Significant Genes", x = "Significant Pos", startRow = significantgenerow)
      addStyle(wb, sheet = "Significant Genes", titleStyle, rows = significantgenerow, cols = 1)
      writeData(wb, sheet = "Significant Genes", x = siggenes.table$genes.up, startRow = significantgenerow + 1)
      significantgenerow = siggenes.table$ngenes.up + 4

    }

    if(!is.null(siggenes.table$genes.lo)){
      writeData(wb, sheet = "Significant Genes", x = "Significant Neg", startRow = significantgenerow)
      writeData(wb, sheet = "Significant Genes", x = siggenes.table$genes.lo, startRow = significantgenerow + 1)
      addStyle(wb, sheet = "Significant Genes", titleStyle, rows = significantgenerow, cols = 1)
    }


    if(!is.null(Allgenes)){
      addWorksheet(wb, sheetName = "All Genes")
      setColWidths(wb, "All Genes", cols= 1:11, widths = 18)
    }

    allgenerow = 1
    if(!is.null(Allgenes$genes.up)){
      writeData(wb, sheet = "All Genes", x = "All Pos", startRow = allgenerow)
      addStyle(wb, sheet = "All Genes", titleStyle, rows = allgenerow, cols = 1)
      writeData(wb, sheet = "All Genes", x = Allgenes$genes.up, startRow = allgenerow + 1)
      allgenerow = Allgenes$ngenes.up + 4
    }

    if(!is.null(Allgenes$genes.lo)){
      writeData(wb, sheet = "All Genes", x = "All Neg", startRow = allgenerow)
      addStyle(wb, sheet = "All Genes", titleStyle, rows = allgenerow, cols = 1)
      writeData(wb, sheet = "All Genes", x = Allgenes$genes.lo, startRow = allgenerow + 1)
    }

    if(!is.null(samr.assess.samplesize.obj)){
      addWorksheet(wb, sheetName = "Sample Size")
      setColWidths(wb, "Sample Size", cols= 1:8, widths = 20)
      insertImage(wb, sheet = "Sample Size", file = "samplesizePlot.png", width = 5, height = 5, startRow = 1)

      dataSample1 = paste("Sample Size = ", samr.assess.samplesize.obj$samplesize.factor[1] * samr.assess.samplesize.obj$n, sep = "")
      writeData(wb, sheet = "Sample Size", x = dataSample1, startRow = 26)
      addStyle(wb, sheet = "Sample Size", titleStyle, rows = 26, cols = 1)
      writeData(wb, sheet = "Sample Size", x = samr.assess.samplesize.obj$results[,,1], startRow = 27)

      dataSample2 = paste("Sample Size = ", samr.assess.samplesize.obj$samplesize.factor[2] * samr.assess.samplesize.obj$n, sep = "")
      writeData(wb, sheet = "Sample Size", x = dataSample2, startRow = 39)
      addStyle(wb, sheet = "Sample Size", titleStyle, rows = 39, cols = 1)
      writeData(wb, sheet = "Sample Size", x = samr.assess.samplesize.obj$results[,,2], startRow = 40)

      dataSample3 = paste("Sample Size = ", samr.assess.samplesize.obj$samplesize.factor[3] * samr.assess.samplesize.obj$n, sep = "")
      writeData(wb, sheet = "Sample Size", x = dataSample3, startRow = 52)
      addStyle(wb, sheet = "Sample Size", titleStyle, rows = 52, cols = 1)
      writeData(wb, sheet = "Sample Size", x = samr.assess.samplesize.obj$results[,,3], startRow = 53)

      dataSample4 = paste("Sample Size = ", samr.assess.samplesize.obj$samplesize.factor[4] * samr.assess.samplesize.obj$n, sep = "")
      writeData(wb, sheet = "Sample Size", x = dataSample4, startRow = 65)
      addStyle(wb, sheet = "Sample Size", titleStyle, rows = 65, cols = 1)
      writeData(wb, sheet = "Sample Size", x = samr.assess.samplesize.obj$results[,,4], startRow = 66)
    }

    if(!is.null(inputParameters)){
      addWorksheet(wb, sheetName = "Current Settings")
      setColWidths(wb, "Current Settings", cols= 1, widths = 46)
      setColWidths(wb, "Current Settings", cols= 2, widths = 18)
      newInputParameters = cbind(rownames(inputParameters), inputParameters)
      colnames(newInputParameters) = c("Questions","Values")
      writeData(wb, sheet = "Current Settings", x = "Input Parameters")
      addStyle(wb, sheet = "Current Settings", titleStyle, rows = 1, cols = 1)
      writeData(wb, sheet = "Current Settings", x = newInputParameters, startRow = 2)

      currentsettingsrow = nrow(newInputParameters) + 4
      writeData(wb, sheet = "Current Settings", x = "Computed Values", startRow = currentsettingsrow)
      addStyle(wb, sheet = "Current Settings", titleStyle, rows = currentsettingsrow, cols = 1)

      newComputedValues = cbind(rownames(computedValues), computedValues)
      colnames(newComputedValues) = c("Questions","Values")
      writeData(wb, sheet = "Current Settings", x = newComputedValues, startRow = currentsettingsrow + 1)


      if(!is.null(eigengene)){
        currentsettingsrow = currentsettingsrow + nrow(newComputedValues)
        writeData(wb, sheet = "Current Settings", x = "Eigengene", startRow = currentsettingsrow + 3)
        addStyle(wb, sheet = "Current Settings", titleStyle, rows = currentsettingsrow + 3, cols = 1)
        writeData(wb, sheet = "Current Settings", x = eigengene, startRow = currentsettingsrow + 4)
      }
    }

    if(!is.null(samr.obj)){
      fname = paste(file, "xlsx", sep = ".")
      saveWorkbook(wb, file.path(dir, fname), overwrite = TRUE)

      if(Sys.info()[['sysname']] == "Windows"){
        shell(file.path(dir, fname))
      } else if(Sys.info()[['sysname']] == "Darwin"){
        system(paste("open", fname))
      }

    }


    })
    }

  })

  ####### Save .xlsx for GSA methods
  savethis2 = observe({

    if(input$saveButton2 != 0){

      isolate({

        dir = input$dir2
        file = input$fname2

        GSA = getGSA()
        GSA.obj = GSA$GSA.obj
        GSA.list = getGSAList()
        GSAFullList = getGSAFullList()

        GSA.correlate.list = getGSACorrelateList()
        geneSetTable = GSA.correlate.list$result
        geneSetQuantile = GSA.correlate.list$QuantileCoverage
        geneSetTableGenes = GSA.correlate.list$tableGenes

        inputParameters = getInputParameters2()
        computedValues = getComputedValues2()

        originalX = getOriginalX()
        imputedX = getImputedX()

        titleStyle = createStyle(fontSize = 14, fontColour = "#FFFFFF", halign = "center", fgFill = "#4F81BD")

        wb = createWorkbook()

        if(!is.null(GSA.obj)){
          png(file = "GSAPlot.png")
          GSA.plot.revised(GSA.obj, FDRcut = 1, fac = 0)
          dev.off()
        }

        if(!is.null(originalX)){
          addWorksheet(wb, sheetName = "Original Data")
          writeData(wb, originalX, sheet = "Original Data")
        }

        if(!is.null(imputedX)){
          addWorksheet(wb, sheetName = "Imputed Data")
          writeData(wb, imputedX, sheet = "Imputed Data")
        }

        if(!is.null(GSA)){
          addWorksheet(wb, sheetName = "GSA Plot")
          insertImage(wb, sheet = "GSA Plot", file = "GSAPlot.png", width = 5, height = 5)
        }

        if(!is.null(GSA.list$positive[1]) || !is.null(GSA.list$negative[1])){
          addWorksheet(wb, sheetName = "Significant Gene Sets")
          setColWidths(wb, "Significant Gene Sets", cols= 1:5, widths = 18)
        }

        significantgenerow = 1
        if(!(GSA.list$nsets.pos == 0)){
          writeData(wb, sheet = "Significant Gene Sets", x = "Significant Pos", startRow = significantgenerow)
          addStyle(wb, sheet = "Significant Gene Sets", titleStyle, rows = significantgenerow, cols = 1)
          writeData(wb, sheet = "Significant Gene Sets", x = GSA.list$positive, startRow = significantgenerow + 1)
          significantgenerow = GSA.list$nsets.pos + 4
        }

        if(!(GSA.list$nsets.neg == 0)){
          writeData(wb, sheet = "Significant Gene Sets", x = "Significant Neg", startRow = significantgenerow)
          writeData(wb, sheet = "Significant Gene Sets", x = GSA.list$negative, startRow = significantgenerow + 1)
          addStyle(wb, sheet = "Significant Gene Sets", titleStyle, rows = significantgenerow, cols = 1)
        }


        if(!is.null(GSAFullList$positive) || !is.null(GSAFullList$negative)){
          addWorksheet(wb, sheetName = "Full Gene Sets")
          setColWidths(wb, "Full Gene Sets", cols= 1:5, widths = 18)
        }

        allgenerow = 1
        if(!is.null(GSAFullList$positive)){
          writeData(wb, sheet = "Full Gene Sets", x = "All Positive", startRow = allgenerow)
          addStyle(wb, sheet = "Full Gene Sets", titleStyle, rows = allgenerow, cols = 1)
          writeData(wb, sheet = "Full Gene Sets", x = GSAFullList$positive, startRow = allgenerow + 1)
          allgenerow = GSAFullList$nsets.pos + 4
        }

        if(!is.null(GSAFullList$negative)){
          writeData(wb, sheet = "Full Gene Sets", x = "All Negative", startRow = allgenerow)
          addStyle(wb, sheet = "Full Gene Sets", titleStyle, rows = allgenerow, cols = 1)
          writeData(wb, sheet = "Full Gene Sets", x = GSAFullList$negative, startRow = allgenerow + 1)
        }

        if(!is.null(geneSetTable)){
          addWorksheet(wb, sheetName = "Gene Set Collection")
          setColWidths(wb, "Gene Set Collection", cols= 1:2, widths = 40)

          genesetinforow = 1
          newGeneSetTable = cbind(rownames(geneSetTable), geneSetTable)
          colnames(newGeneSetTable) = c("Questions","Values")
          writeData(wb, sheet = "Gene Set Collection", x = "Information for gene set collection", startRow = genesetinforow)
          addStyle(wb, sheet = "Gene Set Collection", titleStyle, rows = genesetinforow, cols = 1)
          writeData(wb, sheet = "Gene Set Collection", x = newGeneSetTable, startRow = genesetinforow + 1)
          genesetinforow = nrow(newGeneSetTable) + 4

          newGeneSetQuantile = cbind(colnames(geneSetQuantile), geneSetQuantile[1,])
          colnames(newGeneSetQuantile) = c("Quantiles","Values")
          writeData(wb, sheet = "Gene Set Collection", x = "Quantiles of fraction coverage", startRow = genesetinforow)
          addStyle(wb, sheet = "Gene Set Collection", titleStyle, rows = genesetinforow, cols = 1)
          writeData(wb, sheet = "Gene Set Collection", x = newGeneSetQuantile, startRow = genesetinforow + 1)
          genesetinforow = genesetinforow + nrow(newGeneSetQuantile) + 3

          newGeneSetTableGenes = cbind(rownames(geneSetTableGenes), geneSetTableGenes)
          colnames(newGeneSetTableGenes) = c("Number of genes in a set","count")
          writeData(wb, sheet = "Gene Set Collection", x = "Number of genes", startRow = genesetinforow)
          addStyle(wb, sheet = "Gene Set Collection", titleStyle, rows = genesetinforow, cols = 1)
          writeData(wb, sheet = "Gene Set Collection", x = newGeneSetTableGenes, startRow = genesetinforow + 1)
        }

        if(!is.null(inputParameters)){
          addWorksheet(wb, sheetName = "Current Settings")
          setColWidths(wb, "Current Settings", cols= 1:2, widths = 40)

          newInputParameters = cbind(rownames(inputParameters), inputParameters)
          colnames(newInputParameters) = c("Questions","Values")

          currentsettingsrow = 1
          writeData(wb, sheet = "Current Settings", x = "Input Parameters", startRow = currentsettingsrow)
          addStyle(wb, sheet = "Current Settings", titleStyle, rows = currentsettingsrow, cols = 1)
          writeData(wb, sheet = "Current Settings", x = newInputParameters, startRow = currentsettingsrow + 1)
          currentsettingsrow = nrow(newInputParameters) + 4

          writeData(wb, sheet = "Current Settings", x = "Computed Values", startRow = currentsettingsrow)
          addStyle(wb, sheet = "Current Settings", titleStyle, rows = currentsettingsrow, cols = 1)
          newComputedValues = cbind(rownames(computedValues), computedValues)
          colnames(newComputedValues) = c("Questions","Values")
          writeData(wb, sheet = "Current Settings", x = newComputedValues, startRow = currentsettingsrow + 1)
        }

        if(!is.null(GSA)){
          fname = paste(file, "xlsx", sep = ".")
          saveWorkbook(wb, file.path(dir, fname), overwrite = TRUE)

          if(Sys.info()[['sysname']] == "Windows"){
            shell(file.path(dir, fname))
          } else if(Sys.info()[['sysname']] == "Darwin"){
            system(paste("open", fname))
          }

        }

      })
    }

  })



})

GSA.plot.revised=function(GSA.obj,  fac=1, FDRcut=1){

eps=.001
a=GSA.listsets(GSA.obj, geneset.names=NULL,  FDRcut=FDRcut)
neg=matrix(as.numeric(as.character(a$negative[,-2])),ncol=4)+eps
pos=matrix(as.numeric(as.character(a$positive[,-2])),ncol=4)+eps

ymax=max(c(neg[,4],pos[,4]))
xmax=max(c(neg[,3],pos[,3]))
ymin=min(c(neg[,4],pos[,4]))
xmin=min(c(neg[,3],pos[,3]))

o1=order(neg[,3])
o2=order(pos[,3])
plot(jitter(neg[o1,3],factor=fac),jitter(neg[o1,4],factor=fac),xlab="p-value",ylab="False discovery rate", type="n",log="x", xlim=c(xmin,xmax),
ylim=c(ymin,ymax))
points(jitter(neg[o1,3],factor=fac),jitter(neg[o1,4],factor=fac),col=3,cex=.7,type="b")
points(jitter(pos[o2,3],factor=fac),jitter(pos[o2,4],factor=fac),col=4, cex=.7,type="b")
#axis(3, at = res[,3], lab = paste(round(res[,3],0)), srt = 90, adj = 0,cex=.5)
legend("bottomright",c("Negative","Positive"),col=3:4, pch=c("o","o"))
}

######################################################################
#		A concise version of samr for sequencing data
######################################################################
SAMseq <- function(x, y, censoring.status = NULL, 
	resp.type = c("Quantitative", "Two class unpaired", "Survival", "Multiclass", "Two class paired"), 
	geneid = NULL, genenames = NULL, nperms = 100, random.seed = NULL, nresamp = 20, fdr.output = 0.20)
{
	this.call <- match.call()
	xl.mode = "regular"
	xl.time = NULL
	xl.prevfit = NULL
	
        if(fdr.output <0 | fdr.output>1){
        stop("Error: fdr.output must be between 0 and 1")
       }

	## set gene id and gene names
	if (is.null(geneid))
	{
		geneid = as.character(1:nrow(x))
	}
	if (is.null(genenames))
	{
		genenames = paste("g", as.character(1:nrow(x)), sep = "")
	}
	
	## construct the data list
	data = list(x = x, y = y, censoring.status = censoring.status, geneid = geneid, genenames = genenames)
	
	## call samr with approprate parameters
	samr.obj = samr(data, resp.type = resp.type, assay.type = "seq", nperms = nperms, return.x = TRUE, 
		random.seed = random.seed, nresamp = nresamp, nresamp.perm = nresamp)
		
	## construct a full delta table with no fold change constraints
	delta.table <- samr.compute.delta.table(samr.obj)
	
	## get the significant gene table
	siggenes.table <- del <- NULL
	delta.table <- delta.table[delta.table[, "# called"] > 0, , drop=FALSE]
	if (nrow(delta.table) > 0)
	{
		## find the appropriate delta
		oo <- which(delta.table[, "median FDR"] >= fdr.output)
		if (length(oo) > 0)
		{
			oo <- oo[length(oo)]
		} else
		{
			oo <- 1
		}
		
		## grab the right part of the the delta.table and get the significant gene table
		delta.table <- delta.table[oo : nrow(delta.table), , drop=FALSE]
		del <- delta.table[1, "delta"]
		siggenes.table <- samr.compute.siggenes.table(samr.obj, del, data, delta.table)
		
		## get the right range of table
		rang = 4 : 8
		if (resp.type == "Multiclass")
		{
			nclass = length(table(y))
			rang = 3 : (ncol(siggenes.table$genes.up))
		}
		if (resp.type == "Quantitative" | resp.type == "Survival")
		{
			rang = 4 : 7
		}
		
		siggenes.table$genes.up[, rang] <- round(as.numeric(siggenes.table$genes.up[, rang]), 3)
		siggenes.table$genes.lo[, rang] <- round(as.numeric(siggenes.table$genes.lo[, rang]), 3)
		
		tname <- colnames(siggenes.table$genes.up)
		siggenes.table$genes.up <- siggenes.table$genes.up[, 
			(tname != "Row") & (tname != "Numerator(r)") 
			& (tname != "Denominator(s+s0)")]
		tname <- colnames(siggenes.table$genes.lo)
		siggenes.table$genes.lo <- siggenes.table$genes.lo[, 
			(tname != "Row") & (tname != "Numerator(r)") 
			& (tname != "Denominator(s+s0)")]
	}

	## construct the return values
	out = list(samr.obj = samr.obj, del = del, delta.table = delta.table, siggenes.table = siggenes.table)
	out$call = this.call
	class(out) = "SAMoutput"
	
	return(out)
}

######################################################################
#		implementation of the generic function print
######################################################################
print.SAMoutput = function(x, ...)
{
	cat("Call:\n")
	dput(x$call)
	mat1 = x$siggenes.table$genes.up
	cat("", fill = T)
	cat("Genes up", fill = T)
	print(mat1, quote = FALSE)
	mat2 = x$siggenes.table$genes.lo
	cat("", fill = T)
	cat("Genes down", fill = T)
	print(mat2, quote = FALSE)
	invisible()
}

######################################################################
#		implementation of the generic funciton plot
######################################################################
plot.SAMoutput = function(x, ...)
{
	samr.plot(x$samr.obj, x$del)
	invisible()
} 

C Output from Public domain Ratfor, version 1.0
      subroutine rankcol (x,n,ip,ixr,iscrat)
      integer n,ip,iscrat(n),ixr(n,ip)
      double precision x(n,ip)
      do23000 j=1,ip
      call sortrx(n,x(1,j),iscrat)
      do23002 i=1,n
      ii=iscrat(i)
      ixr(ii,j)=i
23002 continue
23003 continue
23000 continue
23001 continue
      return
      end
C From Leonard J. Moss of SLAC:

C Here's a hybrid QuickSort I wrote a number of years ago.  It's
C based on suggestions in Knuth, Volume 3, and performs much better
C than a pure QuickSort on short or partially ordered input arrays.  

      SUBROUTINE SORTRX(N,DATA,INDEX)

C===================================================================
C
C     SORTRX -- SORT, Double Precision input, indeX output
C
C
C     Input:  N     INTEGER
C             DATA  DOUBLE PRECISION
C
C     Output: INDEX INTEGER (DIMENSION N)
C
C This routine performs an in-memory sort of the first N elements of
C array DATA, returning into array INDEX the indices of elements of
C DATA arranged in ascending order.  Thus,
C
C    DATA(INDEX(1)) will be the smallest number in array DATA;
C    DATA(INDEX(N)) will be the largest number in DATA.
C
C The original data is not physically rearranged.  The original order
C of equal input values is not necessarily preserved.
C
C===================================================================
C
C SORTRX uses a hybrid QuickSort algorithm, based on several
C suggestions in Knuth, Volume 3, Section 5.2.2.  In particular, the
C "pivot key" [my term] for dividing each subsequence is chosen to be
C the median of the first, last, and middle values of the subsequence;
C and the QuickSort is cut off when a subsequence has 9 or fewer
C elements, and a straight insertion sort of the entire array is done
C at the end.  The result is comparable to a pure insertion sort for
C very short arrays, and very fast for very large arrays (of order 12
C micro-sec/element on the 3081K for arrays of 10K elements).  It is
C also not subject to the poor performance of the pure QuickSort on
C partially ordered data.
C
C Created:  15 Jul 1986  Len Moss
C
C===================================================================
 
      INTEGER   N,INDEX(N)
      DOUBLE PRECISION      DATA(N)
 
      INTEGER   LSTK(31),RSTK(31),ISTK
      INTEGER   L,R,I,J,P,INDEXP,INDEXT
      DOUBLE PRECISION      DATAP
 
C     QuickSort Cutoff
C
C     Quit QuickSort-ing when a subsequence contains M or fewer
C     elements and finish off at end with straight insertion sort.
C     According to Knuth, V.3, the optimum value of M is around 9.
 
     INTEGER   M
      PARAMETER (M=9)
 
C===================================================================
C
C     Make initial guess for INDEX
 
      DO 50 I=1,N
         INDEX(I)=I
   50    CONTINUE
 
C     If array is short, skip QuickSort and go directly to
C     the straight insertion sort.
 
      IF (N.LE.M) GOTO 900
 
C===================================================================
C
C     QuickSort
C
C     The "Qn:"s correspond roughly to steps in Algorithm Q,
C     Knuth, V.3, PP.116-117, modified to select the median
C     of the first, last, and middle elements as the "pivot
C     key" (in Knuth's notation, "K").  Also modified to leave
C     data in place and produce an INDEX array.  To simplify
C     comments, let DATA[I]=DATA(INDEX(I)).
 
C Q1: Initialize
      ISTK=0
      L=1
      R=N
 
  200 CONTINUE
 
C Q2: Sort the subsequence DATA[L]..DATA[R].
C
C     At this point, DATA[l] <= DATA[m] <= DATA[r] for all l < L,
C     r > R, and L <= m <= R.  (First time through, there is no
C     DATA for l < L or r > R.)
 
      I=L
      J=R
 
C Q2.5: Select pivot key
C
C     Let the pivot, P, be the midpoint of this subsequence,
C     P=(L+R)/2; then rearrange INDEX(L), INDEX(P), and INDEX(R)
C     so the corresponding DATA values are in increasing order.
C     The pivot key, DATAP, is then DATA[P].
 
      P=(L+R)/2
      INDEXP=INDEX(P)
      DATAP=DATA(INDEXP)
 
      IF (DATA(INDEX(L)) .GT. DATAP) THEN
         INDEX(P)=INDEX(L)
         INDEX(L)=INDEXP
         INDEXP=INDEX(P)
         DATAP=DATA(INDEXP)
      ENDIF
 
      IF (DATAP .GT. DATA(INDEX(R))) THEN
         IF (DATA(INDEX(L)) .GT. DATA(INDEX(R))) THEN
            INDEX(P)=INDEX(L)
            INDEX(L)=INDEX(R)
         ELSE
            INDEX(P)=INDEX(R)
         ENDIF
         INDEX(R)=INDEXP
         INDEXP=INDEX(P)
         DATAP=DATA(INDEXP)
      ENDIF
 
C     Now we swap values between the right and left sides and/or
C     move DATAP until all smaller values are on the left and all
C     larger values are on the right.  Neither the left or right
C     side will be internally ordered yet; however, DATAP will be
C     in its final position.
 
  300 CONTINUE
 
C Q3: Search for datum on left >= DATAP
C
C     At this point, DATA[L] <= DATAP.  We can therefore start scanning
C     up from L, looking for a value >= DATAP (this scan is guaranteed
C     to terminate since we initially placed DATAP near the middle of
C     the subsequence).
 
         I=I+1
         IF (DATA(INDEX(I)).LT.DATAP) GOTO 300
 
  400 CONTINUE
 
C Q4: Search for datum on right <= DATAP
C
C     At this point, DATA[R] >= DATAP.  We can therefore start scanning
C     down from R, looking for a value <= DATAP (this scan is guaranteed
C     to terminate since we initially placed DATAP near the middle of
C     the subsequence).
 
         J=J-1
         IF (DATA(INDEX(J)).GT.DATAP) GOTO 400
 
C Q5: Have the two scans collided?
 
      IF (I.LT.J) THEN
 
C Q6: No, interchange DATA[I] <--> DATA[J] and continue
 
         INDEXT=INDEX(I)
         INDEX(I)=INDEX(J)
         INDEX(J)=INDEXT
         GOTO 300
      ELSE
 
C Q7: Yes, select next subsequence to sort
C
C     At this point, I >= J and DATA[l] <= DATA[I] == DATAP <= DATA[r],
C     for all L <= l < I and J < r <= R.  If both subsequences are
C     more than M elements long, push the longer one on the stack and
C     go back to QuickSort the shorter; if only one is more than M
C     elements long, go back and QuickSort it; otherwise, pop a
C     subsequence off the stack and QuickSort it.
 
         IF (R-J .GE. I-L .AND. I-L .GT. M) THEN
            ISTK=ISTK+1
            LSTK(ISTK)=J+1
            RSTK(ISTK)=R
            R=I-1
         ELSE IF (I-L .GT. R-J .AND. R-J .GT. M) THEN
            ISTK=ISTK+1
            LSTK(ISTK)=L
            RSTK(ISTK)=I-1
            L=J+1
         ELSE IF (R-J .GT. M) THEN
            L=J+1
         ELSE IF (I-L .GT. M) THEN
            R=I-1
         ELSE
C Q8: Pop the stack, or terminate QuickSort if empty
            IF (ISTK.LT.1) GOTO 900
            L=LSTK(ISTK)
            R=RSTK(ISTK)
            ISTK=ISTK-1
         ENDIF
         GOTO 200
      ENDIF
 
  900 CONTINUE
 
C===================================================================
C
C Q9: Straight Insertion sort
 
      DO 950 I=2,N
         IF (DATA(INDEX(I-1)) .GT. DATA(INDEX(I))) THEN
            INDEXP=INDEX(I)
            DATAP=DATA(INDEXP)
            P=I-1
  920       CONTINUE
               INDEX(P+1) = INDEX(P)
               P=P-1
               IF (P.GT.0) THEN
                  IF (DATA(INDEX(P)).GT.DATAP) GOTO 920
               ENDIF
            INDEX(P+1) = INDEXP
         ENDIF
  950    CONTINUE
 
C===================================================================
C
C     All done
 
      END

SAM = function(x, y = NULL, censoring.status = NULL, 
	resp.type = c("Quantitative", "Two class unpaired", "Survival", 
		"Multiclass", "One class", "Two class paired", "Two class unpaired timecourse", 
		"One class timecourse", "Two class paired timecourse", 
		"Pattern discovery"),  
	geneid = NULL, genenames = NULL, s0 = NULL, s0.perc = NULL, 
	nperms = 100, center.arrays = FALSE, testStatistic = c("standard", 
		"wilcoxon"), time.summary.type = c("slope", "signed.area"), 
	regression.method = c("standard", "ranks"), return.x = TRUE, 
	knn.neighbors = 10, random.seed = NULL,
	logged2 = FALSE, fdr.output = 0.20, eigengene.number = 1)
{
	this.call <- match.call()
	xl.mode = "regular"
	xl.time = NULL
	xl.prevfit = NULL
	
       if(fdr.output <0 | fdr.output>1){
        stop("Error: fdr.output must be between 0 and 1")
       }

	## set gene id and gene names
	if (is.null(geneid))
	{
		geneid = as.character(1:nrow(x))
	}
	if (is.null(genenames))
	{
		genenames = paste("g", as.character(1:nrow(x)), sep = "")
	}
	
	## construct the data list
	data = list(x = x, y = y, censoring.status = censoring.status, 
		geneid = geneid, genenames = genenames, logged2 = logged2, 
		eigengene.number = eigengene.number)
	
	## call samr with approprate parameters
	samr.obj = samr(data, resp.type = resp.type, assay.type = "array",
		s0 = s0, s0.perc = s0.perc, nperms = nperms, center.arrays = center.arrays, 
		testStatistic = testStatistic, time.summary.type = time.summary.type, 
		regression.method = regression.method, return.x = return.x, 
		knn.neighbors = knn.neighbors, random.seed = random.seed)
		
	## construct a full delta table with no fold change constraints
	delta.table <- samr.compute.delta.table(samr.obj)
	
	## get the significant gene table
	siggenes.table <- del <- NULL
	delta.table <- delta.table[delta.table[, "# called"] > 0, , drop=FALSE]
	if (nrow(delta.table) > 0)
	{
		## find the appropriate delta
		oo <- which(delta.table[, "median FDR"] >= fdr.output)
		if (length(oo) > 0)
		{
			oo <- oo[length(oo)]
		}
		else
		{
			oo <- 1
		}
		
		## grab the right part of the the delta.table and get the significant gene table
		delta.table <- delta.table[oo : nrow(delta.table), , drop=FALSE]
		del <- delta.table[1, "delta"]
		siggenes.table <- samr.compute.siggenes.table(samr.obj, del, data, delta.table)
		
		## get the right range of table
		rang = 4 : 8
		if (resp.type == "Multiclass")
		{
			nclass = length(table(y))
			rang = 3 : (ncol(siggenes.table$genes.up))
		}
		if (resp.type == "Quantitative" | resp.type == "Pattern discovery" | resp.type == "Survival")
		{
			rang = 4 : 7
		}
		
		siggenes.table$genes.up[, rang] = round(as.numeric(siggenes.table$genes.up[, rang]), 3)
		siggenes.table$genes.lo[, rang] = round(as.numeric(siggenes.table$genes.lo[, rang]), 3)
		siggenes.table$genes.up = siggenes.table$genes.up[, -1]
		siggenes.table$genes.lo = siggenes.table$genes.lo[, -1]
	}

	## construct the return values
	out = list(samr.obj = samr.obj, del = del, delta.table = delta.table, siggenes.table = siggenes.table)
	out$call = this.call
	class(out) = "SAMoutput"
	
	return(out)
}

print.SAMoutput = function(x, ...) {
	cat("Call:\n")
	dput(x$call)
	mat1 = x$siggenes.table$genes.up
	cat("", fill = T)
	cat("Genes up", fill = T)
	print(mat1, quote = FALSE)
	mat2 = x$siggenes.table$genes.lo
	cat("", fill = T)
	cat("Genes down", fill = T)
	print(mat2, quote = FALSE)
	invisible()
}

plot.SAMoutput = function(x, ...) {
	samr.plot(x$samr.obj, x$del)
	invisible()
} 

## Just for memory sake...
##samr.const.quantitative.response <- 0
##samr.const.twoclass.unpaired.response <- 1
##samr.const.survival.response <- 2
##samr.const.multiclass.response <- 3
##samr.const.oneclass.response <- 4
##samr.const.twoclass.paired.response <- 5
##samr.const.twoclass.unpaired.timecourse.response <- 6
##samr.const.oneclass.timecourse.response <- 7
##samr.const.twoclass.paired.timecourse.response <- 8


##SAM R associated functions
## individual functions for each response type

ttest.func <- function(x, y, s0 = 0, sd = NULL) {
	n1 <- sum(y == 1)
	n2 <- sum(y == 2)
	p <- nrow(x)
	m1 <- rowMeans(x[, y == 1, drop = F])
	m2 <- rowMeans(x[, y == 2, drop = F])
	if (is.null(sd)) {
		sd <- sqrt(((n2 - 1) * varr(x[, y == 2], meanx = m2) +
			(n1 - 1) * varr(x[, y == 1], meanx = m1)) * (1/n1 +
			1/n2)/(n1 + n2 - 2))
	}
	numer <- m2 - m1
	dif.obs <- (numer)/(sd + s0)
	return(list(tt = dif.obs, numer = numer, sd = sd))
}

wilcoxon.func <- function(x, y, s0 = 0) {
	n1 <- sum(y == 1)
	n2 <- sum(y == 2)
	p = nrow(x)
	r2 = rowSums(t(apply(x, 1, rank))[, y == 2, drop = F])
	numer = r2 - (n2/2) * (n2 + 1) - (n1 * n2)/2
	sd = sqrt(n1 * n2 * (n1 + n2 + 1)/12)
	tt = (numer)/(sd + s0)
	return(list(tt = tt, numer = numer, sd = rep(sd, p)))
}

onesample.ttest.func <- function(x, y, s0 = 0, sd = NULL) {
	n <- length(y)
	x <- x * matrix(y, nrow = nrow(x), ncol = ncol(x), byrow = TRUE)
	m <- rowMeans(x)
	if (is.null(sd)) {
		sd <- sqrt(varr(x, meanx = m)/n)
	}
	dif.obs <- m/(sd + s0)
	return(list(tt = dif.obs, numer = m, sd = sd))
}

patterndiscovery.func = function(x, s0 = 0, eigengene.number = 1) {
	a = mysvd(x, n.components = eigengene.number)
	v = a$v[, eigengene.number]
	# here we try to guess the most interpretable orientation
	#   for the eigengene
	om = abs(a$u[, eigengene.number]) > quantile(abs(a$u[, eigengene.number]),
		0.95)
	if (median(a$u[om, eigengene.number]) < 0) {
		v = -1 * v
	}
	aa = quantitative.func(x, v, s0 = s0)
	eigengene = cbind(1:nrow(a$v), v)
	dimnames(eigengene) = list(NULL, c("sample number", "value"))
	return(list(tt = aa$tt, numer = aa$numer, sd = aa$sd, eigengene = eigengene))
}

paired.ttest.func <- function(x, y, s0 = 0, sd = NULL) {
	nc <- ncol(x)/2
	o <- 1:nc
	o1 <- rep(0, ncol(x)/2)
	o2 <- o1
	for (j in 1:nc) {
		o1[j] <- (1:ncol(x))[y == -o[j]]
	}
	for (j in 1:nc) {
		o2[j] <- (1:ncol(x))[y == o[j]]
	}
	d <- x[, o2, drop = F] - x[, o1, drop = F]
	su <- x[, o2, drop = F] + x[, o1, drop = F]
	if (is.matrix(d)) {
		m <- rowMeans(d)
	}
	if (!is.matrix(d)) {
		m <- mean(d)
	}
	if (is.null(sd)) {
		if (is.matrix(d)) {
			sd <- sqrt(varr(d, meanx = m)/nc)
		}
		if (!is.matrix(d)) {
			sd <- sqrt(var(d)/nc)
		}
	}
	dif.obs <- m/(sd + s0)
	return(list(tt = dif.obs, numer = m, sd = sd))
}

cox.func <- function(x, y, censoring.status, s0 = 0) {
	# find the index matrix
	Dn <- sum(censoring.status == 1)
	Dset <- c(1:ncol(x))[censoring.status == 1]  # the set of observed
	ind <- matrix(0, ncol(x), Dn)
	# get the matrix
	for (i in 1:Dn) {
		ind[y > y[Dset[i]] - 1e-08, i] <- 1/sum(y > y[Dset[i]] -
			1e-08)
	}
	ind.sums <- rowSums(ind)
	x.ind <- x %*% ind
	# get the derivatives
	numer <- x %*% (censoring.status - ind.sums)
	sd <- sqrt((x * x) %*% ind.sums - rowSums(x.ind * x.ind))
	tt <- numer/(sd + s0)
	return(list(tt = tt, numer = numer, sd = sd))
}

multiclass.func <- function(x, y, s0 = 0) {
	##assumes y is coded 1,2...
	nn <- table(y)
	m <- matrix(0, nrow = nrow(x), ncol = length(nn))
	v <- m
	for (j in 1:length(nn)) {
		m[, j] <- rowMeans(x[, y == j])
		v[, j] <- (nn[j] - 1) * varr(x[, y == j], meanx = m[,
			j])
	}
	mbar <- rowMeans(x)
	mm <- m - matrix(mbar, nrow = length(mbar), ncol = length(nn))
	fac <- (sum(nn)/prod(nn))
	scor <- sqrt(fac * (apply(matrix(nn, nrow = nrow(m), ncol = ncol(m),
		byrow = TRUE) * mm * mm, 1, sum)))
	sd <- sqrt(rowSums(v) * (1/sum(nn - 1)) * sum(1/nn))
	tt <- scor/(sd + s0)
	mm.stand = t(scale(t(mm), center = FALSE, scale = sd))
	return(list(tt = tt, numer = scor, sd = sd, stand.contrasts = mm.stand))
}

#quantitative.func <- function(x,y,s0=0){
#  yy <- y-mean(y)
#  temp <- x%*%yy
#mx=rowMeans(x)
#sxx <-rowSums( (x-mx%*%t(rep(1,ncol(x))))^2 )
#
#  scor <- temp/sxx
#  b0hat <- mean(y)-scor*mx
# yhat <-
#   matrix(b0hat,nrow=nrow(x),ncol=ncol(x))+x*matrix(scor,nrow=nrow(x),ncol=ncol(x))
# ty <-
#   matrix(y,nrow=nrow(yhat),ncol=ncol(yhat),byrow=TRUE)
#  sigma <- sqrt(rowSums((ty-yhat)^2)/(ncol(yhat)-2))
#  sd <- sigma/sqrt(sxx)
#  tt <- scor/(sd+s0)
#  return(list(tt=tt, numer=scor, sd=sd))
#
#}

quantitative.func <- function(x, y, s0 = 0) {
	# regression of x on y
	my = mean(y)
	yy <- y - my
	temp <- x %*% yy
	mx = rowMeans(x)
	syy = sum(yy^2)
	scor <- temp/syy
	b0hat <- mx - scor * my
	ym = matrix(y, nrow = nrow(x), ncol = ncol(x), byrow = T)
	xhat <- matrix(b0hat, nrow = nrow(x), ncol = ncol(x)) + ym *
		matrix(scor, nrow = nrow(x), ncol = ncol(x))
	sigma <- sqrt(rowSums((x - xhat)^2)/(ncol(xhat) - 2))
	sd <- sigma/sqrt(syy)
	tt <- scor/(sd + s0)
	return(list(tt = tt, numer = scor, sd = sd))
}

timearea.func <- function(x, y, s0 = 0) {
	n <- ncol(x)
	xx <- 0.5 * (x[, 2:n] + x[, 1:(n - 1)]) * matrix(diff(y),
		nrow = nrow(x), ncol = n - 1, byrow = T)
	numer <- rowMeans(xx)
	sd <- sqrt(varr(xx, meanx = numer)/n)
	tt <- numer/sqrt(sd + s0)
	return(list(tt = tt, numer = numer, sd = sd))
}

#########################################
detec.slab <- function(samr.obj, del, min.foldchange) {
	## find genes above and below the slab of half-width del
	# this calculation is tricky- for consistency, the slab
	#   condition picks
	# all genes that are beyond the first departure from the
	#   slab
	# then the fold change condition is applied (if applicable)
	n <- length(samr.obj$tt)
	tt <- samr.obj$tt
	evo <- samr.obj$evo
	numer <- samr.obj$tt * (samr.obj$sd + samr.obj$s0)
	tag <- order(tt)
	pup <- NULL
	foldchange.cond.up = rep(T, length(evo))
	foldchange.cond.lo = rep(T, length(evo))
	if (!is.null(samr.obj$foldchange[1]) & (min.foldchange >
		0)) {
		foldchange.cond.up = samr.obj$foldchange >= min.foldchange
		foldchange.cond.lo = samr.obj$foldchange <= 1/min.foldchange
	}
	o1 <- (1:n)[(tt[tag] - evo > del) & evo > 0]
	if (length(o1) > 0) {
		o1 <- o1[1]
		o11 <- o1:n
		o111 <- rep(F, n)
		o111[tag][o11] <- T
		pup <- (1:n)[o111 & foldchange.cond.up]
	}
	plow <- NULL
	o2 <- (1:n)[(evo - tt[tag] > del) & evo < 0]
	if (length(o2) > 0) {
		o2 <- o2[length(o2)]
		o22 <- 1:o2
		o222 <- rep(F, n)
		o222[tag][o22] <- T
		plow <- (1:n)[o222 & foldchange.cond.lo]
	}
	return(list(plow = plow, pup = pup))
}

sumlengths <- function(aa) {
	length(aa$pl) + length(aa$pu)
}

## Jun added starts
samr.compute.delta.table <- function(samr.obj, min.foldchange = 0,
	dels = NULL, nvals = 50) {
	res <- NULL
	if (samr.obj$assay.type == "array") {
		res <- samr.compute.delta.table.array(samr.obj, min.foldchange,
			dels, nvals)
	}
	else if (samr.obj$assay.type == "seq") {
		res <- samr.compute.delta.table.seq(samr.obj, min.foldchange,
			dels)
	}
	return(res)
}
## Jun added ends

## Jun added the first row below, and commented the row
#   after it
samr.compute.delta.table.array <- function(samr.obj,
	min.foldchange = 0, dels = NULL, nvals = 50) {
	#samr.compute.delta.table <- function(samr.obj,
	#   min.foldchange=0, dels=NULL, nvals=50) {
	# computes delta table, starting with samr object 'a', for
	#   nvals values of delta
	lmax = sqrt(max(abs(sort(samr.obj$tt) - samr.obj$evo)))
	if (is.null(dels)) {
		dels = (seq(0, lmax, length = nvals)^2)
	}
	col = matrix(1, nrow = length(samr.obj$evo), ncol = nvals)
	ttstar0 <- samr.obj$ttstar0
	tt <- samr.obj$tt
	n <- samr.obj$n
	evo <- samr.obj$evo
	nsim <- ncol(ttstar0)
	res1 <- NULL
	foldchange.cond.up = matrix(T, nrow = nrow(samr.obj$ttstar),
		ncol = ncol(samr.obj$ttstar))
	foldchange.cond.lo = matrix(T, nrow = nrow(samr.obj$ttstar),
		ncol = ncol(samr.obj$ttstar))
	if (!is.null(samr.obj$foldchange[1]) & (min.foldchange >
		0)) {
		foldchange.cond.up = samr.obj$foldchange.star >= min.foldchange
		foldchange.cond.lo = samr.obj$foldchange.star <= 1/min.foldchange
	}
	cutup = rep(NA, length(dels))
	cutlow = rep(NA, length(dels))
	g2 = rep(NA, length(dels))
	errup = matrix(NA, ncol = length(dels), nrow = ncol(samr.obj$ttstar0))
	errlow = matrix(NA, ncol = length(dels), nrow = ncol(samr.obj$ttstar0))
	cat("", fill = T)
	cat("Computing delta table", fill = T)
	for (ii in 1:length(dels)) {
		cat(ii, fill = TRUE)
		ttt <- detec.slab(samr.obj, dels[ii], min.foldchange)
		cutup[ii] <- 1e+10
		if (length(ttt$pup > 0)) {
			cutup[ii] <- min(samr.obj$tt[ttt$pup])
		}
		cutlow[ii] <- -1e+10
		if (length(ttt$plow) > 0) {
			cutlow[ii] <- max(samr.obj$tt[ttt$plow])
		}
		g2[ii] = sumlengths(ttt)
		errup[, ii] = colSums(samr.obj$ttstar0 > cutup[ii] &
			foldchange.cond.up)
		errlow[, ii] = colSums(samr.obj$ttstar0 < cutlow[ii] &
			foldchange.cond.lo)
	}
	s <- sqrt(apply(errup, 2, var)/nsim + apply(errlow, 2, var)/nsim)
	gmed <- apply(errup + errlow, 2, median)
	g90 = apply(errup + errlow, 2, quantile, 0.9)
	res1 <- cbind(samr.obj$pi0 * gmed, samr.obj$pi0 * g90, g2,
		samr.obj$pi0 * gmed/g2, samr.obj$pi0 * g90/g2, cutlow,
		cutup)
	res1 <- cbind(dels, res1)
	# remove rows with #called=0
	#om=res1[,4]==0
	#res1=res1[!om,,drop=F]
	# remove duplicate rows with same # of genes called
	#omm=!duplicated(res1[,4])
	#res1=res1[omm,,drop=F]
	dimnames(res1) <- list(NULL, c("delta", "# med false pos",
		"90th perc false pos", "# called", "median FDR", "90th perc FDR",
		"cutlo", "cuthi"))
	return(res1)
}

detec.horiz <- function(samr.obj, cutlow, cutup, min.foldchange) {
	## find genes above or below horizontal cutpoints
	dobs <- samr.obj$tt
	n <- length(dobs)
	foldchange.cond.up = rep(T, n)
	foldchange.cond.lo = rep(T, n)
	if (!is.null(samr.obj$foldchange[1]) & (min.foldchange >
		0)) {
		foldchange.cond.up = samr.obj$foldchange >= min.foldchange
		foldchange.cond.lo = samr.obj$foldchange <= 1/min.foldchange
	}
	pup <- (1:n)[dobs > cutup & foldchange.cond.up]
	plow <- (1:n)[dobs < cutlow & foldchange.cond.lo]
	return(list(plow = plow, pup = pup))
}

samr.plot <- function(samr.obj, del = NULL, min.foldchange = 0) {
	## make observed-expected plot
	## takes foldchange into account too
	if (is.null(del)) {
		del = sqrt(max(abs(sort(samr.obj$tt) - samr.obj$evo)))
	}
	LARGE = 1e+10
	b <- detec.slab(samr.obj, del, min.foldchange)
	bb <- c(b$pup, b$plow)
	b1 = LARGE
	b0 = -LARGE
	if (!is.null(b$pup)) {
		b1 <- min(samr.obj$tt[b$pup])
	}
	if (!is.null(b$plow)) {
		b0 <- max(samr.obj$tt[b$plow])
	}
	c1 <- (1:samr.obj$n)[sort(samr.obj$tt) >= b1]
	c0 <- (1:samr.obj$n)[sort(samr.obj$tt) <= b0]
	c2 <- c(c0, c1)
	foldchange.cond.up = rep(T, length(samr.obj$evo))
	foldchange.cond.lo = rep(T, length(samr.obj$evo))
	if (!is.null(samr.obj$foldchange[1]) & (min.foldchange >
		0)) {
		foldchange.cond.up = samr.obj$foldchange >= min.foldchange
		foldchange.cond.lo = samr.obj$foldchange <= 1/min.foldchange
	}
	col = rep(1, length(samr.obj$evo))
	col[b$plow] = 3
	col[b$pup] = 2
	if (!is.null(samr.obj$foldchange[1]) & (min.foldchange >
		0)) {
		col[!foldchange.cond.lo & !foldchange.cond.up] = 1
	}
	col.ordered = col[order(samr.obj$tt)]
	ylims <- range(samr.obj$tt)
	xlims <- range(samr.obj$evo)
	plot(samr.obj$evo, sort(samr.obj$tt), xlab = "expected score",
		ylab = "observed score", ylim = ylims, xlim = xlims,
		type = "n")
	points(samr.obj$evo, sort(samr.obj$tt), col = col.ordered)
	abline(0, 1)
	abline(del, 1, lty = 2)
	abline(-del, 1, lty = 2)
}

localfdr <- function(samr.obj, min.foldchange, perc = 0.01,
	df = 10) {
	## estimates compute.localfdr at score 'd', using SAM
	#   object 'samr.obj'
	## 'd' can be a vector of d scores
	## returns estimate of symmetric fdr  as a percentage
	# this version uses a 1% symmetric window, and does not
	#   estimate fdr in
	# windows  having fewer than 100 genes
	## to use: first run samr and then pass the resulting fit
	#   object to
	## localfdr
	## NOTE: at most 20 of the perms are used to estimate the
	#   fdr (for speed sake)
	# I try two window shapes: symmetric and an assymetric one
	# currently I use the symmetric window to estimate the
	#   compute.localfdr
	ngenes = length(samr.obj$tt)
	mingenes = 50
	# perc is increased, in order to get at least mingenes in a
	#   window
	perc = max(perc, mingenes/length(samr.obj$tt))
	nperms.to.use = min(20, ncol(samr.obj$ttstar))
	nperms = ncol(samr.obj$ttstar)
	d = seq(sort(samr.obj$tt)[1], sort(samr.obj$tt)[ngenes],
		length = 100)
	ndscore <- length(d)
	dvector <- rep(NA, ndscore)
	ind.foldchange = rep(T, length(samr.obj$tt))
	if (!is.null(samr.obj$foldchange[1]) & min.foldchange > 0) {
		ind.foldchange = (samr.obj$foldchange >= min.foldchange) |
			(samr.obj$foldchange <= min.foldchange)
	}
	fdr.temp = function(temp, dlow, dup, pi0, ind.foldchange) {
		return(sum(pi0 * (temp >= dlow & temp <= dup & ind.foldchange)))
	}
	for (i in 1:ndscore) {
		pi0 <- samr.obj$pi0
		r <- sum(samr.obj$tt < d[i])
		r22 <- round(max(r - length(samr.obj$tt) * perc/2, 1))
		dlow.sym <- sort(samr.obj$tt)[r22]
		#      if(d[i]<0)
		#       {
		#         r2 <- max(r-length(samr.obj$tt)*perc/2, 1)
		# r22= min(r+length(samr.obj$tt)*perc/2,
		#   length(samr.obj$tt))
		#
		#          dlow <- sort(samr.obj$tt)[r2]
		#          dup=sort(samr.obj$tt)[r22]
		#       }
		r22 <- min(r + length(samr.obj$tt) * perc/2, length(samr.obj$tt))
		dup.sym <- sort(samr.obj$tt)[r22]
		#     if(d[i]>0)
		#      {
		# r2 <- min(r+length(samr.obj$tt)*perc/2,
		#   length(samr.obj$tt))
		#        r22 <- max(r-length(samr.obj$tt)*perc/2, 1)
		#        dup <- sort(samr.obj$tt)[r2]
		#        dlow <- sort(samr.obj$tt)[r22]
		#
		#       }
		# o <- samr.obj$tt>=dlow & samr.obj$tt<= dup &
		#   ind.foldchange
		oo <- samr.obj$tt >= dlow.sym & samr.obj$tt <= dup.sym &
			ind.foldchange
		nsim <- ncol(samr.obj$ttstar)
		fdr <- rep(NA, nsim)
		fdr2 <- fdr
		if (!is.null(samr.obj$foldchange[1]) & min.foldchange >
			0) {
			temp = as.vector(samr.obj$foldchange.star[, 1:nperms.to.use])
			ind.foldchange = (temp >= min.foldchange) | (temp <=
				min.foldchange)
		}
		temp = samr.obj$ttstar0[, sample(1:nperms, size = nperms.to.use)]
		# fdr <-median(apply(temp,2,fdr.temp,dlow, dup, pi0,
		#   ind.foldchange))
		fdr.sym <- median(apply(temp, 2, fdr.temp, dlow.sym,
			dup.sym, pi0, ind.foldchange))
		#      fdr <- 100*fdr/sum(o)
		fdr.sym <- 100 * fdr.sym/sum(oo)
		dlow.sym <- dlow.sym
		dup.sym <- dup.sym
		dvector[i] <- fdr.sym
	}
	om = !is.na(dvector) & (dvector != Inf)
	aa = smooth.spline(d[om], dvector[om], df = df)
	return(list(smooth.object = aa, perc = perc, df = df))
}

predictlocalfdr = function(smooth.object, d) {
	yhat = predict(smooth.object, d)$y
	yhat = pmin(yhat, 100)
	yhat = pmax(yhat, 0)
	return(yhat)
}

samr.compute.siggenes.table = function(samr.obj, del,
	data, delta.table, min.foldchange = 0, all.genes = FALSE,
	compute.localfdr = FALSE)
{
	## computes significant genes table, starting with samr
	#   object 'a' and 'delta.table'
	##  for a  **single** value del
	## if all.genes is true, all genes are printed (and value
	#   of del is ignored)
	if (is.null(data$geneid))
	{
		data$geneid = paste("g", 1:nrow(data$x), sep = "")
	}
	if (is.null(data$genenames))
	{
		data$genenames = paste("g", 1:nrow(data$x), sep = "")
	}
	if (!all.genes)
	{
		sig = detec.slab(samr.obj, del, min.foldchange)
	}
	if (all.genes)
	{
		p = length(samr.obj$tt)
		pup = (1:p)[samr.obj$tt >= 0]
		plo = (1:p)[samr.obj$tt < 0]
		sig = list(pup = pup, plo = plo)
	}
	if (compute.localfdr)
	{
		aa = localfdr(samr.obj, min.foldchange)
		if (length(sig$pup) > 0)
		{
			fdr.up = predictlocalfdr(aa$smooth.object, samr.obj$tt[sig$pup])
		}
		if (length(sig$plo) > 0)
		{
			fdr.lo = predictlocalfdr(aa$smooth.object, samr.obj$tt[sig$plo])
		}
	}
	qvalues = NULL
	if (length(sig$pup) > 0 | length(sig$plo) > 0)
	{
		qvalues = qvalue.func(samr.obj, sig, delta.table)
	}
	res.up = NULL
	res.lo = NULL
	done = FALSE

	# two class unpaired or paired  (foldchange is reported)
	if ((samr.obj$resp.type == samr.const.twoclass.unpaired.response |
		samr.obj$resp.type == samr.const.twoclass.paired.response))
	{
		if (!is.null(sig$pup))
		{
			res.up = cbind(sig$pup + 1, data$genenames[sig$pup],
				data$geneid[sig$pup], samr.obj$tt[sig$pup], samr.obj$numer[sig$pup],
				samr.obj$sd[sig$pup], samr.obj$foldchange[sig$pup],
				qvalues$qvalue.up)
			if (compute.localfdr)
			{
				res.up = cbind(res.up, fdr.up)
			}
			temp.names = list(NULL, c("Row", "Gene ID", "Gene Name",
				"Score(d)", "Numerator(r)", "Denominator(s+s0)",
				"Fold Change", "q-value(%)"))
			if (compute.localfdr)
			{
				temp.names[[2]] = c(temp.names[[2]], "localfdr(%)")
			}
			dimnames(res.up) = temp.names
		}
		if (!is.null(sig$plo))
		{
			res.lo = cbind(sig$plo + 1, data$genenames[sig$plo],
				data$geneid[sig$plo], samr.obj$tt[sig$plo], samr.obj$numer[sig$plo],
				samr.obj$sd[sig$plo], samr.obj$foldchange[sig$plo],
				qvalues$qvalue.lo)
			if (compute.localfdr)
			{
				res.lo = cbind(res.lo, fdr.lo)
			}
			temp.names = list(NULL, c("Row", "Gene ID", "Gene Name",
				"Score(d)", "Numerator(r)", "Denominator(s+s0)",
				"Fold Change", "q-value(%)"))
			if (compute.localfdr)
			{
				temp.names[[2]] = c(temp.names[[2]], "localfdr(%)")
			}
			dimnames(res.lo) = temp.names
		}
		done = TRUE
	}

	# multiclass
	if (samr.obj$resp.type == samr.const.multiclass.response)
	{
		if (!is.null(sig$pup))
		{
			res.up = cbind(sig$pup + 1, data$genenames[sig$pup],
			data$geneid[sig$pup], samr.obj$tt[sig$pup], samr.obj$numer[sig$pup],
			samr.obj$sd[sig$pup], samr.obj$stand.contrasts[sig$pup, ], qvalues$qvalue.up)

			if (compute.localfdr)
			{
				res.up = cbind(res.up, fdr.up)
			}

			collabs.contrast = paste("contrast-", as.character(1:ncol(samr.obj$stand.contrasts)),
				sep = "")
			temp.names = list(NULL, c("Row", "Gene ID", "Gene Name",
			"Score(d)", "Numerator(r)", "Denominator(s+s0)",
			collabs.contrast, "q-value(%)"))

			if (compute.localfdr)
			{
				temp.names[[2]] = c(temp.names[[2]], "localfdr(%)")
			}
			dimnames(res.up) = temp.names
		}
		res.lo = NULL
		done = TRUE
	}

	#all other cases
	if (!done)
	{
		if (!is.null(sig$pup))
		{
			res.up = cbind(sig$pup + 1, data$genenames[sig$pup],
				data$geneid[sig$pup], samr.obj$tt[sig$pup], samr.obj$numer[sig$pup],
				samr.obj$sd[sig$pup], samr.obj$foldchange[sig$pup],
				qvalues$qvalue.up)
			if (compute.localfdr)
			{
				res.up = cbind(res.up, fdr.up)
			}
			temp.names = list(NULL, c("Row", "Gene ID", "Gene Name",
				"Score(d)", "Numerator(r)", "Denominator(s+s0)",
				"q-value(%)"))
			if (compute.localfdr)
			{
				temp.names[[2]] = c(temp.names[[2]], "localfdr(%)")
			}
			dimnames(res.up) = temp.names
		}
		if (!is.null(sig$plo))
		{
			res.lo = cbind(sig$plo + 1, data$genenames[sig$plo],
				data$geneid[sig$plo], samr.obj$tt[sig$plo], samr.obj$numer[sig$plo],
				samr.obj$sd[sig$plo], samr.obj$foldchange[sig$plo],
				qvalues$qvalue.lo)
			if (compute.localfdr)
			{
				res.lo = cbind(res.lo, fdr.lo)
			}
			temp.names = list(NULL, c("Row", "Gene ID", "Gene Name",
				"Score(d)", "Numerator(r)", "Denominator(s+s0)",
				"q-value(%)"))
			if (compute.localfdr)
			{
				temp.names[[2]] = c(temp.names[[2]], "localfdr(%)")
			}
			dimnames(res.lo) = temp.names
		}
		done = TRUE
	}
	if (!is.null(res.up))
	{
		o1 = order(-samr.obj$tt[sig$pup])
		res.up = res.up[o1, , drop = F]
	}
	if (!is.null(res.lo))
	{
		o2 = order(samr.obj$tt[sig$plo])
		res.lo = res.lo[o2, , drop = F]
	}
	color.ind.for.multi = NULL
	if (samr.obj$resp.type == samr.const.multiclass.response & !is.null(sig$pup))
	{
		color.ind.for.multi = 1 * (samr.obj$stand.contrasts[sig$pup,
			] > samr.obj$stand.contrasts.95[2]) + (-1) * (samr.obj$stand.contrasts[sig$pup,
			] < samr.obj$stand.contrasts.95[1])
	}
	ngenes.up = nrow(res.up)
	if (is.null(ngenes.up))
	{
		ngenes.up = 0
	}
	ngenes.lo = nrow(res.lo)
	if (is.null(ngenes.lo))
	{
		ngenes.lo = 0
	}
	return(list(genes.up = res.up, genes.lo = res.lo, color.ind.for.multi = color.ind.for.multi,
		ngenes.up = ngenes.up, ngenes.lo = ngenes.lo))
}

qvalue.func = function(samr.obj, sig, delta.table) {
	# returns q-value as a percentage (out of 100)
	LARGE = 1e+10
	qvalue.up = rep(NA, length(sig$pup))
	o1 = sig$pup
	cutup = delta.table[, 8]
	FDR = delta.table[, 5]
	ii = 0
	for (i in o1) {
		o = abs(cutup - samr.obj$tt[i])
		o[is.na(o)] = LARGE
		oo = (1:length(o))[o == min(o)]
		oo = oo[length(oo)]
		ii = ii + 1
		qvalue.up[ii] = FDR[oo]
	}
	qvalue.lo = rep(NA, length(sig$plo))
	o2 = sig$plo
	cutlo = delta.table[, 7]
	ii = 0
	for (i in o2) {
		o = abs(cutlo - samr.obj$tt[i])
		o[is.na(o)] = LARGE
		oo = (1:length(o))[o == min(o)]
		oo = oo[length(oo)]
		ii = ii + 1
		qvalue.lo[ii] = FDR[oo]
	}
	# any qvalues that are missing, are set to 1 (the highest
	#   value)
	qvalue.lo[is.na(qvalue.lo)] = 1
	qvalue.up[is.na(qvalue.up)] = 1
	# ensure that each qvalue vector is monotone non-increasing
	o1 = order(samr.obj$tt[sig$plo])
	qv1 = qvalue.lo[o1]
	qv11 = qv1
	if (length(qv1) > 1) {
		for (i in 2:length(qv1)) {
			if (qv11[i] < qv11[i - 1]) {
				qv11[i] = qv11[i - 1]
			}
		}
		qv111 = qv11
		qv111[o1] = qv11
	}
	else {
		qv111 = qv1
	}
	o2 = order(samr.obj$tt[sig$pup])
	qv2 = qvalue.up[o2]
	qv22 = qv2
	if (length(qv2) > 1) {
		for (i in 2:length(qv2)) {
			if (qv22[i] > qv22[i - 1]) {
				qv22[i] = qv22[i - 1]
			}
		}
		qv222 = qv22
		qv222[o2] = qv22
	}
	else {
		qv222 = qv2
	}
	return(list(qvalue.lo = 100 * qv111, qvalue.up = 100 * qv222))
}

foldchange.twoclass = function(x, y, logged2) {
	#  if(logged2){x=2^x}
	m1 <- rowMeans(x[, y == 1, drop = F])
	m2 <- rowMeans(x[, y == 2, drop = F])
	if (!logged2) {
		fc = m2/m1
	}
	if (logged2) {
		fc = 2^{
			m2 - m1
		}
	}
	return(fc)
}

foldchange.paired = function(x, y, logged2) {
	#  if(logged2){x=2^x}
	nc <- ncol(x)/2
	o <- 1:nc
	o1 <- rep(0, ncol(x)/2)
	o2 <- o1
	for (j in 1:nc) {
		o1[j] <- (1:ncol(x))[y == -o[j]]
	}
	for (j in 1:nc) {
		o2[j] <- (1:ncol(x))[y == o[j]]
	}
	if (!logged2) {
		d <- x[, o2, drop = F]/x[, o1, drop = F]
	}
	if (logged2) {
		d <- x[, o2, drop = F] - x[, o1, drop = F]
	}
	if (!logged2) {
		fc <- rowMeans(d)
	}
	if (logged2) {
		fc <- 2^rowMeans(d)
	}
	return(fc)
}

est.s0 <- function(tt, sd, s0.perc = seq(0, 1, by = 0.05)) {
	## estimate s0 (exchangeability) factor for denominator.
	## returns the actual estimate s0 (not a percentile)
	br = unique(quantile(sd, seq(0, 1, len = 101)))
	nbr = length(br)
	a <- cut(sd, br, labels = F)
	a[is.na(a)] <- 1
	cv.sd <- rep(0, length(s0.perc))
	for (j in 1:length(s0.perc)) {
		w <- quantile(sd, s0.perc[j])
		w[j == 1] <- 0
		tt2 <- tt * sd/(sd + w)
		tt2[tt2 == Inf] = NA
		sds <- rep(0, nbr - 1)
		for (i in 1:(nbr - 1)) {
			sds[i] <- mad(tt2[a == i], na.rm = TRUE)
		}
		cv.sd[j] <- sqrt(var(sds))/mean(sds)
	}
	o = (1:length(s0.perc))[cv.sd == min(cv.sd)]
	# we don;t allow taking s0.hat to be 0th percentile when
	#   min sd is 0
	s0.hat = quantile(sd[sd != 0], s0.perc[o])
	return(list(s0.perc = s0.perc, cv.sd = cv.sd, s0.hat = s0.hat))
}

samr.missrate <- function(samr.obj, del, delta.table,
	quant = NULL) {
	# returns miss rate as a percentage
	if (is.null(quant)) {
		if (samr.obj$resp.type != samr.const.multiclass.response) {
			quant = c(0, 0.05, 0.1, 0.15, 0.2, 0.25, 0.75, 0.8,
				0.85, 0.9, 0.95, 1)
		}
		if (samr.obj$resp.type == samr.const.multiclass.response) {
			quant = c(0.75, 0.8, 0.85, 0.9, 0.95, 1)
		}
	}
	## estimate miss rate from sam object 'a'
	o = abs(delta.table[, 1] - del)
	oo = (1:nrow(delta.table))[o == min(o)]
	cut.lo = delta.table[oo, 7]
	cut.up = delta.table[oo, 8]
	ooo = samr.obj$tt > cut.lo & samr.obj$tt < cut.up
	cuts = quantile(samr.obj$tt[ooo], quant)
	ncuts <- length(cuts)
	ngenes <- rep(NA, ncuts)
	ngenes0 <- rep(NA, ncuts)
	ngenes2 <- rep(NA, ncuts)
	missrate <- rep(NA, ncuts)
	nperm = ncol(samr.obj$ttstar)
	for (j in 1:(ncuts - 1)) {
		ngenes2[j] <- sum(samr.obj$tt > cuts[j] & samr.obj$tt <
			cuts[j + 1])
		ngenes0[j] <- sum(samr.obj$ttstar > cuts[j] & samr.obj$ttstar <
			cuts[j + 1])/nperm
		missrate[j] <- (ngenes2[j] - samr.obj$pi0 * ngenes0[j])/ngenes2[j]
		missrate[j] <- max(missrate[j], 0)
	}
	cuts = round(cuts, 3)
	res = matrix(NA, ncol = 3, nrow = ncuts - 1)
	missrate = round(missrate, 4)
	for (i in 1:(ncuts - 1)) {
		res[i, 1] = paste(as.character(quant[i]), as.character(quant[i +
			1]), sep = " -> ")
		res[i, 2] = paste(as.character(cuts[i]), as.character(cuts[i +
			1]), sep = " -> ")
		res[i, 3] = 100 * missrate[i]
	}
	dimnames(res) = list(NULL, c("Quantiles", "Cutpoints", "Miss Rate(%)"))
	return(res)
}

varr <- function(x, meanx = NULL) {
	n <- ncol(x)
	p <- nrow(x)
	Y <- matrix(1, nrow = n, ncol = 1)
	if (is.null(meanx)) {
		meanx <- rowMeans(x)
	}
	ans <- rep(1, p)
	xdif <- x - meanx %*% t(Y)
	ans <- (xdif^2) %*% rep(1/(n - 1), n)
	ans <- drop(ans)
	return(ans)
}

insert.value <- function(vec, newval, pos) {
	if (pos == 1)
		return(c(newval, vec))
	lvec <- length(vec)
	if (pos > lvec)
		return(c(vec, newval))
	return(c(vec[1:pos - 1], newval, vec[pos:lvec]))
}

permute <- function(elem) {
	# generates all perms of the vector elem
	if (!missing(elem)) {
		if (length(elem) == 2)
			return(matrix(c(elem, elem[2], elem[1]), nrow = 2))
		last.matrix <- permute(elem[-1])
		dim.last <- dim(last.matrix)
		new.matrix <- matrix(0, nrow = dim.last[1] * (dim.last[2] +
			1), ncol = dim.last[2] + 1)
		for (row in 1:(dim.last[1])) {
			for (col in 1:(dim.last[2] + 1)) new.matrix[row +
				(col - 1) * dim.last[1], ] <- insert.value(last.matrix[row,
				], elem[1], col)
		}
		return(new.matrix)
	}
	else cat("Usage: permute(elem)\n\twhere elem is a vector\n")
}

sample.perms <- function(elem, nperms) {
	# randomly generates  nperms of the vector elem
	res = permute.rows(matrix(elem, nrow = nperms, ncol = length(elem),
		byrow = T))
	return(res)
}

integer.base.b <- function(x, b = 2) {
	xi <- as.integer(x)
	if (xi == 0) {
		return(0)
	}
	if (any(is.na(xi) | ((x - xi) != 0)))
		print(list(ERROR = "x not integer", x = x))
	N <- length(x)
	xMax <- max(x)
	ndigits <- (floor(logb(xMax, base = 2)) + 1)
	Base.b <- array(NA, dim = c(N, ndigits))
	for (i in 1:ndigits) {
		#i <- 1
		Base.b[, ndigits - i + 1] <- (x%%b)
		x <- (x%/%b)
	}
	if (N == 1)
		Base.b[1, ]
	else Base.b
}

compute.block.perms = function(y, blocky, nperms) {
	# y are the data (eg class label 1 vs 2; or -1,1, -2,2 for
	#   paired data)
	# blocky are the block labels (abs(y) for paired daatr)
	ny = length(y)
	nblocks = length(unique(blocky))
	tab = table(blocky)
	total.nperms = prod(factorial(tab))
	# block.perms is a list of all possible permutations
	block.perms = vector("list", nblocks)
	# first enumerate all perms, when possible
	if (total.nperms <= nperms) {
		all.perms.flag = 1
		nperms.act = total.nperms
		for (i in 1:nblocks) {
			block.perms[[i]] = permute(y[blocky == i])
		}
		kk = 0:(factorial(max(tab))^nblocks - 1)
		#the rows of the matrix outerm runs through the 'outer
		#   product'
		# first we assume that all blocks have max(tab) members;
		#   then we remove rows of outerm that
		#  are illegal (ie when a block has fewer members)
		outerm = matrix(0, nrow = length(kk), ncol = nblocks)
		for (i in 1:length(kk)) {
			kkkk = integer.base.b(kk[i], b = factorial(max(tab)))
			if (length(kkkk) > nblocks) {
				kkkk = kkkk[(length(kkkk) - nblocks + 1):length(kkkk)]
			}
			outerm[i, (nblocks - length(kkkk) + 1):nblocks] = kkkk
		}
		outerm = outerm + 1
		# now remove rows that are illegal perms
		ind = rep(TRUE, nrow(outerm))
		for (j in 1:ncol(outerm)) {
			ind = ind & outerm[, j] <= factorial(tab[j])
		}
		outerm = outerm[ind, , drop = F]
		# finally, construct permutation matrix from outer product
		permsy = matrix(NA, nrow = total.nperms, ncol = ny)
		for (i in 1:total.nperms) {
			junk = NULL
			for (j in 1:nblocks) {
				junk = c(junk, block.perms[[j]][outerm[i, j],
				  ])
			}
			permsy[i, ] = junk
		}
	}
	# next handle case when there are too many perms to
	#   enumerate
	if (total.nperms > nperms) {
		all.perms.flag = 0
		nperms.act = nperms
		permsy = NULL
		block.perms = vector("list", nblocks)
		for (j in 1:nblocks) {
			block.perms[[j]] = sample.perms(y[blocky == j], nperms = nperms)
		}
		for (j in 1:nblocks) {
			permsy = cbind(permsy, block.perms[[j]])
		}
	}
	return(list(permsy = permsy, all.perms.flag = all.perms.flag,
		nperms.act = nperms.act))
}

getperms = function(y, nperms) {
	total.perms = factorial(length(y))
	if (total.perms <= nperms) {
		perms = permute(1:length(y))
		all.perms.flag = 1
		nperms.act = total.perms
	}
	if (total.perms > nperms) {
		perms = matrix(NA, nrow = nperms, ncol = length(y))
		for (i in 1:nperms) {
			perms[i, ] = sample(1:length(y), size = length(y))
		}
		all.perms.flag = 0
		nperms.act = nperms
	}
	return(list(perms = perms, all.perms.flag = all.perms.flag,
		nperms.act = nperms.act))
}

parse.block.labels.for.2classes = function(y) {
	#this only works for 2 class case- having form jBlockn,
	#   where j=1 or 2
	n = length(y)
	y.act = rep(NA, n)
	blocky = rep(NA, n)
	for (i in 1:n) {
		blocky[i] = as.numeric(substring(y[i], 7, nchar(y[i])))
		y.act[i] = as.numeric(substring(y[i], 1, 1))
	}
	return(list(y.act = y.act, blocky = blocky))
}

parse.time.labels.and.summarize.data = function(x,
	y, resp.type, time.summary.type) {
	# parse time labels, and summarize time data for each
	#   person, via a slope or area
	# does some error checking too
	n = length(y)
	last5char = rep(NA, n)
	last3char = rep(NA, n)
	for (i in 1:n) {
		last3char[i] = substring(y[i], nchar(y[i]) - 2, nchar(y[i]))
		last5char[i] = substring(y[i], nchar(y[i]) - 4, nchar(y[i]))
	}
	if (sum(last3char == "End") != sum(last5char == "Start")) {
		stop("Error in format of  time course data: a Start or End tag is missing")
	}
	y.act = rep(NA, n)
	timey = rep(NA, n)
	person.id = rep(NA, n)
	k = 1
	end.flag = FALSE
	person.id[1] = 1
	if (substring(y[1], nchar(y[1]) - 4, nchar(y[1])) != "Start") {
		stop("Error in format of  time course data: first cell should have a Start tag")
	}
	for (i in 1:n) {
		cat(i)
		j = 1
		while (substring(y[i], j, j) != "T") {
			j = j + 1
		}
		end.of.y = j - 1
		y.act[i] = as.numeric(substring(y[i], 1, end.of.y))
		timey[i] = substring(y[i], end.of.y + 5, nchar(y[i]))
		if (nchar(timey[i]) > 3 & substring(timey[i], nchar(timey[i]) -
			2, nchar(timey[i])) == "End") {
			end.flag = TRUE
			timey[i] = substring(timey[i], 1, nchar(timey[i]) -
				3)
		}
		if (nchar(timey[i]) > 3 & substring(timey[i], nchar(timey[i]) -
			4, nchar(timey[i])) == "Start") {
			timey[i] = substring(timey[i], 1, nchar(timey[i]) -
				5)
		}
		if (i < n & !end.flag) {
			person.id[i + 1] = k
		}
		if (i < n & end.flag) {
			k = k + 1
			person.id[i + 1] = k
		}
		end.flag = FALSE
	}
	timey = as.numeric(timey)
	# do a check that the format was correct
	tt = table(person.id, y.act)
	junk = function(x) {
		sum(x != 0)
	}
	if (sum(apply(tt, 1, junk) != 1) > 0) {
		num = (1:nrow(tt))[apply(tt, 1, junk) > 1]
		stop(paste("Error in format of  time course data, timecourse #",
			as.character(num)))
	}
	npeople = length(unique(person.id))
	newx = matrix(NA, nrow = nrow(x), ncol = npeople)
	sd = matrix(NA, nrow = nrow(x), ncol = npeople)
	for (j in 1:npeople) {
		jj = person.id == j
		tim = timey[jj]
		xc = t(scale(t(x[, jj, drop = F]), center = TRUE, scale = FALSE))
		if (time.summary.type == "slope") {
			junk = quantitative.func(xc, tim - mean(tim))
			newx[, j] = junk$numer
			sd[, j] = junk$sd
		}
		if (time.summary.type == "signed.area") {
			junk = timearea.func(x[, jj, drop = F], tim)
			newx[, j] = junk$numer
			sd[, j] = junk$sd
		}
	}
	y.unique = y.act[!duplicated(person.id)]
	return(list(y = y.unique, x = newx, sd = sd))
}

check.format = function(y, resp.type, censoring.status = NULL) {
	# here i do some format checks for the input data$y
	# note that checks for time course data are done in the
	#   parse function for time course;
	#  we then check the output from the parser in this function
	if (resp.type == samr.const.twoclass.unpaired.response |
		resp.type == samr.const.twoclass.unpaired.timecourse.response) {
		if (sum(y == 1) + sum(y == 2) != length(y)) {
			stop(paste("Error in input response data: response type ",
				resp.type, " specified; values must be 1 or 2"))
		}
	}
	if (resp.type == samr.const.twoclass.paired.response | resp.type ==
		samr.const.twoclass.paired.timecourse.response) {
		if (sum(y) != 0) {
			stop(paste("Error in input response data: response type ",
				resp.type, " specified; values must be -1, 1, -2, 2, etc"))
		}
		if (sum(table(y[y > 0]) != abs(table(y[y < 0])))) {
			stop(paste("Error in input response data:  response type ",
				resp.type, " specified; values must be -1, 1, -2, 2, etc"))
		}
	}
	if (resp.type == samr.const.oneclass.response | resp.type ==
		samr.const.oneclass.timecourse.response) {
		if (sum(y == 1) != length(y)) {
			stop(paste("Error in input response data: response type ",
				resp.type, " specified;  values must all be 1"))
		}
	}
	if (resp.type == samr.const.multiclass.response) {
		tt = table(y)
		nc = length(tt)
		if (sum(y <= nc & y > 0) < length(y)) {
			stop(paste("Error in input response data: response type ",
				resp.type, " specified; values must be 1,2, ... number of classes"))
		}
		for (k in 1:nc) {
			if (sum(y == k) < 2) {
				stop(paste("Error in input response data: response type ",
				  resp.type, " specified; there must be >1 sample per class"))
			}
		}
	}
	if (resp.type == samr.const.quantitative.response) {
		if (!is.numeric(y)) {
			stop(paste("Error in input response data: response type",
				resp.type, " specified; values must be numeric"))
		}
	}
	if (resp.type == samr.const.survival.response) {
		if (is.null(censoring.status)) {
			stop(paste("Error in input response data: response type ",
				resp.type, " specified; error in censoring indicator"))
		}
		if (!is.numeric(y) | sum(y < 0) > 0) {
			stop(paste("Error in input response data:  response type ",
				resp.type, " specified; survival times  must be numeric and nonnegative"))
			if (sum(censoring.status == 0) + sum(censoring.status ==
				1) != length(censoring.status)) {
				stop(paste("Error in input response data: response type ",
				  resp.type, " specified; censoring indicators must be 0 (censored) or 1 (failed)"))
			}
		}
		if (sum(censoring.status == 1) < 1) {
			stop(paste("Error in input response data:   response type ",
				resp.type, " specified; there are no uncensored observations"))
		}
	}
	return()
}

mysvd <- function(x, n.components = NULL) {
	# finds PCs of matrix x
	p <- nrow(x)
	n <- ncol(x)
	# center the observations (rows)
	feature.means <- rowMeans(x)
	x <- t(scale(t(x), center = feature.means, scale = F))
	if (is.null(n.components)) {
		n.components = min(n, p)
	}
	if (p > n) {
		a <- eigen(t(x) %*% x)
		v <- a$vec[, 1:n.components, drop = FALSE]
		d <- sqrt(a$val[1:n.components, drop = FALSE])
		u <- scale(x %*% v, center = FALSE, scale = d)
		return(list(u = u, d = d, v = v))
	}
	else {
		junk <- svd(x, LINPACK = TRUE)
		nc = min(ncol(junk$u), n.components)
		return(list(u = junk$u[, 1:nc], d = junk$d[1:nc], v = junk$v[,
			1:nc]))
	}
}

permute.rows <- function(x) {
	dd <- dim(x)
	n <- dd[1]
	p <- dd[2]
	mm <- runif(length(x)) + rep(seq(n) * 10, rep(p, n))
	matrix(t(x)[order(mm)], n, p, byrow = TRUE)
}

samr.assess.samplesize = function(samr.obj, data,
	dif, samplesize.factors = c(1, 2, 3, 5), min.genes = 10,
	max.genes = nrow(data$x)/2) {
	if (length(samplesize.factors) > 4) {
		stop("Length of samplesize.factors must be less than or equal to 4")
	}
	n.ssf = length(samplesize.factors)
	if (samr.obj$resp.type != samr.const.twoclass.unpaired.response &
		samr.obj$resp.type != samr.const.twoclass.paired.response &
		samr.obj$resp.type != samr.const.oneclass.response &
		samr.obj$resp.type != samr.const.survival.response) {
		stop("Function only implemented for  twoclass.unpaired, twoclass.paired,\noneclass and survival data types")
	}
	m = nrow(data$x)
	n = ncol(data$x)
	if (samr.obj$resp.type == samr.const.twoclass.unpaired.response) {
		n1 = sum(data$y == 1)
		n2 = sum(data$y == 2)
	}
	if (samr.obj$resp.type == samr.const.twoclass.paired.response) {
		n1 = n/2
		n2 = n/2
	}
	nreps = 3
	klist = round(exp(seq(log(min.genes), log(max.genes), length = 10)))
	#power=rep(NA,length(klist))
	#type1=rep(NA,length(klist))
	fdr = matrix(NA, nrow = length(klist), ncol = n.ssf)
	fdr90 = matrix(NA, nrow = length(klist), ncol = n.ssf)
	fdr10 = matrix(NA, nrow = length(klist), ncol = n.ssf)
	fnr = matrix(NA, nrow = length(klist), ncol = n.ssf)
	fnr90 = matrix(NA, nrow = length(klist), ncol = n.ssf)
	fnr10 = matrix(NA, nrow = length(klist), ncol = n.ssf)
	cutp = matrix(NA, nrow = length(klist), ncol = n.ssf)
	#sd=samr.obj$sd-samr.obj$s0
	sd = samr.obj$sd
	if (samr.obj$resp.type == samr.const.twoclass.unpaired.response |
		samr.obj$resp.type == samr.const.twoclass.paired.response) {
		sigma = sd/sqrt(1/n1 + 1/n2)
		difm = dif/(sigma * sqrt(1/n1 + 1/n2))
	}
	if (samr.obj$resp.type == samr.const.oneclass.response |
		samr.obj$resp.type == samr.const.survival.response) {
		sigma = (sqrt(n)) * sd
		difm = sqrt(n) * dif/sigma
	}
	# we only use the  20 of the perms, for speed
	nperms = min(20, samr.obj$nperms.act)
	perms.to.use = sample(1:samr.obj$nperms.act, size = nperms)
	#note: here I permute within each col of zstar, so that the
	# genes that are modified are different for each
	#   permutation
	zstar0 = t(permute.rows(t(samr.obj$ttstar0[, perms.to.use])))
	ii = 0
	for (k in klist) {
		ii = ii + 1
		oo = sample(1:m, size = k)
		temp = matrix(F, nrow = nrow(zstar0), ncol = ncol(zstar0))
		temp[oo, ] = T
		for (kk in 1:n.ssf) {
			zstar = zstar0
			zstar[oo, ] = zstar[oo, ] + difm[oo] * sqrt(samplesize.factors[kk])
			cutp[ii, kk] = quantile(abs(zstar), 1 - (k/m))
			temp[oo, ] = T
			#
			#   power[ii]=(sum(abs(zstar[oo,])>cutp[ii,kk])/samr.obj$nperms)/k
			#
			#   type1[ii]=(sum(abs(zstar[-oo,])>cutp[ii,kk])/samr.obj$nperms)/(m-k)
			u0 = colSums(abs(zstar) > cutp[ii, kk] & !temp)
			r0 = colSums(abs(zstar) > cutp[ii, kk])
			oo2 = !is.na(u0/r0)
			fdr[ii, kk] = median((u0/r0)[oo2])
			fdr90[ii, kk] = quantile((u0/r0)[oo2], 0.9)
			fdr10[ii, kk] = quantile((u0/r0)[oo2], 0.1)
			v0 = colSums(abs(zstar) < cutp[ii, kk] & temp)
			w0 = colSums(abs(zstar) < cutp[ii, kk])
			oo3 = !is.na(v0/w0)
			fnr[ii, kk] = median((v0/w0)[oo3])
			fnr90[ii, kk] = quantile((v0/w0)[oo3], 0.9)
			fnr10[ii, kk] = quantile((v0/w0)[oo3], 0.1)
		}
	}
	ng = round(klist)
	oo = !duplicated(ng)
	results = array(NA, c(sum(oo), 8, n.ssf))
	for (kk in 1:n.ssf) {
		results[, , kk] = cbind(ng, cutp[, kk], fdr[, kk], fdr90[,
			kk], fdr10[, kk], fnr[, kk], fnr90[, kk], fnr10[,
			kk])[oo, ]
	}
	dimnames(results) = list(NULL, c("number of genes", "cutpoint",
		"FDR,1-power", "FDR90", "FDR10", "FNR,type1 error", "FNR90",
		"FNR10"), as.character(samplesize.factors))
	return(list(results = results, dif.call = dif, difm = mean(difm),
		samplesize.factors = samplesize.factors, n = ncol(data$x)))
}

samr.assess.samplesize.plot <- function(samr.assess.samplesize.obj,
	logx = TRUE) {
	n.ssf = length(samr.assess.samplesize.obj$samplesize.factors)
	if (n.ssf == 1) {
		par(mfrow = c(1, 1))
	}
	if (n.ssf == 2) {
		par(mfrow = c(1, 2))
	}
	if (n.ssf > 2) {
		par(mfrow = c(2, 2))
	}
	par(oma = c(0, 0, 2, 0))
	na.min = function(x) {
		min(x[!is.na(x)])
	}
	na.max = function(x) {
		max(x[!is.na(x)])
	}
	temp = samr.assess.samplesize.obj$results
	ymax = max(c(temp[, "FDR,1-power", ], temp[, "FDR90", ],
		temp[, "FDR10", ], temp[, "FNR,type1 error", ], temp[,
			"FDR90", ], temp[, "FDR10", ]))
	for (kk in 1:n.ssf) {
		results = samr.assess.samplesize.obj$results[, , kk]
		if (logx) {
			plot(results[, "number of genes"], results[, "FDR,1-power"],
				log = "x", xlab = "Number of genes", ylab = "",
				type = "n", ylim = c(0, ymax))
		}
		if (!logx) {
			plot(results[, "number of genes"], results[, "FDR,1-power"],
				xlab = "Number of genes", ylab = "", type = "n",
				ylim = c(0, ymax))
		}
		lines(results[, "number of genes"], results[, "FDR,1-power"],
			col = 2, type = "b", pch = 19)
		lines(results[, "number of genes"], results[, "FDR90"],
			col = 2, lty = 2, pch = 19)
		lines(results[, "number of genes"], results[, "FDR10"],
			col = 2, lty = 2, pch = 19)
		lines(results[, "number of genes"], results[, "FNR,type1 error"],
			col = 3, type = "b", pch = 19)
		lines(results[, "number of genes"], results[, "FNR90"],
			col = 3, lty = 2, pch = 19)
		lines(results[, "number of genes"], results[, "FNR10"],
			col = 3, lty = 2, pch = 19)
		mtext("FDR, 1-Power", side = 2, col = 2, cex = 0.8)
		mtext("FNR, Type 1 error", side = 4, col = 3, cex = 0.8)
		abline(h = 0.05, lty = 3)
		fac = samr.assess.samplesize.obj$samplesize.factors[kk]
		n = samr.assess.samplesize.obj$n
		title(paste("Sample size=", round(n * fac, 0)), cex = 0.7)
	}
	title(paste("Results for mean difference=", round(samr.assess.samplesize.obj$dif.call,
		2)), outer = T)
	return()
}

samr.pvalues.from.perms = function(tt, ttstar) {
	r = rank(c(abs(tt), abs(as.vector(ttstar))))[1:length(tt)]
	r2 = rank(c(abs(tt)))
	r3 = r - r2
	pv = (length(tt) - r3/ncol(ttstar) + 1)/length(tt)
	return(pv)
}

samr.tail.strength = function(samr.obj) {
	tt = samr.obj$tt
	ttstar = samr.obj$ttstar0
	pv = samr.pvalues.from.perms(tt, ttstar)
	m = length(pv)
	pvs = sort(pv)
	ts = (1/m) * sum((1 - pvs * (m + 1)/(1:m)))
	res = NULL
	nperms = min(ncol(ttstar), 20)
	ttstar.temp = ttstar[, 1:nperms]
	for (i in 1:nperms) {
		cat(i, fill = T)
		pvstar = samr.pvalues.from.perms(ttstar.temp[, i], ttstar.temp[,
			-i])
		pvstar = sort(pvstar)
		tsstar = (1/m) * sum((1 - pvstar * (m + 1)/(1:m)))
		res = c(res, tsstar)
	}
	se.ts.perm = sqrt(var(res))
	return(list(ts = ts, se.ts = se.ts.perm))
}

## quant <- function(x) {
## 	#dyn.load('/home/tibs/PAPERS/copa/quant.so')
## 	p = as.integer(nrow(x))
## 	n = as.integer(ncol(x))
## 	xx = t(x)
## 	storage.mode(xx) = "single"
## 	junk = .Fortran("quant", as.matrix(xx), n, p, integer(n),
## 		out = single(2 * p), PACKAGE = "samr")
## 	return(matrix(junk$out, ncol = 2))
## }

################
#new functions for SAMseq
######################################################################
#\t\tEstimate sequencing depths
#\tArguments:
#\t\tx: data matrix. nrow=#gene, ncol=#sample
#\tValue:
# depth: estimated sequencing depth. a vector with len
#   #sample.
######################################################################
samr.estimate.depth <- function(x) {
	iter <- 5
	cmeans <- colSums(x)/sum(x)
	for (i in 1:iter) {
		n0 <- rowSums(x) %*% t(cmeans)
		prop <- rowSums((x - n0)^2/(n0 + 1e-08))
		qs <- quantile(prop, c(0.25, 0.75))
		keep <- (prop >= qs[1]) & (prop <= qs[2])
		cmeans <- colMeans(x[keep, ])
		cmeans <- cmeans/sum(cmeans)
	}
	depth <- cmeans/mean(cmeans)
	return(depth)
}

######################################################################
#\t\tResampling
#\tArguments:
#\t\tx: data matrix. nrow=#gene, ncol=#sample
#\t\td: estimated sequencing depth
#\t\tnresamp: number of resamplings
#\tValue:
#\t\txresamp: an rank array with dim #gene*#sample*nresamp
######################################################################
resample <- function(x, d, nresamp = 20) {
	ng <- nrow(x)
	ns <- ncol(x)
	dbar <- exp(mean(log(d)))
	xresamp <- array(0, dim = c(ng, ns, nresamp))
	for (k in 1:nresamp) {
		for (j in 1:ns) {
			xresamp[, j, k] <- rpois(n = ng, lambda = (dbar/d[j]) *
				x[, j]) + runif(ng) * 0.1
		}
	}
	for (k in 1:nresamp) {
		xresamp[, , k] <- t(rankcol(t(xresamp[, , k])))
	}
	return(xresamp)
}

######################################################################
#\t\tTwoclass Wilcoxon statistics
#\tArguments:
#\t\txresamp: an rank array with dim #gene*#sample*nresamp
#\t\ty: outcome vector of values 1 and 2
#\tValue:
#\t\ttt: the statistic.
######################################################################
#
wilcoxon.unpaired.seq.func <- function(xresamp, y) {
	tt <- rep(0, dim(xresamp)[1])
	for (i in 1:dim(xresamp)[3]) {
		tt <- tt + rowSums(xresamp[, y == 2, i]) - sum(y == 2) *
			(length(y) + 1)/2
	}
	tt <- tt/dim(xresamp)[3]
	return(list(tt = tt, numer = tt, sd = rep(1, length(tt))))
}

######################################################################
#\t\tTwoclass paired Wilcoxon statistics
#\tArguments:
#\t\txresamp: an rank array with dim #gene*#sample*nresamp
#\t\ty: outcome vector of values +1, -1, +2, -2, ...
#\tValue:
#\t\ttt: the statistic.
######################################################################
wilcoxon.paired.seq.func <- function(xresamp, y) {
	tt <- rep(0, dim(xresamp)[1])
	for (i in 1:dim(xresamp)[3]) {
		tt <- tt + rowSums(xresamp[, y > 0, i]) - sum(y > 0) *
			(length(y) + 1)/2
	}
	tt <- tt/dim(xresamp)[3]
	return(list(tt = tt, numer = tt, sd = rep(1, length(tt))))
}

######################################################################
#\t\tMulticlass statistics
#\tArguments:
#\t\txresamp: an rank array with dim #gene*#sample*nresamp
#\t\ty: outcome vector of values 1, 2, ..., K
#\tValue:
#\t\ttt: the statistic.
######################################################################
multiclass.seq.func <- function(xresamp, y)
{
	# number of classes and number of samples in each class
	K <- max(y)
	n.each <- rep(0, K)
	for (k in 1 : K)
	{
		n.each[k] <- sum(y == k)
	}
	# the statistic
	tt <- temp <- rep(0, dim(xresamp)[1])
	stand.contrasts <- matrix(0, dim(xresamp)[1], K)

	for (i in 1 : dim(xresamp)[3])
	{
		for (k in 1 : K)
		{
			temp <- rowSums(xresamp[, y == k, i])
			tt <- tt + temp ^2 / n.each[k]
			stand.contrasts[, k] <- stand.contrasts[, k] + temp
		}
	}
	# finalize
	nresamp <- dim(xresamp)[3]
	ns <- dim(xresamp)[2]
	tt <- tt / nresamp * 12 / ns / (ns + 1) - 3 * (ns + 1)
	stand.contrasts <- stand.contrasts / nresamp
	stand.contrasts <- scale(stand.contrasts, center=n.each * (ns + 1) / 2,
		scale=sqrt(n.each * (ns - n.each) * (ns + 1) / 12))
	return(list(tt = tt, numer = tt, sd = rep(1, length(tt)),
		stand.contrasts = stand.contrasts))
}

## Jun commented this function
#######################################################################
##\t\tQuantitative statistics
##\tArguments:
##\t\txresamp: an rank array with dim #gene*#sample*nresamp
##\t\ty: outcome vector of real values
##\tValue:
##\t\ttt: the statistic.
#######################################################################
#quantitative.seq.func <- function(xresamp, y)
#{
#\ty.ranked <- rank(y) - (dim(xresamp)[2] + 1) / 2
#
#\ttt <- rep(0, dim(xresamp)[1])
#
#\tfor (i in 1 : dim(xresamp)[3])
#\t{
# tt <- tt + (xresamp[, , i] - (dim(xresamp)[2] + 1) / 2)
#   %*% y.ranked
#\t}
#
#\tns <- dim(xresamp)[2]
#\ttt <- tt / (dim(xresamp)[3] * (ns ^ 3 - ns) / 12)
#
#
#   return(list(tt=as.vector(tt),numer=as.vector(tt),sd=rep(1,length(tt))))
#}

## Jun added starts
######################################################################
#\t\tQuantitative statistics
#\tArguments:
#\t\txresamp: an rank array with dim #gene*#sample*nresamp
#\t\ty: outcome vector of real values
#\tValue:
#\t\ttt: the statistic.
######################################################################
quantitative.seq.func <- function(xresamp, y) {
	tt <- rep(0, dim(xresamp)[1])
	for (i in 1:dim(xresamp)[3]) {
		y.ranked <- rank(y, ties.method = "random") - (dim(xresamp)[2] +
			1)/2
		tt <- tt + (xresamp[, , i] - (dim(xresamp)[2] + 1)/2) %*%
			y.ranked
	}
	ns <- dim(xresamp)[2]
	tt <- tt/(dim(xresamp)[3] * (ns^3 - ns)/12)
	return(list(tt = as.vector(tt), numer = as.vector(tt), sd = rep(1,
		length(tt))))
}
## Jun added ends

######################################################################
#\t\tSurvival statistics
#\tArguments:
#\t\txresamp: an rank array with dim #gene*#sample*nresamp
#\t\ty: outcome vector of real values
#\t\tcensoring.status: 1=died, 0=censored
#\tValue:
#\t\ttt: the statistic.
######################################################################
cox.seq.func <- function(xresamp, y, censoring.status) {
	# get the dimensions
	ng <- dim(xresamp)[1]
	ns <- dim(xresamp)[2]
	# prepare for the calculation
	# find the index matrix
	Dn <- sum(censoring.status == 1)
	Dset <- c(1:ns)[censoring.status == 1]  # the set of died
	ind <- matrix(0, ns, Dn)
	# get the matrix
	for (i in 1:Dn) {
		ind[y >= y[Dset[i]] - 1e-08, i] <- 1/sum(y >= y[Dset[i]] -
			1e-08)
	}
	ind.sums <- rowSums(ind)
	# calculate the score statistic
	tt <- apply(xresamp, 3, function(x, cen.ind, ind.para, ind.sums.para) {
		dev1 <- x %*% cen.ind
		x.ind <- x %*% ind.para
		dev2 <- (x * x) %*% ind.sums.para - rowSums(x.ind * x.ind)
		dev1/(sqrt(dev2) + 1e-08)
	}, (censoring.status - ind.sums), ind, ind.sums)
	tt <- rowMeans(tt)
	return(list(tt = tt, numer = tt, sd = rep(1, length(tt))))
}

rankcol = function(x) {
	# ranks the elements within each col of the matrix x
	# and returns these ranks in a matrix
	n = nrow(x)
	p = ncol(x)
	mode(n) = "integer"
	mode(p) = "integer"
	mode(x) = "double"
	if (!is.loaded("rankcol")) {
		#dyn.load('/home/tibs/PAPERS/jun2/test/rankcol.so')
	}
	junk = .Fortran("rankcol", x, n, ip = p, ixr = integer(n * p),
		integer(n), PACKAGE = "samr")
	xr = matrix(junk$ixr, nrow = n, ncol = p)
	return(xr)
}

## Jun added starts
######################################################################
#\t\tfoldchange of twoclass unpaired sequencing data
######################################################################
foldchange.seq.twoclass.unpaired <- function(x, y, depth)
{
	x.norm <- scale(x, center = F, scale = depth) + 1e-08
	fc <- rowMedians(x.norm[, y == 2])/rowMedians(x.norm[, y ==
		1])
	return(fc)
}

######################################################################
#\t\tfoldchange of twoclass paired sequencing data
######################################################################
foldchange.seq.twoclass.paired <- function(x, y, depth) {
	nc <- ncol(x)/2
	o1 <- o2 <- rep(0, nc)
	for (j in 1:nc) {
		o1[j] <- which(y == -j)
		o2[j] <- which(y == j)
	}
	x.norm <- scale(x, center = F, scale = depth) + 1e-08
	d <- x.norm[, o2, drop = F]/x.norm[, o1, drop = F]
	fc <- rowMedians(d, na.rm = T)
	return(fc)
}
## Jun added ends

## Jun added starts
#######################################################################
#\tcompute the delta table for sequencing data
#######################################################################
samr.compute.delta.table.seq <- function(samr.obj,
	min.foldchange = 0, dels = NULL) {
	res1 <- NULL
	flag <- T
	## check whether any gene satisfies the foldchange
	#   restrictions
	if ((samr.obj$resp.type == samr.const.twoclass.unpaired.response |
		samr.obj$resp.type == samr.const.twoclass.paired.response) &
		(min.foldchange > 0)) {
		sat.up <- (samr.obj$foldchange >= min.foldchange) & (samr.obj$evo >
			0)
		sat.dn <- (samr.obj$foldchange <= 1/min.foldchange) &
			(samr.obj$evo < 0)
		if (sum(sat.up) + sum(sat.dn) == 0) {
			flag <- F
		}
	}
	if (flag) {
		if (is.null(dels)) {
			dels <- generate.dels(samr.obj, min.foldchange = min.foldchange)
		}
		cat("Number of thresholds chosen (all possible thresholds) =",
			length(dels), fill = T)
		if (length(dels) > 0) {
			## sort delta to make the fast calculation right
			dels <- sort(dels)
			## get the upper and lower cutoffs
			cat("Getting all the cutoffs for the thresholds...\n")
			slabs <- samr.seq.detec.slabs(samr.obj, dels, min.foldchange)
			cutup <- slabs$cutup
			cutlow <- slabs$cutlow
			g2 <- slabs$g2
			## get the number of errors under the null hypothesis
			cat("Getting number of false positives in the permutation...\n")
			errnum <- samr.seq.null.err(samr.obj, min.foldchange,
				cutup, cutlow)
			res1 <- NULL
			gmed <- apply(errnum, 2, median)
			g90 = apply(errnum, 2, quantile, 0.9)
			res1 <- cbind(samr.obj$pi0 * gmed, samr.obj$pi0 *
				g90, g2, samr.obj$pi0 * gmed/g2, samr.obj$pi0 *
				g90/g2, cutlow, cutup)
			res1 <- cbind(dels, res1)
			dimnames(res1) <- list(NULL, c("delta", "# med false pos",
				"90th perc false pos", "# called", "median FDR",
				"90th perc FDR", "cutlo", "cuthi"))
		}
	}
	return(res1)
}

######################################################################
#\tget the number of significance in the null distribution
######################################################################
samr.seq.null.err <- function(samr.obj, min.foldchange,
	cutup, cutlow) {
	errup = matrix(NA, ncol = length(cutup), nrow = ncol(samr.obj$ttstar0))
	errlow = matrix(NA, ncol = length(cutlow), nrow = ncol(samr.obj$ttstar0))
	cutup.rank <- rank(cutup, ties.method = "min")
	cutlow.rank <- rank(-cutlow, ties.method = "min")
	for (jj in 1:ncol(samr.obj$ttstar0)) {
		#cat(jj, fill=TRUE)
		keep.up <- keep.dn <- samr.obj$ttstar0[, jj]
		if ((samr.obj$resp.type == samr.const.twoclass.unpaired.response |
			samr.obj$resp.type == samr.const.twoclass.paired.response) &
			(min.foldchange > 0)) {
			keep.up <- keep.up[samr.obj$foldchange.star[, jj] >=
				min.foldchange]
			keep.dn <- keep.dn[samr.obj$foldchange.star[, jj] <=
				1/min.foldchange]
		}
		errup[jj, ] <- length(keep.up) - (rank(c(cutup, keep.up),
			ties.method = "min")[1:length(cutup)] - cutup.rank)
		errlow[jj, ] <- length(keep.dn) - (rank(c(-cutlow, -keep.dn),
			ties.method = "min")[1:length(cutlow)] - cutlow.rank)
	}
	errnum <- errup + errlow
	return(errnum)
}

######################################################################
#\tdetect multiple slabs
######################################################################
samr.seq.detec.slabs <- function(samr.obj, dels, min.foldchange) {
	## initialize calculation
	tag <- order(samr.obj$tt)
	if ((samr.obj$resp.type == samr.const.twoclass.unpaired.response |
		samr.obj$resp.type == samr.const.twoclass.paired.response) &
		(min.foldchange > 0)) {
		res.mat <- data.frame(tt = samr.obj$tt[tag], fc = samr.obj$foldchange[tag],
			evo = samr.obj$evo, dif = samr.obj$tt[tag] - samr.obj$evo)
		res.up <- res.mat[res.mat$evo > 0, ]
		res.lo <- res.mat[res.mat$evo < 0, ]
		res.up <- res.up[res.up$fc >= min.foldchange, ]
		res.lo <- res.lo[res.lo$fc <= 1/min.foldchange, ]
	}
	else {
		res.mat <- data.frame(tt = samr.obj$tt[tag], evo = samr.obj$evo,
			dif = samr.obj$tt[tag] - samr.obj$evo)
		res.up <- res.mat[res.mat$evo > 0, ]
		res.lo <- res.mat[res.mat$evo < 0, ]
	}
	## begin calculating
	cutup <- rep(1e+10, length(dels))
	cutlow <- rep(-1e+10, length(dels))
	g2.up <- g2.lo <- rep(0, length(dels))
	if (nrow(res.up) > 0) {
		res.up <- data.frame(dif = res.up$dif, tt = res.up$tt,
			num = nrow(res.up):1)
		## get the upper part
		j <- 1
		ii <- 1
		while (j <= nrow(res.up) & ii <= length(dels)) {
			if (res.up$dif[j] > dels[ii]) {
				cutup[ii] <- res.up$tt[j]
				g2.up[ii] <- res.up$num[j]
				ii <- ii + 1
			}
			else {
				j <- j + 1
			}
		}
	}
	if (nrow(res.lo) > 0) {
		res.lo <- data.frame(dif = res.lo$dif, tt = res.lo$tt,
			num = 1:nrow(res.lo))
		## get the lower part
		j <- nrow(res.lo)
		ii <- 1
		while (j >= 1 & ii <= length(dels)) {
			if (res.lo$dif[j] < -dels[ii]) {
				cutlow[ii] <- res.lo$tt[j]
				g2.lo[ii] <- res.lo$num[j]
				ii <- ii + 1
			}
			else {
				j <- j - 1
			}
		}
	}
	g2 <- g2.up + g2.lo
	return(list(cutup = cutup, cutlow = cutlow, g2 = g2))
}

#######################################################################
#\tcompute the delta table for sequencing data
#######################################################################
generate.dels <- function(samr.obj, min.foldchange = 0) {
	dels <- NULL
	## initialize calculation
	tag <- order(samr.obj$tt)
	if ((samr.obj$resp.type == samr.const.twoclass.unpaired.response |
		samr.obj$resp.type == samr.const.twoclass.paired.response) &
		(min.foldchange > 0)) {
		res.mat <- data.frame(tt = samr.obj$tt[tag], fc = samr.obj$foldchange[tag],
			evo = samr.obj$evo, dif = samr.obj$tt[tag] - samr.obj$evo)
		res.up <- res.mat[res.mat$evo > 0, ]
		res.lo <- res.mat[res.mat$evo < 0, ]
		res.up <- res.up[res.up$fc >= min.foldchange, ]
		res.lo <- res.lo[res.lo$fc <= 1/min.foldchange, ]
	}
	else {
		res.mat <- data.frame(tt = samr.obj$tt[tag], evo = samr.obj$evo,
			dif = samr.obj$tt[tag] - samr.obj$evo)
		res.up <- res.mat[res.mat$evo > 0, ]
		res.lo <- res.mat[res.mat$evo < 0, ]
	}
	## for the upper part
	up.vec <- rep(NA, nrow(res.up))
	if (nrow(res.up) > 0) {
		st <- 1e-08
		i.cur <- 1
		for (i in 1:nrow(res.up)) {
			if (res.up$dif[i] > st) {
				st <- res.up$dif[i]
				up.vec[i.cur] <- st
				i.cur <- i.cur + 1
			}
		}
	}
	## for the lower part
	lo.vec <- rep(NA, nrow(res.lo))
	if (nrow(res.lo) > 0) {
		st <- -1e-08
		i.cur <- 1
		for (i in nrow(res.lo):1) {
			if (res.lo$dif[i] < st) {
				st <- res.lo$dif[i]
				lo.vec[i.cur] <- st
				i.cur <- i.cur + 1
			}
		}
	}
	## combine them
	vec <- c(up.vec, -lo.vec)
	vec <- vec[!is.na(vec)]
	vec <- vec - 1e-08
	dels <- sort(unique(vec))
	return(dels)
}

#######################################################################
#\tgenerate normalized data
#######################################################################
samr.norm.data <- function(x, depth = NULL) {
	if (is.null(depth)) {
		depth <- samr.estimate.depth(x)
	}
	scaling.factors <- (prod(depth)^(1/length(depth)))/depth
	x <- 2 * sqrt(x + 3/8)
	x <- scale(x, center = F, scale = sqrt(1/scaling.factors))
	return(x)
}
## Jun added ends

## Shiny webapp function

runSAM <- function() shiny::runApp(system.file("webapp", package="samr"))

# for robustness of code, we define constants to be used
#   later
samr.const.quantitative.response <- "Quantitative"
samr.const.twoclass.unpaired.response <- "Two class unpaired"
samr.const.survival.response <- "Survival"
samr.const.multiclass.response <- "Multiclass"
samr.const.oneclass.response <- "One class"
samr.const.twoclass.paired.response <- "Two class paired"
samr.const.twoclass.unpaired.timecourse.response <- "Two class unpaired timecourse"
samr.const.oneclass.timecourse.response <- "One class timecourse"
samr.const.twoclass.paired.timecourse.response <- "Two class paired timecourse"
samr.const.patterndiscovery.response <- "Pattern discovery"
samr.const.red.color <- 3
samr.const.green.color <- 10
samr.const.black.color <- 1
# Note: the table samr.xl.data.types created by the
# the Excel calling program has exactly these names as well
samr <- function(data, resp.type = c("Quantitative",
	"Two class unpaired", "Survival", "Multiclass", "One class",
	"Two class paired", "Two class unpaired timecourse", "One class timecourse",
	"Two class paired timecourse", "Pattern discovery"), assay.type = c("array",
	"seq"), s0 = NULL, s0.perc = NULL, nperms = 100, center.arrays = FALSE,
	testStatistic = c("standard", "wilcoxon"), time.summary.type = c("slope",
		"signed.area"), regression.method = c("standard", "ranks"),
	return.x = FALSE,
	knn.neighbors = 10, random.seed = NULL,
	nresamp = 20, nresamp.perm = NULL, xl.mode = c("regular",
		"firsttime", "next20", "lasttime"), xl.time = NULL, xl.prevfit = NULL) {
	##SAM method. copyright june 2000: Goss, Tibshirani and
	#   Chu.
	## coded by r tibshirani;
	## y is response measure: 1,2 for two twoclass groups,
	## y=1,1,1 for onesample problem,
	## -1, 1,2-,2 for paired groups, with -1 paired with 1 etc
	## or survival time, or categorical for unordered groups
	#   1,2,3,..
	## quantitative for continuous ordered y#
	## timecourse data is also handled. In addition, pattern
	#   discovery is available- in which the eigengenes
	##  are used as a quantitative outcome
	##
	##
	## s0 is the exchangeability factor; you can specify
	## s0 as an actual value
	## s0.perc, the percentile of sd values to use for s0
	## or if both s0 and s0.perc are null (the default), then
	#   s0 is automatically estimated
	## returns
	##  evo= expected order statistics (length p=# of genes)
	## tt=numer/(sd+s0) test statistics on original data (and
	#   the ingredients)
	## ttstar0= p by nperms matrix of test statistics on
	#   permted data
	## ttstar= ttstar0 with columns sorted (largest values in
	#   row 1)
	## also returns permuted values: foldchange.star, ystar,
	#   sdstar, censoring.statusstar (for survival data)
	## in xl.mode firsttime or next 20, the function also
	#   returns x , which data$x, except for time-course data,
	##   where it is computed from in this function
	# from time summaries of data$x. However, this quantity is
	#   deleted the last time the function is called,
	#   as it is very large and not needed further
	this.call = match.call()
	resp.type.arg = match.arg(resp.type)
	assay.type = match.arg(assay.type)
	xl.mode = match.arg(xl.mode)
	if (!is.null(random.seed)) {
		set.seed(random.seed)
	}
	if (is.null(nresamp.perm)) {
		nresamp.perm = nresamp
	}
	nresamp.perm = min(nresamp, nresamp.perm)
	if (xl.mode == "regular" | xl.mode == "firsttime") {
		# initialize some things (harmlessly), just so that xl.mode
		#   will work correctly
		x = NULL
		xresamp = NULL
		ttstar0 = NULL
		evo = NULL
		ystar = NULL
		sdstar.keep = NULL
		censoring.status = NULL
		#censoring.status.star.keep = NULL	# Jun commented this line
		sdstar = NULL
		pi0 = NULL
		stand.contrasts = NULL
		stand.contrasts.star = NULL
		stand.contrasts.95 = NULL
		foldchange = NULL
		foldchange.star = NULL
		perms = NULL
		permsy = NULL
		eigengene = NULL
		eigengene.number = NULL
		##
		testStatistic <- match.arg(testStatistic)
		time.summary.type <- match.arg(time.summary.type)
		regression.method <- match.arg(regression.method)
		x = data$x
		y = data$y
		argy = y
		if (!is.null(data$eigengene.number)) {
			eigengene.number = data$eigengene.number
		}
		# impute missing data
		if (sum(is.na(x)) > 0) {
                  x = impute.knn(x, k = knn.neighbors)
                  if (!is.matrix(x)) {
                    x = x$data
                  }
		}
		are.blocks.specified = FALSE
		# check that resp.type ok for seq data
		cond = (resp.type == "One class") | (resp.type == "Two class unpaired timecourse") |
			(resp.type == "One class unpaired timecourse") |
			(resp.type == "Two class paired timecourse") | (resp.type ==
			"Pattern discovery")
		if (assay.type == "seq" & cond) {
			stop(paste("Resp.type=", resp.type, " not allowed when assay.type='seq'"))
		}
		if (assay.type == "seq" & min(x) < 0) {
			stop(paste("Negative values not allowed when assay.type='seq'"))
		}

		## Jun added starts
		## check whether x are counts
		if (assay.type == "seq" & (sum(x%%1 != 0) != 0)) {
			stop("Non-integer values not alled when assay.type='seq'")
		}
		## Jun added ends

		# center columns of  array data if requested.
		# should not be allowed for seq data
		if (assay.type == "seq" & center.arrays) {
			stop(paste("Centering  not allowed when assay.type='seq'"))
		}
		if (assay.type == "seq" & regression.method == "ranks") {
			stop(paste("regression.method==ranks not allowed when assay.type='seq'"))
		}
		if (center.arrays) {
			x <- scale(x, center = apply(x, 2, median), scale = FALSE)
		}
		depth = scaling.factors = rep(NA, ncol(x))
		scaling.factors = (prod(depth)^(1/length(depth)))/depth
		# estimate seq depth and create resampled 3way array for
		#   seq data
		if (assay.type == "seq") {
			cat("Estimating sequencing depths...", fill = T)  ## Jun added this line
			depth = samr.estimate.depth(x)
			cat("Resampling to get new data matrices...", fill = T)  ## Jun added this line
			xresamp = resample(x, depth, nresamp = nresamp)
		}
		scaling.factors = (prod(depth)^(1/length(depth)))/depth
		# check if there are blocks for 2 class unpaired case
		if (resp.type == samr.const.twoclass.unpaired.response) {
			if (substring(y[1], 2, 6) == "Block" | substring(y[1],
				2, 6) == "block") {
				junk = parse.block.labels.for.2classes(y)
				y = junk$y
				blocky = junk$blocky
				are.blocks.specified = TRUE
			}
		}
		# make sure 1,2, -1,1,, etc are non-character values coming
		#   from Excel
		if (resp.type == samr.const.twoclass.unpaired.response |
			resp.type == samr.const.twoclass.paired.response |
			resp.type == samr.const.oneclass.response | resp.type ==
			samr.const.quantitative.response | resp.type == samr.const.multiclass.response) {
			y = as.numeric(y)
		}
		# parse and summarize, if timecourse data
		sd.internal = NULL
		if (resp.type == samr.const.twoclass.unpaired.timecourse.response |
			resp.type == samr.const.twoclass.paired.timecourse.response |
			resp.type == samr.const.oneclass.timecourse.response) {
			junk = parse.time.labels.and.summarize.data(x, y,
				resp.type, time.summary.type)
			y = junk$y
			x = junk$x
			sd.internal = sqrt(rowMeans(junk$sd^2))
			if (min(table(y)) == 1) {
				cat("", fill = T)
				cat("Warning: only one timecourse in one or more classes;\nSAM plot and FDRs will be unreliable; only gene scores are informative",
				  fill = T)
			}
		}
		# if the data is timecourse, we have already summarized the
		#   time aspect.
		# Thus we change the resp.type to the appropriate
		#   non-time-course type. Note that the original value
		#  of resp.type was saved above in resp.type.arg
		if (resp.type == samr.const.twoclass.unpaired.timecourse.response) {
			resp.type = samr.const.twoclass.unpaired.response
		}
		if (resp.type == samr.const.twoclass.paired.timecourse.response) {
			resp.type = samr.const.twoclass.paired.response
		}
		if (resp.type == samr.const.oneclass.timecourse.response) {
			resp.type = samr.const.oneclass.response
		}
		stand.contrasts = NULL
		stand.contrasts.95 = NULL
		if (resp.type == samr.const.survival.response) {
			censoring.status = data$censoring.status
		}
		# do a thorough error  checking of the response data
		check.format(y, resp.type = resp.type, censoring.status = censoring.status)
		# transform to ranks if appropriate
		if (resp.type == samr.const.quantitative.response & regression.method ==
			"ranks") {
			y = rank(y)
			x = t(apply(x, 1, rank))
		}
		n <- nrow(x)
		ny <- length(y)
		sd <- NULL
		numer <- NULL
		# initial computation to get sd
		if (resp.type == samr.const.twoclass.unpaired.response &
			testStatistic == "standard" & assay.type == "array") {
			init.fit <- ttest.func(x, y, sd = sd.internal)
			numer <- init.fit$numer
			sd <- init.fit$sd
		}
		if (resp.type == samr.const.twoclass.unpaired.response &
			testStatistic == "wilcoxon" & assay.type == "array") {
			init.fit <- wilcoxon.func(x, y)
			numer <- init.fit$numer
			sd <- init.fit$sd
		}
		if (resp.type == samr.const.oneclass.response & assay.type ==
			"array") {
			init.fit <- onesample.ttest.func(x, y, sd = sd.internal)
			numer <- init.fit$numer
			sd <- init.fit$sd
		}
		if (resp.type == samr.const.twoclass.paired.response &
			assay.type == "array") {
			init.fit <- paired.ttest.func(x, y, sd = sd.internal)
			numer <- init.fit$numer
			sd <- init.fit$sd
		}
		if (resp.type == samr.const.survival.response & assay.type ==
			"array") {
			init.fit <- cox.func(x, y, censoring.status)
			numer <- init.fit$numer
			sd <- init.fit$sd
		}
		if (resp.type == samr.const.multiclass.response & assay.type ==
			"array") {
			init.fit <- multiclass.func(x, y)
			numer <- init.fit$numer
			sd <- init.fit$sd
		}
		if (resp.type == samr.const.quantitative.response & assay.type ==
			"array") {
			init.fit <- quantitative.func(x, y)
			numer <- init.fit$numer
			sd <- init.fit$sd
		}
		if (resp.type == samr.const.patterndiscovery.response &
			assay.type == "array") {
			init.fit <- patterndiscovery.func(x)
			numer <- init.fit$numer
			sd <- init.fit$sd
		}
		# for wilcoxon or rank regression or patterndiscovery , we
		#   set s0 to the 5th percentile of the sd values
		#   (automatic
		# estimation is not possible as the values of sd are too
		#   coarse)
		# also if dataset is small (< 500 genes), we don't attempt
		#   automatic
		# estimation of s0
		if ((resp.type == samr.const.quantitative.response &
			(testStatistic == "wilcoxon" | regression.method ==
				"ranks" & assay.type == "array") | resp.type ==
			samr.const.patterndiscovery.response) | resp.type ==
			samr.const.twoclass.unpaired.response & assay.type ==
			"array" & testStatistic == "wilcoxon" | (nrow(x) <
			500) & is.null(s0) & is.null(s0.perc)) {
			s0 = quantile(sd, 0.05)
			s0.perc = 0.05
		}
		# estimate s0 if necessary
		if (is.null(s0) & assay.type == "array") {
			if (!is.null(s0.perc)) {
				if ((s0.perc != -1 & s0.perc < 0) | s0.perc >
				  100) {
				  stop("Illegal value for s0.perc: must be between 0 and 100, or equal\nto (-1) (meaning that s0 should be set to zero)")
				}
				if (s0.perc == -1) {
				  s0 = 0
				}
				if (s0.perc >= 0) {
				  s0 <- quantile(init.fit$sd, s0.perc/100)
				}
			}
			if (is.null(s0.perc)) {
				s0 = est.s0(init.fit$tt, init.fit$sd)$s0.hat
				s0.perc = 100 * sum(init.fit$sd < s0)/length(init.fit$sd)
			}
		}
		if (assay.type == "seq") {
			s0 = 0
			s0.perc = 0
		}
		# compute test statistics on original data
		##################
		# array type data
		if (resp.type == samr.const.twoclass.unpaired.response &
			testStatistic == "standard" & assay.type == "array") {
			tt <- ttest.func(x, y, s0 = s0, sd = sd.internal)$tt
		}
		if (resp.type == samr.const.twoclass.unpaired.response &
			testStatistic == "wilcoxon" & assay.type == "array") {
			tt <- wilcoxon.func(x, y, s0 = s0)$tt
		}
		if (resp.type == samr.const.oneclass.response & assay.type ==
			"array") {
			tt <- onesample.ttest.func(x, y, s0 = s0, sd = sd.internal)$tt
		}
		if (resp.type == samr.const.twoclass.paired.response &
			assay.type == "array") {
			tt <- paired.ttest.func(x, y, s0 = s0, sd = sd.internal)$tt
		}
		if (resp.type == samr.const.survival.response & assay.type ==
			"array") {
			tt <- cox.func(x, y, censoring.status, s0 = s0)$tt
		}
		if (resp.type == samr.const.multiclass.response & assay.type ==
			"array") {
			junk2 <- multiclass.func(x, y, s0 = s0)
			tt = junk2$tt
			stand.contrasts = junk2$stand.contrasts
		}
		if (resp.type == samr.const.quantitative.response & assay.type ==
			"array") {
			tt <- quantitative.func(x, y, s0 = s0)$tt
		}
		if (resp.type == samr.const.patterndiscovery.response &
			assay.type == "array") {
			junk <- patterndiscovery.func(x, s0 = s0, eigengene.number = eigengene.number)
			tt <- junk$tt
			eigengene = junk$eigengene
		}
		#seq data
		if (resp.type == samr.const.twoclass.unpaired.response &
			assay.type == "seq") {
			junk = wilcoxon.unpaired.seq.func(xresamp, y)
			tt = junk$tt
			numer = junk$numer
			sd = junk$sd
		}
		if (resp.type == samr.const.twoclass.paired.response &
			assay.type == "seq") {
			junk <- wilcoxon.paired.seq.func(xresamp, y)
			tt = junk$tt
			numer = junk$numer
			sd = junk$sd
		}
		if (resp.type == samr.const.quantitative.response & assay.type ==
			"seq") {
			junk <- quantitative.seq.func(xresamp, y)
			tt = junk$tt
			numer = junk$numer
			sd = junk$sd
		}
		if (resp.type == samr.const.survival.response & assay.type ==
			"seq") {
			junk <- cox.seq.func(xresamp, y, censoring.status)
			tt = junk$tt
			numer = junk$numer
			sd = junk$sd
		}
		if (resp.type == samr.const.multiclass.response & assay.type ==
			"seq") {
			junk2 <- multiclass.seq.func(xresamp, y)
			tt = junk2$tt
			numer = junk2$numer
			sd = junk2$sd
			stand.contrasts = junk2$stand.contrasts
		}
		###########
		# construct matrix of permutations
		if (resp.type == samr.const.quantitative.response | resp.type ==
			samr.const.multiclass.response | resp.type == samr.const.survival.response) {
			junk <- getperms(y, nperms)
			perms = junk$perms
			all.perms.flag = junk$all.perms.flag
			nperms.act = junk$nperms.act
		}
		if (resp.type == samr.const.twoclass.unpaired.response) {
			if (are.blocks.specified) {
				junk = compute.block.perms(y, blocky, nperms)
				permsy = matrix(junk$permsy, ncol = length(y))
				all.perms.flag = junk$all.perms.flag
				nperms.act = junk$nperms.act
			}
			else {
				junk <- getperms(y, nperms)
				permsy = matrix(y[junk$perms], ncol = length(y))
				all.perms.flag = junk$all.perms.flag
				nperms.act = junk$nperms.act
			}
		}
		if (resp.type == samr.const.oneclass.response) {
			if ((length(y) * log(2)) < log(nperms)) {
				allii = 0:((2^length(y)) - 1)
				nperms.act = 2^length(y)
				all.perms.flag = 1
			}
			else {
				nperms.act = nperms
				all.perms.flag = 0
			}
			permsy = matrix(NA, nrow = nperms.act, ncol = length(y))
			if (all.perms.flag == 1) {
				k = 0
				for (i in allii) {
				  junk = integer.base.b(i, b = 2)
				  if (length(junk) < length(y)) {
					junk = c(rep(0, length(y) - length(junk)),
					  junk)
				  }
				  k = k + 1
				  permsy[k, ] = y * (2 * junk - 1)
				}
			}
			else {
				for (i in 1:nperms.act) {
				  permsy[i, ] = sample(c(-1, 1), size = length(y),
					replace = TRUE)
				}
			}
		}
		if (resp.type == samr.const.twoclass.paired.response) {
			junk = compute.block.perms(y, abs(y), nperms)
			permsy = junk$permsy
			all.perms.flag = junk$all.perms.flag
			nperms.act = junk$nperms.act
		}
		if (resp.type == samr.const.patterndiscovery.response) {
			nperms.act = nperms
			perms = NULL
			permsy = NULL
			all.perms.flag = FALSE
		}
		# compute test statistics on permuted  data
		# sdstar.keep <- matrix(0,ncol=nperms.act,nrow=nrow(x)) ##
		#   Jun commented this line
		## Jun added starts
		sdstar.keep <- NULL
		if (assay.type != "seq") {
			sdstar.keep <- matrix(0, ncol = nperms.act, nrow = nrow(x))
		}
		## Jun added ends

		## Jun commented the following 4 lines
#		censoring.status.star.keep <- NULL
#		if (resp.type == samr.const.survival.response) {
#			censoring.status.star.keep <- matrix(0, ncol = nperms.act,
#				nrow = length(y))
#		}

		ttstar <- matrix(0, nrow = nrow(x), ncol = nperms.act)
		foldchange.star = NULL
		if (resp.type == samr.const.twoclass.unpaired.response |
			resp.type == samr.const.twoclass.paired.response) {
			foldchange.star <- matrix(0, nrow = nrow(x), ncol = nperms.act)
		}
		if (resp.type == samr.const.multiclass.response)
		{
			stand.contrasts.star = array(NA, c(nrow(x), length(table(y)),
				nperms.act))
		}
		# end of if(xltime=='regular' etc
	}
	if (xl.mode == "next20" | xl.mode == "lasttime") {
		# get stuff from prevfit
		evo = xl.prevfit$evo
		tt = xl.prevfit$tt
		numer = xl.prevfit$numer
		eigengene = xl.prevfit$eigengene
		eigengene.number = xl.prevfit$eigengene.number
		sd = xl.prevfit$sd - xl.prevfit$s0
		sd.internal = xl.prevfit$sd.internal
		ttstar = xl.prevfit$ttstar
		ttstar0 = xl.prevfit$ttstar0
		n = xl.prevfit$n
		pi0 = xl.prevfit$pi0
		foldchange = xl.prevfit$foldchange
		y = xl.prevfit$y
		x = xl.prevfit$x
		xresamp = xl.prevfit$xresamp
		censoring.status = xl.prevfit$censoring.status
		argy = xl.prevfit$argy
		testStatistic = xl.prevfit$testStatistic
		foldchange.star = xl.prevfit$foldchange.star
		s0 = xl.prevfit$s0
		s0.perc = xl.prevfit$s0.perc
		# ystar= xl.prevfit$ystar
		resp.type = xl.prevfit$resp.type
		resp.type.arg = xl.prevfit$resp.type.arg
		#censoring.status.star.keep = xl.prevfit$censoring.status.star.keep	# Jun commented this line
		assay.type = xl.prevfit$assay.type
		# sdstar= xl.prevfit$sdstar
		sdstar.keep = xl.prevfit$sdstar.keep
		resp.type = xl.prevfit$resp.type
		stand.contrasts = xl.prevfit$stand.contrasts
		stand.contrasts.star = xl.prevfit$stand.contrasts.star
		stand.contrasts.95 = xl.prevfit$stand.contrasts.95
		perms = xl.prevfit$perms
		permsy = xl.prevfit$permsy
		nperms = xl.prevfit$nperms
		nperms.act = xl.prevfit$nperms.act
		all.perms.flag = xl.prevfit$all.perms.flag
		depth = xl.prevfit$depth
		scaling.factors = xl.prevfit$scaling.factors
		nresamp = xl.prevfit$nresamp
		nresamp.perm = xl.prevfit$nresamp.perm
	}
	if (xl.mode == "regular") {
		first = 1
		last = nperms.act
	}
	if (xl.mode == "firsttime") {
		first = 1
		last = 1
	}
	if (xl.mode == "next20") {
		first = xl.time
		last = min(xl.time + 19, nperms.act - 1)
	}
	if (xl.mode == "lasttime") {
		first = nperms.act
		last = nperms.act
	}
	for (b in first:last) {
		cat(c("perm=", b), fill = TRUE)
		if (assay.type == "array") {
			xstar <- x
		}
		if (assay.type == "seq") {
			xstar <- xresamp[, , 1:nresamp.perm]
		}
		if (resp.type == samr.const.oneclass.response) {
			ystar = permsy[b, ]
			if (testStatistic == "standard") {
				ttstar[, b] <- onesample.ttest.func(xstar, ystar,
				  s0 = s0, sd = sd.internal)$tt
			}
		}
		if (resp.type == samr.const.twoclass.paired.response) {
			ystar = permsy[b, ]
			if (assay.type == "array") {
				ttstar[, b] <- paired.ttest.func(xstar, ystar,
				  s0 = s0, sd = sd.internal)$tt
				foldchange.star[, b] = foldchange.paired(xstar,
				  ystar, data$logged2)
			}
			if (assay.type == "seq") {
				ttstar[, b] <- wilcoxon.paired.seq.func(xstar,
				  ystar)$tt
				foldchange.star[, b] <- foldchange.seq.twoclass.paired(x,
				  ystar, depth)  ## Jun added this line
			}
		}
		if (resp.type == samr.const.twoclass.unpaired.response) {
			ystar = permsy[b, ]
			if (assay.type == "array") {
				if (testStatistic == "standard") {
				  junk <- ttest.func(xstar, ystar, s0 = s0, sd = sd.internal)
				}
				if (testStatistic == "wilcoxon") {
				  junk <- wilcoxon.func(xstar, ystar, s0 = s0)
				}
				ttstar[, b] <- junk$tt
				sdstar.keep[, b] <- junk$sd
				foldchange.star[, b] = foldchange.twoclass(xstar,
				  ystar, data$logged2)
			}
			if (assay.type == "seq") {
				ttstar[, b] <- wilcoxon.unpaired.seq.func(xstar,
				  ystar)$tt
				foldchange.star[, b] <- foldchange.seq.twoclass.unpaired(x,
				  ystar, depth)  ## Jun added this line
			}
		}
		if (resp.type == samr.const.survival.response) {
			o <- perms[b, ]
			if (assay.type == "array") {
				ttstar[, b] <- cox.func(xstar, y[o], censoring.status = censoring.status[o],
				  s0 = s0)$tt
			}
			if (assay.type == "seq") {
				ttstar[, b] <- cox.seq.func(xstar, y[o], censoring.status = censoring.status[o])$tt
				#censoring.status.star.keep[, b] <- censoring.status[o]	# Jun commented this line
			}
		}
		if (resp.type == samr.const.multiclass.response) {
			ystar = y[perms[b, ]]
			if (assay.type == "array") {
				junk <- multiclass.func(xstar, ystar, s0 = s0)
				ttstar[, b] <- junk$tt
				sdstar.keep[, b] <- junk$sd
				stand.contrasts.star[, , b] = junk$stand.contrasts
			}
			if (assay.type == "seq") {
				junk <- multiclass.seq.func(xstar, ystar)
				ttstar[, b] <- junk$tt
				stand.contrasts.star[, , b] <- junk$stand.contrasts
			}
		}
		if (resp.type == samr.const.quantitative.response) {
			ystar = y[perms[b, ]]
			if (assay.type == "array") {
				junk <- quantitative.func(xstar, ystar, s0 = s0)
				ttstar[, b] <- junk$tt
				sdstar.keep[, b] <- junk$sd
			}
			if (assay.type == "seq") {
				junk <- quantitative.seq.func(xstar, ystar)
				ttstar[, b] <- junk$tt
			}
		}
		if (resp.type == samr.const.patterndiscovery.response) {
			xstar = permute.rows(x)
			junk <- patterndiscovery.func(xstar, s0 = s0, eigengene.number = eigengene.number)
			ttstar[, b] <- junk$tt
			sdstar.keep[, b] <- junk$sd
		}
		# end of for b in first:last
	}
	# sort columns of statistics from permuted samples, and
	#   compute expected order statistics
	if (xl.mode == "regular" | xl.mode == "lasttime") {
		ttstar0 <- ttstar
		for (j in 1:ncol(ttstar)) {
			ttstar[, j] <- -1 * sort(-1 * ttstar[, j])
		}
		for (i in 1:nrow(ttstar)) {
			ttstar[i, ] <- sort(ttstar[i, ])
		}
		evo <- apply(ttstar, 1, mean)
		evo <- evo[length(evo):1]
		#censoring.statusstar <- censoring.status.star.keep	## Jun commented this line
		sdstar <- sdstar.keep
		# estimation of pi0= prop of null genes
		pi0 = 1
		if (resp.type != samr.const.multiclass.response) {
			qq <- quantile(ttstar, c(0.25, 0.75))
		}
		if (resp.type == samr.const.multiclass.response) {
			qq <- quantile(ttstar, c(0, 0.5))
		}
		pi0 <- sum(tt > qq[1] & tt < qq[2])/(0.5 * length(tt))
		# compute fold changes, when applicable
		foldchange = NULL
		if (resp.type == samr.const.twoclass.unpaired.response &
			assay.type == "array") {
			foldchange = foldchange.twoclass(x, y, data$logged2)
		}
		if (resp.type == samr.const.twoclass.paired.response &
			assay.type == "array") {
			foldchange = foldchange.paired(x, y, data$logged2)
		}
		if (resp.type == samr.const.oneclass.response & assay.type ==
			"array") {
		}
		stand.contrasts.95 = NULL
		if (resp.type == samr.const.multiclass.response)
		{
			stand.contrasts.95 = quantile(stand.contrasts.star,
				c(0.025, 0.975))
		}
		#if(assay.type=='seq'){foldchange=rep(NA,length(tt))} ##
		#   Jun commented this line
		# the last time through, unless otherwise specified, we
		#   delete x, since it is very big and is not needed
		#   further
		## Jun added starts
		if (resp.type == samr.const.twoclass.unpaired.response &
			assay.type == "seq") {
			foldchange <- foldchange.seq.twoclass.unpaired(x,
				y, depth)
		}
		if (resp.type == samr.const.twoclass.paired.response &
			assay.type == "seq") {
			foldchange <- foldchange.seq.twoclass.paired(x, y,
				depth)
		}
		## Jun added ends
		if (return.x == FALSE) {
			x = NULL
		}
	}

	return(list(n=n,
	x=x,
	xresamp=xresamp,
	y=y,
	argy=argy,
	censoring.status= censoring.status,
	testStatistic=testStatistic,
	nperms=nperms,
	nperms.act=nperms.act,
	tt=tt,
	numer=numer,
	sd=sd+s0,
	sd.internal=sd.internal,
	s0=s0,
	s0.perc=s0.perc,
	evo=evo,
	perms=perms,
	permsy=permsy,
	nresamp=nresamp,
	nresamp.perm=nresamp.perm,
	all.perms.flag=all.perms.flag,
	ttstar=ttstar,
	ttstar0=ttstar0,
	eigengene=eigengene,
	eigengene.number=eigengene.number,
	pi0=pi0,
	foldchange=foldchange,
	foldchange.star=foldchange.star,
	sdstar.keep=sdstar.keep,
	#censoring.status.star.keep=censoring.status.star.keep,	## Jun commented this line
	resp.type=resp.type,
	resp.type.arg=resp.type.arg,
	assay.type=assay.type,
	stand.contrasts=stand.contrasts,
	stand.contrasts.star=stand.contrasts.star,
	stand.contrasts.95=stand.contrasts.95,
	depth=depth,
	#scaling.factors=scaling.factors,	## Jun commented this line
	call=this.call))
}

\name{SAMseq}
\alias{SAMseq}
\title{Significance analysis of sequencing data - simple user interface}
\description{
Correlates a large number of features (eg. genes) with an outcome
variable, such as a group indicator, quantitative variable or survival time.
This is a simple user interface for the samr function applied to sequencing data.
For array data applications, see the function SAM.

}
\usage{
SAMseq(x, y, censoring.status = NULL, 
resp.type = c("Quantitative", "Two class unpaired", 
"Survival", "Multiclass", "Two class paired"), 
geneid = NULL, genenames = NULL, nperms = 100, 
random.seed = NULL, nresamp = 20, fdr.output = 0.20)
}
\arguments{
\item{x}{Feature matrix: p (number of features) by n (number of samples),
one observation per column (missing values allowed)}
\item{y}{n-vector of outcome measurements}
\item{censoring.status}{n-vector of censoring censoring.status (1=died or event occurred, 0=survived, or event was censored), needed for a censored survival outcome}
\item{resp.type}{Problem type:
"Quantitative" for a continuous parameter;
"Two class unpaired" for two classes with unpaired observations; 
"Survival" for censored survival outcome; 
"Multiclass": more than 2 groups;  
"Two class paired" for two classes with paired observations.}
\item{geneid}{Optional character vector of geneids for output.}
\item{genenames}{Optional character vector of genenames for output.}
\item{nperms}{Number of permutations used to estimate false discovery rates}
\item{random.seed}{Optional initial seed for random number generator (integer)}
\item{nresamp}{Number of resamples used to construct test statistic. Default 20.}
\item{fdr.output}{(Approximate) False Discovery Rate cutoff for output in significant genes table}

}

\details{This is a simple, user-friendly interface to the samr package used on sequencing data.
It automatically disables arguments/features that do not apply to sequencing data.
It calls samr, samr.compute.delta.table and samr.compute.siggenes.table.
samr detects differential expression for microarray data,
and sequencing data,
and other data with a large number of features. samr is the R package
that is called by the "official" SAM Excel Addin.
The format of the response vector y and the calling sequence
is illustrated in the examples below. A more complete description
is given in the SAM manual
at http://www-stat.stanford.edu/~tibs/SAM}



\value{
A list with components
\item{samr.obj}{Output of samr. See documentation for samr for details }
\item{siggenes.table}{Table of significant genes, output of  samr.compute.siggenes.table.
This has components: genes.up---matrix of significant genes having positive correlation with the outcome and
genes.lo---matrix of significant genes having negative correlation with the outcome.
For survival data, genes.up are those genes having positive correlation with risk-
 that is, increased expression corresponds to higher risk (shorter survival)
genes.lo are those whose increased expression corresponds to lower risk (longer survival).}
\item{delta.table}{Output of  samr.compute.delta.table.}
\item{del}{Value of delta (distance from 45 degree line in SAM plot) for 
 used for creating delta.table and siggenes.table. Changing the input value fdr.output
will change the resulting del.}
\item{call}{The calling sequence}
}

\references{Tusher, V.,  Tibshirani, R.  and Chu, G. (2001): 
Significance analysis of microarrays applied to the ionizing radiation response  PNAS 2001 98: 5116-5121, (Apr 24). 
http://www-stat.stanford.edu/~tibs/SAM

Li, Jun and Tibshirani, R. (2011). Finding consistent patterns: a nonparametric
approach for identifying differential expression in
RNA-Seq data. To appear, Statistical Methods in Medical Research.
}
\author{Jun Li and Balasubrimanian Narasimhan and Robert Tibshirani}


\examples{

######### two class unpaired comparison
set.seed(100)
mu <- matrix(100, 1000, 20)
mu[1:100, 11:20] <- 200
mu <- scale(mu, center=FALSE, scale=runif(20, 0.5, 1.5))
x <- matrix(rpois(length(mu), mu), 1000, 20)
y <- c(rep(1, 10), rep(2, 10))

samfit <- SAMseq(x, y, resp.type = "Two class unpaired") 

# examine significant gene list
print(samfit)

# plot results
plot(samfit)

######### two class paired comparison
set.seed(100)
mu <- matrix(100, 1000, 20)
mu[1:100, 11:20] <- 200
mu <- scale(mu, center=FALSE, scale=runif(20, 0.5, 1.5))
x <- matrix(rpois(length(mu), mu), 1000, 20)
y <- c(-(1:10), 1:10)

samfit <- SAMseq(x, y, resp.type = "Two class paired") 

# examine significant gene list
print(samfit)

# plot results
plot(samfit)

######### Multiclass comparison
set.seed(100)
mu <- matrix(100, 1000, 20)
mu[1:20, 1:5] <- 120
mu[21:50, 6:10] <- 80
mu[51:70, 11:15] <- 150
mu[71:100, 16:20] <- 60
mu <- scale(mu, center=FALSE, scale=runif(20, 0.5, 1.5))
x <- matrix(rpois(length(mu), mu), 1000, 20)
y <- c(rep(1:4, rep(5, 4)))

samfit <- SAMseq(x, y, resp.type = "Multiclass") 

# examine significant gene list
print(samfit)

# plot results
plot(samfit)

######### Quantitative comparison
set.seed(100)
mu <- matrix(100, 1000, 20)
y <- runif(20, 1, 3)
mu[1 : 100, ] <- matrix(rep(100 * y, 100), ncol=20, byrow=TRUE)
mu <- scale(mu, center=FALSE, scale=runif(20, 0.5, 1.5))
x <- matrix(rpois(length(mu), mu), 1000, 20)
samfit <- SAMseq(x, y, resp.type = "Quantitative")

# examine significant gene list
print(samfit)

# plot results
plot(samfit)

######### Survival comparison
set.seed(100)
mu <- matrix(100, 1000, 20)
y <- runif(20, 1, 3)
mu[1 : 100, ] <- matrix(rep(100 * y, 100), ncol=20, byrow=TRUE)
mu <- scale(mu, center=FALSE, scale=runif(20, 0.5, 1.5))
x <- matrix(rpois(length(mu), mu), 1000, 20)
y <- y + runif(20, -0.5, 0.5)
censoring.status <- as.numeric(y < 2.3)
y[y >= 2.3] <- 2.3
samfit <- SAMseq(x, y, censoring.status = censoring.status, 
resp.type = "Survival")

# examine significant gene list
print(samfit)

# plot results
plot(samfit)
}

\keyword{univar}% at least one, from doc/KEYWORDS
\keyword{survival}
\keyword{ts}
\keyword{nonparametric}

\name{samr.missrate}
\alias{samr.missrate}
\title{Estimate the  miss rate table for a SAM analysis}
\description{
Estimates the  miss rate table, showing the local false negative rate, for a SAM
analysis
}
\usage{
samr.missrate(samr.obj, del, delta.table, quant=NULL)
}
\arguments{
\item{samr.obj}{ Object returned from call to samr}
\item{del}{Value of delta to define cutoff rule}
\item{delta.table}{ Object returned from call to samr.compute.delta.table}
\item{quant}{Optional vector of quantiles to used in the miss rate calculation}
}
\references{Tusher, V.,  Tibshirani, R.  and Chu, G. (2001):
Significance analysis of microarrays applied to the ionizing radiation response"  PNAS 2001 98: 5116-5121, (Apr 24).
http://www-stat.stanford.edu/~tibs/sam

Taylor, J., Tibshirani, R. and Efron. B. (2005). 
The ``Miss rate'' for the analysis of gene expression data.  Biostatistics 2005 6(1):111-117. 
}

\author{Jun Li and Balasubrimanian Narasimhan and Robert Tibshirani}


\examples{
#generate some example data
set.seed(100)
x<-matrix(rnorm(1000*20),ncol=20)
dd<-sample(1:1000,size=100)

u<-matrix(2*rnorm(100),ncol=10,nrow=100)
x[dd,11:20]<-x[dd,11:20]+u

y<-c(rep(1,10),rep(2,10))

data=list(x=x,y=y, geneid=as.character(1:nrow(x)),
genenames=paste("g",as.character(1:nrow(x)),sep=""), logged2=TRUE)


samr.obj<-samr(data,  resp.type="Two class unpaired", nperms=100)

delta.table<-samr.compute.delta.table(samr.obj)
del<- 0.3
siggenes.table<- samr.compute.siggenes.table(samr.obj, del, data, delta.table)


samr.missrate(samr.obj, del, delta.table)
}
\keyword{univar}% at least one, from doc/KEYWORDS
\keyword{survival}
\keyword{ts}
\keyword{nonparametric}

\name{samr.compute.delta.table}
\alias{samr.compute.delta.table}
\title{Compute delta table for SAM analysis}
\description{
Computes tables of thresholds, cutpoints and corresponding
False Discovery rates for SAM (Significance analysis of microarrays)
analysis
}
\usage{

samr.compute.delta.table(samr.obj, min.foldchange=0, dels=NULL, nvals=50)
}
\arguments{
  \item{samr.obj}{ Object returned from call to samr}
 \item{min.foldchange}{The minimum fold change desired; should be >1;
default is zero, meaning no fold change criterion is applied}
\item{dels}{vector of delta values used. 
Delta is the vertical distance from the 45 degree line to the upper and lower
parallel lines that define the SAM threshold rule.
By default, for array data, 50 values
are chosen in the relevant operating change for delta.
For sequencing data, the maximum number of effective delta values
are chosen automatically according to the data.
}

\item{nvals}{Number of delta values used. 
For array data, the default value is 50. For sequencing data,
the value will be chosen automatically.}
}
\details{Returns a table of the FDR and upper and lower cutpoints for various values of
delta, for a SAM analysis.
}

\references{Tusher, V.,  Tibshirani, R.  and Chu, G. (2001): 
Significance analysis of microarrays applied to the ionizing radiation response"  PNAS 2001 98: 5116-5121, (Apr 24).
http://www-stat.stanford.edu/~tibs/sam}
\author{ Balasubrimanian Narasimhan and Robert Tibshirani}


\examples{
#generate some example data
set.seed(100)
x<-matrix(rnorm(1000*20),ncol=20)
dd<-sample(1:1000,size=100)

u<-matrix(2*rnorm(100),ncol=10,nrow=100)
x[dd,11:20]<-x[dd,11:20]+u

y<-c(rep(1,10),rep(2,10))

data=list(x=x,y=y, geneid=as.character(1:nrow(x)),
genenames=paste("g",as.character(1:nrow(x)),sep=""), logged2=TRUE)


samr.obj<-samr(data,  resp.type="Two class unpaired", nperms=50)

delta.table<- samr.compute.delta.table(samr.obj)

}
\keyword{univar}% at least one, from doc/KEYWORDS
\keyword{survival}
\keyword{ts}
\keyword{nonparametric}

\name{samr.norm.data}
\alias{samr.norm.data}
\title{output normalized sequencing data}
\description{
Output a normalized sequencing data matrix from the original count matrix.
}
\usage{
samr.norm.data(x, depth=NULL)}
\arguments{
  \item{x}{the original count matrix. p by n matrix of features, one observation per column.}
  \item{depth}{sequencing depth of each experiment. a vector of length n.
  This function will estimate the sequencing depth if it is not specified.}
}

\details{normalize the data matrix so that each number looks roughly like
Gaussian distributed and each experiment has the same sequencing depth.
To do this, we first use Anscombe transformation to stablize the variance
and makes each number look like Gaussian,
and then divide each experiment by the square root of the sequencing depth.}



\value{
\item{x}{the normalized data matrix.}
}

 \references{Tusher, V.,  Tibshirani, R.  and Chu, G. (2001): 
Significance analysis of microarrays applied to the ionizing radiation response  PNAS 2001 98: 5116-5121, (Apr 24). 
http://www-stat.stanford.edu/~tibs/SAM}


\author{Jun Li and Balasubrimanian Narasimhan and Robert Tibshirani}

\examples{
set.seed(100)
mu <- matrix(100, 1000, 20)
mu[1:100, 11:20] <- 200
mu <- scale(mu, center=FALSE, scale=runif(20, 0.5, 1.5))
x <- matrix(rpois(length(mu), mu), 1000, 20)
y <- c(rep(1, 10), rep(2, 10))
data=list(x=x,y=y, geneid=as.character(1:nrow(x)),
genenames=paste("g",as.character(1:nrow(x)),sep=""))
x.norm <- samr.norm.data(data$x)
}

\keyword{univar}% at least one, from doc/KEYWORDS

\name{samr.tail.strength}
\alias{samr.tail.strength}
\title{Estimate tail strength for a dataset, from a SAM analysis}
\description{Estimate tail strength for a dataset, from a SAM analysis
}
\usage{
samr.tail.strength(samr.obj)
}
\arguments{
\item{samr.obj}{Object returned by samr}
}
\value{
A list with components
\item{ts}{Estimated tail strength. A number less than or equal to 1.
Zero means all genes are null; 1 means all genes are differentially expressed.},
\item{se.ts}{Estimated standard error of tail strength.}
}



\references{Taylor, J. and Tibshirani, R.   (2005):
A tail strength measure for assessing the overall significance in a dataset.
Submitted.
}

\author{Jun Li and Balasubrimanian Narasimhan and Robert Tibshirani}


\examples{
#generate some example data
set.seed(100)
x<-matrix(rnorm(1000*20),ncol=20)
dd<-sample(1:1000,size=100)

u<-matrix(2*rnorm(100),ncol=10,nrow=100)
x[dd,11:20]<-x[dd,11:20]+u

y<-c(rep(1,10),rep(2,10))

data=list(x=x,y=y, geneid=as.character(1:nrow(x)),
genenames=paste("g",as.character(1:nrow(x)),sep=""), logged2=TRUE)

samr.obj<-samr(data,  resp.type="Two class unpaired", nperms=100)

samr.tail.strength(samr.obj)

}
\keyword{univar}% at least one, from doc/KEYWORDS
\keyword{survival}
\keyword{nonparametric}

\name{runSAM}
\alias{runSAM}
\title{Run the sam webapp}
\description{
Runs the sam web application for a graphical user interface.
}

\usage{
runSAM()
}

\details{Uses shiny to create a graphical user interface for SAM}

\references{Tusher, V.,  Tibshirani, R.  and Chu, G. (2001):
  Significance analysis of microarrays applied to the ionizing radiation response"  PNAS 2001 98: 5116-5121, (Apr 24).
  http://www-stat.stanford.edu/~tibs/sam}
\author{Michael J. Seo}


\examples{
\dontrun{runSAM()}
}

\name{samr.plot}
\alias{samr.plot}

\title{Make Q-Q plot for SAM analysis}
\description{
Makes the Q-Q plot for a SAM analysis}

\usage{
samr.plot(samr.obj, del, min.foldchange=0) 
}
\arguments{
  \item{samr.obj}{ Object returned from call to samr}
\item{del}{Value of delta to use. 
Delta is the  vertical  distance from the 45 degree line to the upper and lower
parallel lines that define the SAM threshold rule.}
 \item{min.foldchange}{The minimum fold change desired; should be >1;
default is zero, meaning no fold change criterion is applied}
}

\details{Creates the Q-Q plot fro a SAm analysis, marking features (genes)
that are significant, ie. lie outside a slab around teh 45 degreee line of
width delta. A gene must also pass the  min.foldchange criterion 
to be called significant, if this criterion is specified in the call.
}

\references{Tusher, V.,  Tibshirani, R.  and Chu, G. (2001): 
Significance analysis of microarrays applied to the ionizing radiation response"  PNAS 2001 98: 5116-5121, (Apr 24).
http://www-stat.stanford.edu/~tibs/sam}
\author{Jun Li and Balasubrimanian Narasimhan and Robert Tibshirani}


\examples{
#generate some example data
set.seed(100)
x<-matrix(rnorm(1000*20),ncol=20)
dd<-sample(1:1000,size=100)

u<-matrix(2*rnorm(100),ncol=10,nrow=100)
x[dd,11:20]<-x[dd,11:20]+u

y<-c(rep(1,10),rep(2,10))

data=list(x=x,y=y, geneid=as.character(1:nrow(x)),
genenames=paste("g",as.character(1:nrow(x)),sep=""), logged2=TRUE)


samr.obj<-samr(data,  resp.type="Two class unpaired", nperms=50)


samr.plot(samr.obj, del=.3)
}
\keyword{univar}% at least one, from doc/KEYWORDS
\keyword{survival}
\keyword{ts}
\keyword{nonparametric}

\name{SAM}
\alias{SAM}
\title{Significance analysis of microarrays - simple user interface}
\description{
Correlates a large number of features (eg genes) with an outcome
variable, such as a group indicator, quantitative variable or survival time.
This is a simple user interface for the samr function applied to array data.
For sequencing data applications, see the function SAMseq.
}
\usage{
SAM(x,y=NULL,censoring.status=NULL,
resp.type=c("Quantitative","Two class unpaired","Survival","Multiclass",
"One class", "Two class paired","Two class unpaired timecourse",
"One class timecourse","Two class paired timecourse", "Pattern discovery"),
geneid = NULL,
genenames = NULL,
s0=NULL, 
s0.perc=NULL, 
nperms=100, 
center.arrays=FALSE, 
testStatistic=c("standard","wilcoxon"), 
time.summary.type=c("slope","signed.area"), 
regression.method=c("standard","ranks"), 
return.x=TRUE, 
knn.neighbors=10, 
random.seed=NULL,
logged2 = FALSE,
fdr.output = 0.20,
eigengene.number = 1)
}
\arguments{
  \item{x}{Feature matrix: p (number of features) by n (number of samples),
one observation per column (missing values allowed)}
\item{y}{n-vector of outcome measurements}
\item{censoring.status}{n-vector of censoring censoring.status (1= died or event occurred, 0=survived, or event was censored), needed for a censored survival outcome}
\item{resp.type}{Problem type:
"Quantitative" for a continuous parameter;
"Two class unpaired"; 
"Survival" for censored survival outcome; 
"Multiclass": more than 2 groups;  
"One class" for a single group; 
"Two class paired" for two classes with paired observations;
"Two class unpaired timecourse", 
"One class time course",
"Two class.paired timecourse",
"Pattern discovery"}
\item{geneid}{Optional character vector of geneids for output.}
\item{genenames}{Optional character vector of genenames for output.}
\item{s0}{Exchangeability factor  for denominator of test statistic; Default
is automatic choice. Only used for array data.}
\item{s0.perc}{Percentile of standard  deviation values to use for s0; default is
automatic choice; -1 means s0=0 (different from s0.perc=0, meaning
s0=zeroeth percentile of  standard  deviation values= min of sd values.
Only used for array data.}
\item{nperms}{Number of permutations used to estimate false discovery rates}
\item{center.arrays}{Should the data for each sample (array) be median centered
at the outset? Default =FALSE. Only used for array data.},
\item{testStatistic}{Test statistic to use in two class unpaired case.Either
"standard" (t-statistic) or ,"wilcoxon" (Two-sample wilcoxon or Mann-Whitney
test). Only used for array data.} 
\item{time.summary.type}{Summary measure for each time course: "slope", or "signed.area"). Only used for array data.} 
\item{regression.method}{Regression method for quantitative case:
"standard", (linear least squares) or "ranks" (linear least squares 
on ranked data). Only used for array data.}
\item{return.x}{Should the matrix of feature values be returned?
Only useful for time course data, where x contains summaries of the features
over time. Otherwise x is the same as the input data data$x}
\item{knn.neighbors}{Number of nearest neighbors to use for imputation
of missing features values. Only used for array data.}
\item{random.seed}{Optional initial seed for random number generator (integer)}
\item{logged2}{Has the data been transformed by log (base 2)? This information is used only 
for computing fold changes }
\item{fdr.output}{(Approximate) False Discovery Rate cutoff for output in significant genes table}
\item{eigengene.number}{Eigengene to be used  (just for resp.type="Pattern discovery")}
}

\details{This is a simple, user-friendly interface to the samr package used on array data.
It calls samr, samr.compute.delta.table and samr.compute.siggenes.table.
samr detects differential expression for  microarray data,
and sequencing data,
and other data with a large number of features. samr is the R package
that is called by the "official" SAM Excel Addin.
The format of the response vector y and the calling sequence
is illustrated in the examples below. A more complete  description
is given in the SAM manual
at http://www-stat.stanford.edu/~tibs/SAM}



\value{
A list with components
\item{samr.obj}{Output of samr. See documentation for samr for details.}
\item{siggenes.table}{Table of significant genes, output of  samr.compute.siggenes.table.
This has components: genes.up--- matrix of significant genes having positive correlation with the outcome and
genes.lo---matrix of significant genes having negative correlation with the outcome.
For survival data, genes.up are those genes having positive correlation with risk-
 that is, increased expression corresponds to higher risk (shorter survival)
genes.lo are those whose increased expression corresponds to lower risk (longer survival).}
\item{delta.table}{Output of  samr.compute.delta.table.}
\item{del}{Value of delta (distance from 45 degree line in SAM plot) for 
 used for creating delta.table and siggenes.table. Changing the input value fdr.output
will change the resulting del.}
\item{call}{The calling sequence}
}

 \references{Tusher, V.,  Tibshirani, R.  and Chu, G. (2001): 
Significance analysis of microarrays applied to the ionizing radiation response  PNAS 2001 98: 5116-5121, (Apr 24). 
http://www-stat.stanford.edu/~tibs/SAM

Li, Jun and Tibshirani, R. (2011). Finding consistent patterns: a nonparametric
approach for identifying differential expression in
RNA-Seq data. To appear, Statistical Methods in Medical Research.
}
\author{Jun Li and Balasubrimanian Narasimhan and Robert Tibshirani}


\examples{

######### two class unpaired comparison
# y must take values 1,2

set.seed(100)
x<-matrix(rnorm(1000*20),ncol=20)
dd<-sample(1:1000,size=100)

u<-matrix(2*rnorm(100),ncol=10,nrow=100)
x[dd,11:20]<-x[dd,11:20]+u
y<-c(rep(1,10),rep(2,10))

samfit<-SAM(x,y,resp.type="Two class unpaired")

# examine significant gene list

print(samfit)

# plot results
plot(samfit)

########### two class paired

# y must take values  -1, 1, -2,2 etc, with (-k,k) being a pair

set.seed(100)
x<-matrix(rnorm(1000*20),ncol=20)
dd<-sample(1:1000,size=100)

u<-matrix(2*rnorm(100),ncol=10,nrow=100)
x[dd,11:20]<-x[dd,11:20]+u

y=c(-(1:10),1:10)



samfit<-SAM(x,y, resp.type="Two class paired",fdr.output=.25)




#############quantitative response


set.seed(30)
p=1000
x<-matrix(rnorm(p*20),ncol=20)
y<-rnorm(20)
x[1:20,y>0]=x[1:20,y>0]+4
a<-SAM(x,y,resp.type="Quantitative",nperms=50,fdr.output=.5)






###########survival data
# y is numeric; censoring.status=1 for failures, and 0 for censored

set.seed(84048)
x=matrix(rnorm(1000*50),ncol=50)
x[1:50,26:50]= x[1:50,26:50]+2
x[51:100,26:50]= x[51:100,26:50]-2

y=abs(rnorm(50))
y[26:50]=y[26:50]+2
censoring.status <- sample(c(0,1),size=50,replace=TRUE)

a<-SAM(x,y,censoring.status=censoring.status,resp.type="Survival",
nperms=20)



################multi-class example
# y takes values 1,2,3,...k where k= number of classes

set.seed(84048)
x=matrix(rnorm(1000*10),ncol=10)

y=c(rep(1,3),rep(2,3),rep(3,4))
x[1:50,y==3]=x[1:50,y==3]+5

a <- SAM(x,y,resp.type="Multiclass",nperms=50)






##################### pattern discovery
# here there is no outcome y; the desired eigengene is indicated by 
# the argument eigengene.numbe in the data object

set.seed(32)
x=matrix(rnorm(1000*9),ncol=9)
mu=c(3,2,1,0,0,0,1,2,3)
b=3*runif(100)+.5
x[1:100,]=x[1:100,]+ b%*%t(mu)



d=list(x=x,eigengene.number=1,
geneid=as.character(1:nrow(x)),genenames=paste("gene", as.character(1:nrow(x))))


a <- SAM(x, resp.type="Pattern discovery", nperms=50)


#################### timecourse data

# elements of y are of the form  kTimet  where k is the class label and t
# is the time; in addition, the   suffixes Start or End indicate the first
# and last observation in a given time course
# the class label can be that for a two class unpaired, one class or
# two class paired problem

set.seed(8332)
y=paste(c(rep(1,15),rep(2,15)),"Time",rep(c(1,2,3,4,5,1.1,2.5, 3.7, 4.1,5.5),3),
sep="")
start=c(1,6,11,16,21,26)
for(i in start){
y[i]=paste(y[i],"Start",sep="")
}
for(i in  start+4){
y[i]=paste(y[i],"End",sep="")
}
x=matrix(rnorm(1000*30),ncol=30)
x[1:50,16:20]=x[1:50,16:20]+matrix(3*c(0,1,2,3,4),ncol=5,nrow=50,byrow=TRUE)
x[1:50,21:25]=x[1:50,21:25]+matrix(3*c(0,1,2,3,4),ncol=5,nrow=50,byrow=TRUE)
x[1:50,26:30]=x[1:50,26:30]+matrix(3*c(0,1,2,3,4),ncol=5,nrow=50,byrow=TRUE)

x[51:100,16:20]=x[51:100,16:20]-matrix(3*c(0,1,2,3,4),ncol=5,nrow=50,byrow=TRUE)
x[51:100,21:25]=x[51:100,21:25]-matrix(3*c(0,1,2,3,4),ncol=5,nrow=50,byrow=TRUE)
x[51:100,26:30]=x[51:100,26:30]-matrix(3*c(0,1,2,3,4),ncol=5,nrow=50,byrow=TRUE)

a<- SAM(x,y,  resp.type="Two class unpaired timecourse",
 nperms=100, time.summary.type="slope")


}

\keyword{univar}% at least one, from doc/KEYWORDS
\keyword{survival}
\keyword{ts}
\keyword{nonparametric}

\name{samr.estimate.depth}
\alias{samr.estimate.depth}
\title{estimate the sequencing depth}
\description{
Estimate the sequencing depth of each experiment for sequencing data.
}
\usage{
samr.estimate.depth(x)}
\arguments{
  \item{x}{the original count matrix. p by n matrix of features, one observation per column.}
}

\details{normalize the data matrix so that each number looks roughly like
Gaussian distributed and each experiment has the same sequencing depth.
To do this, we first use Anscombe transformation to stablize the variance
and makes each number look like Gaussian,
and then divide each experiment by the square root of the sequencing depth.}



\value{
\item{depth}{sequencing depth of each experiment. a vector of length n.}
}

 \references{Tusher, V.,  Tibshirani, R.  and Chu, G. (2001): 
Significance analysis of microarrays applied to the ionizing radiation response  PNAS 2001 98: 5116-5121, (Apr 24). 
http://www-stat.stanford.edu/~tibs/SAM}

\author{Jun Li and Balasubrimanian Narasimhan and Robert Tibshirani}


\examples{
set.seed(100)
mu <- matrix(100, 1000, 20)
mu[1:100, 11:20] <- 200
mu <- scale(mu, center=FALSE, scale=runif(20, 0.5, 1.5))
x <- matrix(rpois(length(mu), mu), 1000, 20)
y <- c(rep(1, 10), rep(2, 10))
data=list(x=x,y=y, geneid=as.character(1:nrow(x)),
genenames=paste("g",as.character(1:nrow(x)),sep=""))
depth <- samr.estimate.depth(data$x)
}

\keyword{univar}% at least one, from doc/KEYWORDS

\name{samr.pvalues.from.perms}
\alias{samr.pvalues.from.perms}
\title{Report estimated p-values for each gene, from a  SAM analysis}
\description{Report estimated p-values for each gene, from set of
permutations in a SAM analysis
}
\usage{
samr.pvalues.from.perms(tt, ttstar)
}
\arguments{
\item{tt}{Vector of gene scores, returned by samr in component tt}
\item{ttstar}{Matrix of gene scores (p by nperm) from nperm permutations.
Returned by samr in component ttstar}
}
\references{Taylor, J. and Tibshirani, R.   (2005):
A tail strength measure for assessing the overall significance in a dataset.
Submitted.
}

\author{Jun Li and Balasubrimanian Narasimhan and Robert Tibshirani}


\examples{
#generate some example data
set.seed(100)
x<-matrix(rnorm(1000*20),ncol=20)
dd<-sample(1:1000,size=100)

u<-matrix(2*rnorm(100),ncol=10,nrow=100)
x[dd,11:20]<-x[dd,11:20]+u

y<-c(rep(1,10),rep(2,10))

data=list(x=x,y=y, geneid=as.character(1:nrow(x)),
genenames=paste("g",as.character(1:nrow(x)),sep=""), logged2=TRUE)
samr.obj<-samr(data,  resp.type="Two class unpaired", nperms=100)

pv=samr.pvalues.from.perms(samr.obj$tt, samr.obj$ttstar)
}
\keyword{univar}% at least one, from doc/KEYWORDS
\keyword{survival}
\keyword{nonparametric}

\name{samr}
\alias{samr}
\title{Significance analysis of microarrays}
\description{
Correlates a large number of features (eg genes) with an outcome
variable, such as a group indicator, quantitative variable or  survival time.
NOTE: for most users, the interface function SAM--- which calls samr-- will be more 
convenient for array data, and the interface function SAMseq-- which also calls samr--
will be more convenient for sequencing data.
}
\usage{
samr(data, resp.type=c("Quantitative","Two class unpaired",
"Survival","Multiclass", "One class", "Two class paired",
"Two class unpaired timecourse", "One class timecourse",
"Two class paired timecourse", "Pattern discovery"),
assay.type=c("array","seq"), s0=NULL, s0.perc=NULL, nperms=100, 
center.arrays=FALSE, testStatistic=c("standard","wilcoxon"), 
time.summary.type=c("slope","signed.area"), 
regression.method=c("standard","ranks"), return.x=FALSE, 
knn.neighbors=10, random.seed=NULL, nresamp=20,nresamp.perm=NULL, 
xl.mode=c("regular","firsttime","next20","lasttime"), 
xl.time=NULL,  xl.prevfit=NULL)
}
\arguments{
  \item{data}{Data object with components x- p by n matrix of features,
one observation per column (missing values allowed); y- n-vector of outcome measurements;
censoring.status- n-vector of censoring censoring.status (1= died or event occurred, 0=survived, or event was censored), needed
for a censored survival outcome}
  \item{resp.type}{Problem type:
"Quantitative" for a continuous parameter (Available for both array and sequencing data);
"Two class unpaired" (for both array and sequencing data); 
"Survival" for censored survival outcome (for both array and sequencing data); 
"Multiclass": more than 2 groups (for both array and sequencing data);  
"One class" for a single group (only for array data); 
"Two class paired" for two classes with paired observations (for both array and sequencing data);
"Two class unpaired timecourse" (only for array data), 
"One class time course" (only for array data),
"Two class.paired timecourse" (only for array data),
or "Pattern discovery" (only for array data)}
\item{assay.type}{Assay type: "array" for microarray data, "seq" for counts from sequencing}
\item{s0}{Exchangeability factor  for denominator of test statistic; Default
is automatic choice. Only used for array data.}
\item{s0.perc}{Percentile of standard  deviation values to use for s0; default is
automatic choice; -1 means s0=0 (different from s0.perc=0, meaning
s0=zeroeth percentile of  standard  deviation values= min of sd values.
Only used for array data.}
\item{nperms}{Number of permutations used to estimate false discovery rates}
\item{center.arrays}{Should the data for each sample (array) be median centered
at the outset? Default =FALSE. Only used for array data.},
\item{testStatistic}{Test statistic to use in two class unpaired case.Either
"standard" (t-statistic) or ,"wilcoxon" (Two-sample wilcoxon or Mann-Whitney
test). Only used for array data.} 
\item{time.summary.type}{Summary measure for each time course: "slope", or "signed.area"). Only used for array data.} 
\item{regression.method}{Regression method for quantitative case:
"standard", (linear least squares) or "ranks" (linear least squares 
on ranked data). Only used for array data.}
\item{return.x}{Should the matrix of feature values be returned?
Only useful for time course data, where x contains summaries of the features
over time. Otherwise x is the same as the input data data$x}
\item{knn.neighbors}{Number of nearest neighbors to use for imputation
of missing features values. Only used for array data.}
\item{random.seed}{Optional initial seed for random number generator (integer)}
\item{nresamp}{For assay.type="seq", number of resamples used to construct test statistic. Default 20. Only used for sequencing data.}
\item{nresamp.perm}{For assay.type="seq", number of resamples used to construct test statistic for permutations. Default is equal to nresamp and it must be at most nresamp. Only used for sequencing data.}
\item{xl.mode}{Used by Excel interface}
\item{xl.time}{Used by Excel interface}
\item{xl.prevfit}{Used by Excel interface}
}

\details{Carries out a SAM analysis. Applicable to microarray data,
sequencing data,
and other data with a large number of features. This is the R package
that is called by the "official" SAM Excel package v2.0.
The format of the response vector y and the calling sequence
is illustrated in the examples below. A more complete  description
is given in the SAM manual
at http://www-stat.stanford.edu/~tibs/SAM}



\value{
A list with components
\item{n}{Number of observations}
\item{x}{Data matrix p by n (p=# genes or features). Equal to the  matrix data$x  in the original call to samr except
for (1) time course analysis, where is contains the summarized data
or (2) quantitative outcome with rank regression, where it contains
the data transformed to ranks. Hence it is null except for in time course analysis. }
\item{y}{Vector of n outcome values. equal the values data$y in the original call to samr, except
for (1) time course analysis, where is contains the summarized y 
or (2) quantitative outcome with rank regression, where it contains
the y values transformed to ranks}
\item{argy}{The  values data$y in the original call to samr} 
\item{censoring.status}{Censoring status indicators if applicable}
\item{testStatistic}{Test Statistic used},
\item{nperms}{Number of permutations requested}
\item{nperms.act}{Number of permutations actually used. Will be <
nperms when # of possible permutations  <= nperms (in which case
all permutations are done)}
\item{tt}{tt=numer/sd, the  vector of p test statistics for original data}
\item{numer}{Numerators for tt}
\item{sd}{Denominators for tt. Equal to standard deviation for feature plus s0}
\item{s0}{Computed  exchangeability factor}
\item{s0.perc}{Computed percentile of standard deviation values.
s0= s0.perc percentile of the gene standard deviations}
\item{eva}{p-vector of expected values for tt under permutation sampling}
\item{perms}{nperms.act by n matrix  of permutations used. Each row is 
a permutation of 1,2...n}
\item{permsy}{nperms.act by n matrix  of permutations used. Each row is
a permutation of y1,y2,...yn. Only one of perms or permys is non-Null, depending on resp.type} 
\item{all.perms.flag}{Were all possible permutations used?}
\item{ttstar}{p by nperms.aca matrix t of test statistics from permuted data. Each column if sorted in descending order}
\item{ttstar0}{p by nperms.act matrix  of test statistics from permuted data. Columns are in order of data}
\item{eigengene.number}{The number of the eigengene (eg 1,2,..) that was requested for Pattern discovery}
\item{eigengene}{Computed eigengene}
\item{pi0}{Estimated proportion of non-null features (genes)}
\item{foldchange}{p-vector of foldchanges for original data}
\item{foldchange.star}{p by nperms.act matrix estimated foldchanges from permuted data}
\item{sdstar.keep}{n by nperms.act matrix of standard deviations
from each permutation}
\item{censoring.status.star.keep}{n by nperms.act matrix of 
censoring.status indicators from each permutation}
\item{resp.type}{The response type used. Same as resp.type.arg, except for
time course data, where time data is summarized and then treated as non-time course. Eg if resp.type.arg="oneclass.timecourse" then  resp.type="oneclass"}
\item{resp.type.arg}{The response type requested in the call to samr}
\item{stand.contrasts}{For multiclass data, p by nclass matrix of  standardized differences
between the class mean and the overall mean}
\item{stand.contrasts.star}{For multiclass data, p by nclass by nperms.act array of standardized contrasts for permuted datasets}
\item{stand.contrasts.95}{For multiclass data, 2.5% and 97.5% percentiles
of standardized contrasts. Useful for determining which class contrast
for significant genes, are large}
\item{depth}{For array.type="seq", estimated sequencing depth for each sample.}
\item{call}{calling sequence}
}

 \references{Tusher, V.,  Tibshirani, R.  and Chu, G. (2001): 
Significance analysis of microarrays applied to the ionizing radiation response  PNAS 2001 98: 5116-5121, (Apr 24). 
http://www-stat.stanford.edu/~tibs/SAM

Li, Jun and Tibshirani, R. (2011). Finding consistent patterns: a nonparametric
approach for identifying differential expression in
RNA-Seq data. To appear, Statistical Methods in Medical Research.
}
\author{Jun Li and Balasubrimanian Narasimhan and Robert Tibshirani}


\examples{

######### two class unpaired comparison
# y must take values 1,2

set.seed(100)
x<-matrix(rnorm(1000*20),ncol=20)
dd<-sample(1:1000,size=100)

u<-matrix(2*rnorm(100),ncol=10,nrow=100)
x[dd,11:20]<-x[dd,11:20]+u
y<-c(rep(1,10),rep(2,10))

data=list(x=x,y=y, geneid=as.character(1:nrow(x)),
genenames=paste("g",as.character(1:nrow(x)),sep=""), logged2=TRUE)


samr.obj<-samr(data,  resp.type="Two class unpaired", nperms=100)

delta=.4
samr.plot(samr.obj,delta)

delta.table <- samr.compute.delta.table(samr.obj)

siggenes.table<-samr.compute.siggenes.table(samr.obj,delta, data, delta.table)

# sequence data

set.seed(3)
x<-abs(100*matrix(rnorm(1000*20),ncol=20))
x=trunc(x)
y<- c(rep(1,10),rep(2,10))
x[1:50,y==2]=x[1:50,y==2]+50
data=list(x=x,y=y, geneid=as.character(1:nrow(x)),
genenames=paste("g",as.character(1:nrow(x)),sep=""))

samr.obj<-samr(data,  resp.type="Two class unpaired",assay.type="seq",  nperms=100)

delta=5
samr.plot(samr.obj,delta)

delta.table <- samr.compute.delta.table(samr.obj)

siggenes.table<-samr.compute.siggenes.table(samr.obj,delta, data, delta.table)





########### two class paired

# y must take values  -1, 1, -2,2 etc, with (-k,k) being a pair

set.seed(100)
x<-matrix(rnorm(1000*20),ncol=20)
dd<-sample(1:1000,size=100)

u<-matrix(2*rnorm(100),ncol=10,nrow=100)
x[dd,11:20]<-x[dd,11:20]+u
y=c(-(1:10),1:10)


d=list(x=x,y=y, geneid=as.character(1:nrow(x)),
genenames=paste("g",as.character(1:nrow(x)),sep=""), logged2=TRUE)


samr.obj<-samr(d,  resp.type="Two class paired", nperms=100)




#############quantitative response

# y must take numeric values

set.seed(84048)
x=matrix(rnorm(1000*9),ncol=9)

mu=c(3,2,1,0,0,0,1,2,3)
b=runif(100)+.5
x[1:100,]=x[1:100,]+ b%*%t(mu)

y=mu

d=list(x=x,y=y, 
geneid=as.character(1:nrow(x)),genenames=paste("gene", as.character(1:nrow(x))))

samr.obj =samr(d,  resp.type="Quantitative", nperms=50)



########### oneclass
# y is a vector of ones

set.seed(100)
x<-matrix(rnorm(1000*20),ncol=20)
dd<-sample(1:1000,size=100)

u<-matrix(2*rnorm(100),ncol=10,nrow=100)
x[dd,11:20]<-x[dd,11:20]+u

y<-c(rep(1,20))

data=list(x=x,y=y, geneid=as.character(1:nrow(x)),
genenames=paste("g",as.character(1:nrow(x)),sep=""), logged2=TRUE)

samr.obj<-samr(data,  resp.type="One class", nperms=100)



###########survival data
# y is numeric; censoring.status=1 for failures, and 0 for censored

set.seed(84048)
x=matrix(rnorm(1000*50),ncol=50)
x[1:50,26:50]= x[1:50,26:50]+2
x[51:100,26:50]= x[51:100,26:50]-2

y=abs(rnorm(50))
y[26:50]=y[26:50]+2
censoring.status=sample(c(0,1),size=50,replace=TRUE)
d=list(x=x,y=y,censoring.status=censoring.status,
geneid=as.character(1:1000),genenames=paste("gene", as.character(1:1000)))

samr.obj=samr(d,  resp.type="Survival", nperms=20)


################multi-class example
# y takes values 1,2,3,...k where k= number of classes

set.seed(84048)
x=matrix(rnorm(1000*10),ncol=10)
x[1:50,6:10]= x[1:50,6:10]+2
x[51:100,6:10]= x[51:100,6:10]-2

y=c(rep(1,3),rep(2,3),rep(3,4))
d=list(x=x,y=y,geneid=as.character(1:1000),
genenames=paste("gene", as.character(1:1000))) 

samr.obj <- samr(d,  resp.type="Multiclass")



#################### timecourse data

# elements of y are of the form  kTimet  where k is the class label and t
# is the time; in addition, the   suffixes Start or End indicate the first
# and last observation in a given time course
# the class label can be that for a two class unpaired, one class or
# two class paired problem

set.seed(8332)
y=paste(c(rep(1,15),rep(2,15)),"Time",rep(c(1,2,3,4,5,1.1,2.5, 3.7, 4.1,5.5),3),
sep="")
start=c(1,6,11,16,21,26)
for(i in start){
y[i]=paste(y[i],"Start",sep="")
}
for(i in  start+4){
y[i]=paste(y[i],"End",sep="")
}
x=matrix(rnorm(1000*30),ncol=30)
x[1:50,16:20]=x[1:50,16:20]+matrix(3*c(0,1,2,3,4),ncol=5,nrow=50,byrow=TRUE)
x[1:50,21:25]=x[1:50,21:25]+matrix(3*c(0,1,2,3,4),ncol=5,nrow=50,byrow=TRUE)
x[1:50,26:30]=x[1:50,26:30]+matrix(3*c(0,1,2,3,4),ncol=5,nrow=50,byrow=TRUE)

x[51:100,16:20]=x[51:100,16:20]-matrix(3*c(0,1,2,3,4),ncol=5,nrow=50,byrow=TRUE)
x[51:100,21:25]=x[51:100,21:25]-matrix(3*c(0,1,2,3,4),ncol=5,nrow=50,byrow=TRUE)
x[51:100,26:30]=x[51:100,26:30]-matrix(3*c(0,1,2,3,4),ncol=5,nrow=50,byrow=TRUE)

data=list(x=x,y=y, geneid=as.character(1:nrow(x)),
genenames=paste("g",as.character(1:nrow(x)),sep=""), logged2=TRUE)

samr.obj<- samr(data,  resp.type="Two class unpaired timecourse",
 nperms=100, time.summary.type="slope")


##################### pattern discovery
# here there is no outcome y; the desired eigengene is indicated by 
# the argument eigengene.numbe in the data object

set.seed(32)
x=matrix(rnorm(1000*9),ncol=9)
mu=c(3,2,1,0,0,0,1,2,3)
b=3*runif(100)+.5
x[1:100,]=x[1:100,]+ b%*%t(mu)



d=list(x=x,eigengene.number=1,
geneid=as.character(1:nrow(x)),genenames=paste("gene", as.character(1:nrow(x))))


samr.obj=samr(d,  resp.type="Pattern discovery", nperms=50)


}

\keyword{univar}% at least one, from doc/KEYWORDS
\keyword{survival}
\keyword{ts}
\keyword{nonparametric}

\name{samr.assess.samplesize.plot}
\alias{samr.assess.samplesize.plot}
\title{Make a plot of the results from samr.assess.samplesize}
\description{
Plots of the results from samr.assess.samplesize
}
\usage{
samr.assess.samplesize.plot(samr.assess.samplesize.obj, logx=TRUE)
}
\arguments{
\item{samr.assess.samplesize.obj}{ Object returned from call to samr.assess.samplesize}
\item{logx}{Should logs be used on the horizontal (# of genes) axis? Default TRUE} 
}
\references{Tusher, V.,  Tibshirani, R.  and Chu, G. (2001):
Significance analysis of microarrays applied to the ionizing radiation response"  PNAS 2001 98: 5116-5121, (Apr 24).
http://www-stat.stanford.edu/~tibs/sam
}


\details{
Plots results: FDR (or 1-power) and FNR (or 1-type 1 error) from samr.assess.samplesize}



\author{Jun Li and Balasubrimanian Narasimhan and Robert Tibshirani}


\examples{
#generate some example data
set.seed(100)
x<-matrix(rnorm(1000*20),ncol=20)
dd<-sample(1:1000,size=100)

u<-matrix(2*rnorm(100),ncol=10,nrow=100)
x[dd,11:20]<-x[dd,11:20]+u

y<-c(rep(1,10),rep(2,10))

data=list(x=x,y=y, geneid=as.character(1:nrow(x)),
genenames=paste("g",as.character(1:nrow(x)),sep=""), logged2=TRUE)

log2=function(x){log(x)/log(2)}

# run SAM first
samr.obj<-samr(data,  resp.type="Two class unpaired", nperms=100)

# assess current sample size (20), assuming 1.5fold difference on the log base 2 scale

samr.assess.samplesize.obj<- samr.assess.samplesize(samr.obj, data, log2(1.5))

samr.assess.samplesize.plot(samr.assess.samplesize.obj)
}
\keyword{univar}% at least one, from doc/KEYWORDS
\keyword{survival}
\keyword{ts}
\keyword{nonparametric}

\name{samr.assess.samplesize}
\alias{samr.assess.samplesize}
\title{Assess the sample size for a SAM analysis}
\description{
Estimate the false discovery rate, 
false negative rate, power and type I error for a SAM
analysis. Currently
implemented only for two class (unpaired or paired), one-sample and survival problems). 
}
\usage{
samr.assess.samplesize(samr.obj, data, dif, samplesize.factors=c(1,2,3,5), 
min.genes = 10, max.genes = nrow(data$x)/2)
}
\arguments{
\item{samr.obj}{ Object returned from call to samr}
\item{data}{Data list, same as that passed to samr.train} 
\item{dif}{Change in  gene expression between groups 1 and 2, for genes that
are differentially expressed. For log base 2 data, a value of 1 means
a 2-fold change. For One-sample problems, dif is the number of units away from
zero for differentially expressed genes. For survival data, dif is the numerator of the Cox score statistic (this info is provided in the output of samr).}
\item{samplesize.factors}{Integer vector of length 4, indicating the
sample sizes to be examined.
The values are factors that multiply the original sample size.
So the value 1 means a sample size of ncol(data$x), 2 means a sample size of ncol(data$x), etc.}
\item{min.genes}{Minimum number of genes that are assumed to truly changed  in the population}
\item{max.genes}{Maximum number of genes that are assumed to truly changed  in the population}
}
\references{Tusher, V.,  Tibshirani, R.  and Chu, G. (2001):
Significance analysis of microarrays applied to the ionizing radiation response"  PNAS 2001 98: 5116-5121, (Apr 24).
http://www-stat.stanford.edu/~tibs/sam

Taylor, J., Tibshirani, R. and Efron. B. (2005). 
The ``Miss rate'' for the analysis of gene expression data.  Biostatistics 2005 6(1):111-117. 

 A more complete  description
is given in the SAM manual
at http://www-stat.stanford.edu/~tibs/SAM
}

\details{Estimates  false discovery rate,
false negative rate, power and type I error for a SAM
analysis. The argument samplesize.factor allows the use to assess the
effect of  varying the 
 sample size (total number of samples).  A  detailed  description
of this calculation
is given in the SAM manual
at http://www-stat.stanford.edu/~tibs/SAM}


\value{
A list with components
\item{Results}{A matrix with columns: number of genes- both  the number differentially expressed genes in the population and number called significant;
cutpoint- the threshold used for the absolute  SAM score d; FDR, 1-power- the median
false discovery rate, also equal to the power for each gene; FDR-90perc-
 the
upper 90th percentile of the FDR; FNR, Type 1 error- the false negative rate, also equal to 
the type I error for each gene; FNR-90perc- the upper 90th percentile of the FNR }
\item{dif.call}{Change in  gene expression between groups 1 and 2, that was provided in the
call to samr.assess.samplesize}
\item{difm}{The average difference in SAM score d for the genes
differentially expressed vs unexpressed}
\item{samplesize.factor}{The  samplesize.factor that was passed to  samr.assess.samplesiz}
\item{n}{Number of samples in input data (i.e. ncol of x component in data)}
}
\author{Jun Li and Balasubrimanian Narasimhan and Robert Tibshirani}


\examples{
#generate some example data
set.seed(100)
x<-matrix(rnorm(1000*20),ncol=20)
dd<-sample(1:1000,size=100)

u<-matrix(2*rnorm(100),ncol=10,nrow=100)
x[dd,11:20]<-x[dd,11:20]+u

y<-c(rep(1,10),rep(2,10))

data=list(x=x,y=y, geneid=as.character(1:nrow(x)),
genenames=paste("g",as.character(1:nrow(x)),sep=""), logged2=TRUE)

log2=function(x){log(x)/log(2)}

# run SAM first
samr.obj<-samr(data,  resp.type="Two class unpaired", nperms=100)

# assess current sample size (20), assuming 1.5fold difference on log base 2 scale

samr.assess.samplesize.obj<- samr.assess.samplesize(samr.obj, data, log2(1.5))

# assess the effect of doubling the sample size

samr.assess.samplesize.obj2<- samr.assess.samplesize(samr.obj, data, log2(1.5))

}
\keyword{univar}% at least one, from doc/KEYWORDS
\keyword{survival}
\keyword{ts}
\keyword{nonparametric}

\name{samr-internal}
\title{Internal samr functions}
\alias{ttest.func}
\alias{wilcoxon.func}
\alias{onesample.ttest.func}
\alias{paired.ttest.func}
\alias{cox.func}
\alias{multiclass.func}
\alias{quantitative.func}
\alias{patterndiscovery.func}
\alias{timearea.func}
\alias{sumlengths}
\alias{detec.slab}
\alias{detec.horiz}
\alias{localfdr}
\alias{predictlocalfdr}
\alias{qvalue.func}
\alias{foldchange.twoclass}
\alias{foldchange.paired}
\alias{est.s0}
\alias{permute}
\alias{permute.rows}
\alias{sample.perms}
\alias{insert.value}
\alias{integer.base.b}
\alias{varr}
\alias{mysvd}
\alias{getperms}
\alias{compute.block.perms}
\alias{parse.block.labels.for.2classes}
\alias{parse.time.labels.and.summarize.data}
\alias{check.format}
\alias{samr.const.quantitative.response}
\alias{samr.const.twoclass.unpaired.response}
\alias{samr.const.survival.response}
\alias{samr.const.multiclass.response}
\alias{samr.const.oneclass.response}
\alias{samr.const.twoclass.paired.response}
\alias{samr.const.twoclass.unpaired.timecourse.response}
\alias{samr.const.twoclass.paired.timecourse.response}
\alias{samr.const.oneclass.timecourse.response}
\alias{samr.const.patterndiscovery.response}
\alias{resample}
\alias{wilcoxon.unpaired.seq.func}
\alias{wilcoxon.paired.seq.func}
\alias{multiclass.seq.func}
\alias{quantitative.seq.func}
\alias{cox.seq.func}
\alias{rankcol}
\alias{samr.compute.delta.table.seq}
\alias{samr.seq.null.err}
\alias{samr.seq.detec.slabs}
\alias{generate.dels}
\alias{foldchange.seq.twoclass.paired}
\alias{foldchange.seq.twoclass.unpaired}
\alias{samr.compute.delta.table.array}
\alias{plot.SAMoutput}
\alias{print.SAMoutput}
\description{Internal samr functions}
\usage{
ttest.func(x, y, s0 = 0, sd=NULL)
wilcoxon.func(x, y, s0 = 0)
onesample.ttest.func(x, y, s0 = 0, sd=NULL)
paired.ttest.func(x, y, s0 = 0, sd=NULL)
cox.func(x, y, censoring.status, s0 = 0)
multiclass.func(x, y, s0 = 0)
quantitative.func(x, y, s0 = 0)
patterndiscovery.func(x,s0=0, eigengene.number=1)
timearea.func(x, y, s0 = 0)
sumlengths(aa)
detec.slab(samr.obj, del, min.foldchange)
detec.horiz(samr.obj, cutlow, cutup, min.foldchange)
localfdr(samr.obj, min.foldchange, perc = 0.01, df = 10)
predictlocalfdr(smooth.object, d)
qvalue.func(samr.obj, sig, delta.table)
foldchange.twoclass(x, y, logged2)
foldchange.paired(x, y, logged2)
est.s0(tt, sd, s0.perc = seq(0, 1, by = 0.05))
permute(elem)
sample.perms(elem, nperms)
permute.rows(x)
insert.value(vec, newval, pos)
integer.base.b(x, b = 2)
varr(x, meanx = NULL)
mysvd(x, n.components=NULL)
getperms(y, nperms)
compute.block.perms(y, blocky, nperms)
parse.block.labels.for.2classes(y)
parse.time.labels.and.summarize.data(x, y, resp.type,time.summary.type)

check.format(y, resp.type, censoring.status = NULL)
samr.const.quantitative.response
samr.const.twoclass.unpaired.response
samr.const.survival.response
samr.const.multiclass.response
samr.const.oneclass.response
samr.const.twoclass.paired.response
samr.const.twoclass.unpaired.timecourse.response
samr.const.twoclass.paired.timecourse.response
samr.const.oneclass.timecourse.response
samr.const.patterndiscovery.response
resample(x, d, nresamp=20)
wilcoxon.unpaired.seq.func(xresamp, y)
wilcoxon.paired.seq.func(xresamp, y)
multiclass.seq.func(xresamp, y)
quantitative.seq.func(xresamp, y)
cox.seq.func(xresamp, y, censoring.status)
rankcol(x)
samr.compute.delta.table.seq(samr.obj, min.foldchange=0, dels=NULL)
samr.seq.null.err(samr.obj, min.foldchange, cutup, cutlow)
samr.seq.detec.slabs(samr.obj, dels, min.foldchange)
generate.dels(samr.obj, min.foldchange=0)
foldchange.seq.twoclass.paired(x, y, depth)
foldchange.seq.twoclass.unpaired(x, y, depth)
samr.compute.delta.table.array(samr.obj, min.foldchange=0, dels=NULL, nvals=50)
\method{plot}{SAMoutput}(x, ...)
\method{print}{SAMoutput}(x, ...)
}
\author{Jun Li and Balasubrimanian Narasimhan and Robert Tibshirani}

\details{
  These are not to be called by the user.
}
\keyword{internal}

\name{samr.compute.siggenes.table}
\alias{samr.compute.siggenes.table}
\title{Compute significant genes table}
\description{
 Computes significant genes table, starting with samr object "samr.obj" and  delta.table "delta.table"
}
\usage{
samr.compute.siggenes.table(samr.obj, del, data, delta.table, 
min.foldchange=0, all.genes=FALSE, compute.localfdr=FALSE)
}
\arguments{
\item{samr.obj}{ Object returned from call to samr}
\item{del}{Value of delta to define cutoff rule}
\item{data}{Data object, same as that used in call to samr}
\item{delta.table}{ Object returned from call to
  samr.compute.delta.table}
\item{min.foldchange}{The minimum fold change desired; should be >1;
default is zero, meaning no fold change criterion is applied}
\item{all.genes}{Should all genes be listed? Default FALSE}
\item{compute.localfdr}{Should the local fdrs be computed (this can take some time)? Default FALSE}
}
\value{
 return(list(genes.up=res.up,
genes.lo=res.lo, 
color.ind.for.multi=color.ind.for.multi, 
ngenes.up=ngenes.up, 
ngenes.lo=ngenes.lo))

\item{genes.up}{Matrix of significant genes having posative correlation with the outcome.
For survival data, genes.up are those genes having positive correlation with risk-
 that is, increased expression corresponds to higher risk (shorter survival).}
\item{genes.lo}{Matrix of significant genes having negative correlation with the outcome.
For survival data,genes. lo are those whose increased expression corresponds to lower risk (longer survival).}
\item{color.ind.for.multi}{For multiclass response: a matrix with entries +1
if the class mean is larger  than the overall mean at the 95% significance
levels, -1 if less, and zero otehrwise. This is useful in 
determining which class or classes causes a feature to be significant}
\item{ngenes.up}{Number of significant genes with  positive correlation}
\item{ngenes.lo}{Number of significant genes with  negative correlation}
}
\references{Tusher, V.,  Tibshirani, R.  and Chu, G. (2001):
Significance analysis of microarrays applied to the ionizing radiation response"  PNAS 2001 98: 5116-5121, (Apr 24).
http://www-stat.stanford.edu/~tibs/sam}

\author{ Balasubrimanian Narasimhan and Robert Tibshirani}


\examples{
#generate some example data
set.seed(100)
x<-matrix(rnorm(1000*20),ncol=20)
dd<-sample(1:1000,size=100)

u<-matrix(2*rnorm(100),ncol=10,nrow=100)
x[dd,11:20]<-x[dd,11:20]+u

y<-c(rep(1,10),rep(2,10))

data=list(x=x,y=y, geneid=as.character(1:nrow(x)),
genenames=paste("g",as.character(1:nrow(x)),sep=""), logged2=TRUE)


samr.obj<-samr(data,  resp.type="Two class unpaired", nperms=100)

delta.table<-samr.compute.delta.table(samr.obj)
del<- 0.3
siggenes.table<- samr.compute.siggenes.table(samr.obj, del, data, delta.table)


}
\keyword{univar}% at least one, from doc/KEYWORDS
\keyword{survival}
\keyword{ts}
\keyword{nonparametric}
